"use client"

import React, { createContext, useContext, useState, useEffect, ReactNode, useCallback } from "react";
import { ILive, ILiveStatistic, ILiveStatisticsSummary, LiveStatus } from "@/shared/types/live";
import { getLiveStatistics, getRealTimeStats } from "@/shared/api/live";
import usePolling from "@/hooks/use-polling";

type LiveDetailsContextType = {
    stats: ILiveStatisticsSummary;
    liveData: ILive | null;
}

const LiveDetailsContext = createContext<LiveDetailsContextType>({ liveData: null, stats: { statistics: [], totalMessages: 0, totalViewers: 0, totalPurchases: 0 } });

interface LiveStatisticsProviderProps {
    liveData: ILive;
    children: ReactNode;
}

const LiveDetailsProvider: React.FC<LiveStatisticsProviderProps> = ({ children, liveData }) => {
    const { id, status } = liveData;
    const [statSummary, setStatsSummary] = useState<ILiveStatisticsSummary>({ statistics: [], totalMessages: 0, totalViewers: 0, totalPurchases: 0 });


    useEffect(() => {
        const fetchReplayStatistics = async () => {
            try {
                const statsData = await getLiveStatistics(id);
                setStatsSummary(statsData);
            } catch (err) {
                console.error("Error fetching replay statistics", err);
            }
        };

        if (status === LiveStatus.REPLAY) {
            fetchReplayStatistics();
        }
    }, [id, status]);

    const fetchRealTimeStats = useCallback(() => getRealTimeStats(id), [id]);

    const [liveStats] = usePolling<ILiveStatistic>(
        10000,
        fetchRealTimeStats,
        status === LiveStatus.ONGOING
    );

    useEffect(() => {
        if (status === LiveStatus.ONGOING && liveStats) {
            console.log("Live stats", liveStats);
            setStatsSummary((prev) => ({
                totalMessages: prev.totalMessages + liveStats.numberOfMessages,
                totalPurchases: prev.totalPurchases + liveStats.purchases,
                totalViewers: liveStats.numberOfViewers,
                statistics: [...prev.statistics, liveStats].slice(-10),
            }));
        }
    }, [liveStats, status]);

    return (
        <LiveDetailsContext.Provider value={{ stats: statSummary, liveData }}>
            {children}
        </LiveDetailsContext.Provider>
    );
};

const useLiveStatistics = (): ILiveStatisticsSummary => {
    const { stats } = useContext(LiveDetailsContext);
    return stats;
}

const useGetLiveNumberOfviewers = (): number => {
    const { stats } = useContext(LiveDetailsContext);
    return stats.statistics[stats.statistics.length - 1]?.numberOfViewers || 0;
}

const useLiveData = (): ILive => {
    const { liveData } = useContext(LiveDetailsContext);
    return liveData as ILive;
}

export { LiveDetailsProvider, useLiveData, useLiveStatistics ,useGetLiveNumberOfviewers}
