import { ShippingMethodType, IShippingAreaItem, ShippingMethod, RequiresFreeShipping, CalculationType, MondialRelayMode, ShippingClass, IColisColissimoItem, IHistoricBordereauColissimoItem, IOrderMondialRelayItem, IOrderMondialRelayStatus, MethodMondialRelayType, IMethodMondialRelayItem } from '@/contexts/types/Shipping';
import { _mock } from './_mock';
import { IOrderColissimoItem, IOrderColissimoStatus } from '../types/shipping';
import { countries } from '../assets/data';


const methodDescriptions: { [key in ShippingMethodType]: string } = {
    [ShippingMethodType.FreeShipping]: 'Livraison gratuite pour vos achats',
    [ShippingMethodType.FlatRate]: 'Frais de livraison fixes',
    [ShippingMethodType.MondialRelay]: 'Livraison via Mondial Relay',
    [ShippingMethodType.ColissimoWithSignature]: 'Livraison Colissimo avec signature',
    [ShippingMethodType.ColissimoWithoutSignature]: 'Livraison Colissimo sans signature',
    [ShippingMethodType.PointOfSale]: 'Retrait depuis un point de vente',
};

const countriesList = ['France', 'Germany', 'Spain', 'Italy', 'Europe', 'Paris'];


const generateShippingMethods = (): ShippingMethod[] => {
    return Object.values(ShippingMethodType).map((type: ShippingMethodType, index) => {
        let methodData: any = {};

        switch (type) {
            case ShippingMethodType.FreeShipping:
                methodData = {
                    requires: RequiresFreeShipping.PROMO_CODE_OR_MINIMUM_ORDER_AMOUNT,
                    minimumOrderAmount: 50,
                    applyMinOrderRuleBeforePromoCodeDiscount: true,
                };
                break;
            case ShippingMethodType.FlatRate:
                methodData = {
                    tvaStatut: true,
                    cost: 5,
                    costHeavyShippingClass: 10,
                    CostNoShippingClass: 3,
                    calculationType: CalculationType.BY_ORDER,
                };
                break;
            case ShippingMethodType.MondialRelay:
                methodData = {
                    descriptionAdditionnelle: methodDescriptions[type],
                    amountFreeShipping: 60,
                    methodMondialRelay: MondialRelayMode.MED,
                    listCostPerWeight: [
                        { weightCart: 0, weightPackaging: 0, cost: 4.5 },
                        { weightCart: 1, weightPackaging: 0.2, cost: 5 },
                        { weightCart: 2, weightPackaging: 0.5, cost: 5.5 },
                    ]
                };
                break;
            case ShippingMethodType.PointOfSale:
                methodData = {
                    tvaStatut: true,
                    cost: 5,
                };
                break;
            default:
                break;
        }

        return {
            id: _mock.id(index),
            name: type,
            active: _mock.boolean(index),
            description: methodDescriptions[type] || `Livraison pour ${type}`,
            type: type,
            data: methodData
        };
    });
};

//------------------------------------------
const generateShippingAreas = (): IShippingAreaItem[] => {
    const zones: IShippingAreaItem[] = [];
    const zoneNames = ["Zone 1", "Zone 2", "Zone 3", "Zone 4", "Zone 5", "Zone 6"];

    for (let i = 0; i < 6; i++) {
        const numCountries = Math.floor(Math.random() * 2) + 2;
        const selectedCountries: string[] = [];
        const selectedMethods: ShippingMethod[] = [];
        const selectedMethodTypes: Set<ShippingMethodType> = new Set<ShippingMethodType>();

        while (selectedCountries.length < numCountries) {
            const randomCountryIndex = Math.floor(Math.random() * countries.length);
            const randomCountry = countries[randomCountryIndex].label;
            if (!selectedCountries.includes(randomCountry)) {
                selectedCountries.push(randomCountry);
            }
        }

        for (let j = 0; j < numCountries; j++) {
            let randomMethodIndex = Math.floor(Math.random() * shippingMethodsList.length);
            let randomMethod = shippingMethodsList[randomMethodIndex];

            while (selectedMethodTypes.has(randomMethod.type)) {
                randomMethodIndex = Math.floor(Math.random() * shippingMethodsList.length);
                randomMethod = shippingMethodsList[randomMethodIndex];
            }

            selectedMethods.push(randomMethod);
            selectedMethodTypes.add(randomMethod.type);
        }

        zones.push({
            id: _mock.id(i),
            name: zoneNames[i],
            region: selectedCountries.map(country => `${country}`),
            shippingMethodList: selectedMethods,
        });
    }

    return zones;
};

//----------------------------------------------
const generateShippingClasses = (): ShippingClass[] => {
    const shippingClasses: ShippingClass[] = [
        { id: '1', name: 'Standard', code: 'STD', description: 'Livraison standard', productAssigned: 0 },
        { id: '2', name: 'Express', code: 'EXP', description: 'Livraison express', productAssigned: 0 },
        { id: '3', name: 'Économique', code: 'ECO', description: 'Livraison économique', productAssigned: 0 },
        { id: '4', name: 'Livraison le lendemain', code: 'NEXT', description: 'Livraison le lendemain', productAssigned: 0 },
        { id: '5', name: 'Livraison en 2 jours', code: '2DAY', description: 'Livraison en 2 jours', productAssigned: 0 },
        { id: '6', name: 'Livraison le même jour', code: 'SAM', description: 'Livraison le même jour', productAssigned: 0 },
        { id: '7', name: 'Livraison de nuit', code: 'OVR', description: 'Livraison de nuit', productAssigned: 0 },
    ];

    return shippingClasses;
};

export const shippingClassesList: ShippingClass[] = generateShippingClasses();
export const shippingMethodsList: ShippingMethod[] = generateShippingMethods();
export const shippingAreaList: IShippingAreaItem[] = generateShippingAreas();


//----------------------------------------------------------------------------------------------------------------

const customerNames: string[] = ['John Doe', 'Jane Smith', 'Alice Johnson', 'Bob Brown', 'Emma Davis', 'Michael Wilson', 'Sophia Martinez', 'William Anderson', 'Olivia Taylor', 'James Thomas', 'Emily Jackson', 'Benjamin White', 'Mia Harris', 'Alexander Martin', 'Charlotte Thompson', 'Daniel Garcia', 'Samantha Martinez', 'Joseph Lee', 'Victoria Perez', 'David Allen'];

const generateAddress = (index: number): string => {
    const country = ['France', 'Germany', 'Spain', 'Italy', 'Europe', 'Paris'][index % 6]; //
    return `${index + 1} St, ${country}`;
};


const orderColissimoList: IOrderColissimoItem[] = [];

for (let i = 0; i < 20; i++) {
    const customerName = customerNames[i % customerNames.length];
    const address = generateAddress(i);
    const shippingMethod = i % 2 === 0 ? ShippingMethodType.ColissimoWithSignature : ShippingMethodType.ColissimoWithoutSignature;
    const status = Object.values(IOrderColissimoStatus)[i % Object.keys(IOrderColissimoStatus).length] as IOrderColissimoStatus;

    const order: IOrderColissimoItem = {
        id: (i + 1).toString(),
        createdAt: new Date(),
        customer: customerName,
        adress: address,
        country: address.split(',').slice(-1)[0].trim(),
        shippingMethod: shippingMethod,
        status: status,
        etiquette: `Etiquette ${i + 1}`,
        startDate: new Date(),
        endDate: new Date(),
    };

    orderColissimoList.push(order);
}

export default orderColissimoList;

//------------------------------------------------

function getRandomDate(start: Date, end: Date): Date {
    return new Date(start.getTime() + Math.random() * (end.getTime() - start.getTime()));
}


function generateMockItem(): IColisColissimoItem {
    const id = Math.random().toString(36).substring(7);
    const trackingNumber = Math.random().toString(36).substring(7);
    const etiquetteCreatedAt = getRandomDate(new Date(2020, 0, 1), new Date());
    const orderCreatedAt = getRandomDate(new Date(2020, 0, 1), new Date());

    const countriesList = ['France', 'Espagne', 'Belgique', 'Italie'];
    const randomCountry = countriesList[Math.floor(Math.random() * countriesList.length)];

    const shippingMethods = Object.values(ShippingMethodType);
    const randomShippingMethod = shippingMethods[Math.floor(Math.random() * shippingMethods.length)];

    return {
        id,
        trackingNumber,
        etiquetteCreatedAt,
        orderCreatedAt,
        country: randomCountry,
        shippingMethod: randomShippingMethod,
    };
}

export const colisColissimoList: IColisColissimoItem[] = Array.from({ length: 20 }, () => generateMockItem());

//-----------------------------------------------------------

const generateTrackingNumber = (index: number): string => {
    return `${index + 100}`;
};

const historic: IHistoricBordereauColissimoItem[] = [];

for (let i = 0; i < 20; i++) {
    const orderIds = [(i + 12).toString()];
    const trackingNumber = generateTrackingNumber(i);

    const historicItem: IHistoricBordereauColissimoItem = {
        id: (i + 1).toString(),
        colisNumber: i + 1000,
        orderIds: orderIds,
        trackingNumber: trackingNumber,
        createdAt: new Date(),
    };

    historic.push(historicItem);
  }
  
  export const historicBordereauColissimoList = historic;
  


  //-------------------------------------------------------

const mondialRelayTypes: MethodMondialRelayType[] = Object.values(MethodMondialRelayType);

const generateRandomMondialRelayType = (): MethodMondialRelayType => {
    return mondialRelayTypes[Math.floor(Math.random() * mondialRelayTypes.length)];
};

const shippingModes: MondialRelayMode[] = Object.values(MondialRelayMode);

shippingModes.sort(() => Math.random() - 0.5);

const methodMondialRelayItemMock: IMethodMondialRelayItem[] = [];

for (let i = 0; i < 15; i++) {
    const methodMondialRelayItem: IMethodMondialRelayItem = {
        id: `${i + 1}`,
        name: `Mondial Relay ${i + 1}`,
        type: generateRandomMondialRelayType(),
        shippingArea: shippingAreaList[i % shippingAreaList.length],
        shippingMode: shippingModes[i % shippingModes.length]
    };
    methodMondialRelayItemMock.push(methodMondialRelayItem);
}

export const methodMondialRelayList: IMethodMondialRelayItem[] = methodMondialRelayItemMock;
  

//----------------------------------------------------------

const expeditionNames: string[] = [
    'John Doe', 'Jane Smith', 'Alice Johnson', 'Bob Brown', 'Emma Davis',
    'Michael Wilson', 'Sophia Martinez', 'William Anderson', 'Olivia Taylor',
    'James Thomas', 'Emily Jackson', 'Benjamin White', 'Mia Harris',
    'Alexander Martin', 'Charlotte Thompson', 'Daniel Garcia', 'Samantha Martinez',
    'Joseph Lee', 'Victoria Perez', 'David Allen'
];

const generateAddressOrderMondialRelay = (index: number): string => {
    const countriesList = ['France', 'Germany', 'Spain', 'Italy', 'Europe', 'Paris'];
    const country = countriesList[index % countriesList.length];
    return `${index + 1} St, ${country}`;
};

const generateMockMondialRelayData = (): IOrderMondialRelayItem[] => {
    const orders: IOrderMondialRelayItem[] = [];
    const shippingModes: MondialRelayMode[] = [
        MondialRelayMode.MED, MondialRelayMode.APM, MondialRelayMode.HOM,
        MondialRelayMode.PRL, MondialRelayMode.HOC, MondialRelayMode.HOD,
        MondialRelayMode.OMR
    ];

    const deliveryStatus: string[] = ['Colis enregistré', 'Expédition créée'];

    for (let i = 0; i < 20; i++) {
        const expeditionName = expeditionNames[i % expeditionNames.length];
        const address = generateAddressOrderMondialRelay(i);
        const shippingMode = shippingModes[i % shippingModes.length];
        const statusValues: IOrderMondialRelayStatus[] = [
            IOrderMondialRelayStatus.WaitingForPayment,
            IOrderMondialRelayStatus.InProgress,
            IOrderMondialRelayStatus.Pending
        ];
        const status = statusValues[i % statusValues.length];

        const randomDeliveryStatus = deliveryStatus[Math.floor(Math.random() * deliveryStatus.length)];

        const order: IOrderMondialRelayItem = {
            id: (i + 1).toString(),
            status,
            shippingMode,
            dateCreation: new Date(),
            expedition: expeditionName,
            address,
            weight: parseFloat((Math.random() * 10).toFixed(2)),
            expeditionNumber: `N°${i + 1}`,
            delivery: randomDeliveryStatus
        };

        orders.push(order);
    }

    return orders;
};

export const orderMondialRelayList: IOrderMondialRelayItem[] = generateMockMondialRelayData();


