import { AllLivesAnalytics, ILive, ILiveComment, ILiveItem, ILiveProduct, ILiveStatistic, ILiveStatisticsSummary, LiveStatus } from "@/shared/types/live";
import { _lives, generateStatisticsForLives } from "../_mock";
import useSWR, { mutate } from 'swr';
import { useMemo } from 'react';
import axiosInstance, { fetcher, endpoints } from './server';

const options = {
    revalidateIfStale: false,
    revalidateOnFocus: false,
    revalidateOnReconnect: false,
};


/////////////////////////////////////Live API/////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////


export async function addLive(data: any) {
    const formData = {
        title: data.title,
        chatName: data.chatName,
        description: data.description,
        startDate: data.startDate,
        keyWord: data.keyWord,
        staging: data.staging.toString(),
        enableChat: data.enableChat.toString(),
        enableLive: data.enableLive.toString(),
        videoLink: "dede",
        image: "https://dailystar.com.au/wp-content/uploads/2015/12/Mode-presents-100-years-Of-Mens-New-Years-Eve-Fashion.png",
        videoFormat: data.videoFormat,
        products: data.products,
    }

    const response = await axiosInstance.post('/live', formData);
    const newData: ILiveItem = response.data;
    mutate([endpoints.live.stream], (lives: ILiveItem[] = []) => [...lives, newData], true);
    return newData;
}

export async function editLive(liveId: string, data: any) {
    try {
        const response = await axiosInstance.put(`${endpoints.live.stream}/${liveId}`, {...data, status:LiveStatus.COMING});
        const newData = await response.data;
        mutate([endpoints.live.stream, { params: [] }], (lives: ILiveItem[] = []) => {
            const index = lives.findIndex((live) => live.id === liveId);
            if (index > -1) {
                lives[index] = newData;
            }
            return [...lives];
        }, false);
        return newData;
    } catch (error) {
        console.error('Error editing live:', error);
        throw error;
    }
}

export function useGetLives() {
    const URL = [endpoints.live.stream];

    const { data, isLoading, error, isValidating } = useSWR<ILiveItem[]>(URL, fetcher, options);

    const memoizedValue = useMemo(
        () => ({
            livesData: (data as ILiveItem[]) || [],
            livesLoading: isLoading,
            livesError: error,
            livesValidating: isValidating,
        }),
        [data, error, isLoading, isValidating]
    );
    return memoizedValue;
}


export function useGetLive(liveId: string) {
    const URL = [`${endpoints.live.stream}/${liveId}`]
    const { data, isLoading, error } = useSWR(URL, fetcher, options);
    return { liveData: data as ILive, liveIsLoading: isLoading, liveError: error };
}

/////////////////////////////////Live Statistics API//////////////////////////////
//////////////////////////////////////////////////////////////////////////////////

export async function getRealTimeStats(id: string): Promise<ILiveStatistic> {
    const response = await axiosInstance.get<ILiveStatistic>(endpoints.live.stats.realTime(id));
    return response.data ;
}

export async function getLiveStatistics(liveId: string) {
    const response = await axiosInstance.get<ILiveStatisticsSummary>(endpoints.live.stats.summary(liveId));
    return response.data;
}


export async function getLiveStatisticsForAllLives() {
    const response = await axiosInstance.get<AllLivesAnalytics>(endpoints.live.stats.summaryAll);
    return response.data;
}



export function useGetLivesStatistic(startDate: Date, endDate: Date, lives: string[]) {
    const memoizedValue = useMemo(
        () => (
            {
                statsLoading: false,
                statsError: false,
                statsValidating: false,
                statsData: generateStatisticsForLives(2)
            }
        ),
        [startDate, endDate, lives]
    );
    return memoizedValue;
}




/////////////////////////////////Live Comment API/wwwwwwww///////////////////////////////
//////////////////////////////////////////////////////////////////////////////////


export async function addCommentLive(commentData: ILiveComment) {
    const response = await axiosInstance.post(endpoints.comments.add, commentData);
    return response.data as ILiveComment;

}


export function useGetCommentsLive(liveId: string) {
    const URL = [endpoints.comments.get(liveId)];

    const { data, isLoading, error, isValidating } = useSWR(URL, fetcher, options);

    const memoizedValue = useMemo(
        () => ({
            commentsData: (data as ILiveComment[]) || [],
            commentsLoading: isLoading,
            commentsError: error,
            commentsValidating: isValidating,
        }),
        [data, error, isLoading, isValidating]
    );
    return memoizedValue;
}



export async function deleteCommentLive(commentId: string) {

}

/////////////////////////////////Live Product API////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////


export async function addProductLive(liveId: string, productId: string, startTime: number, endTime: number) {
    await axiosInstance.post(`${endpoints.live.products}/${productId}`, {
        startTime,
        endTime
    });

    mutate(
        [endpoints.live.stream + '/' + liveId + '/products'],
        (products: ILiveProduct[] = []) => {
            const index = products.findIndex((product) => product.id === productId);
            if (index > -1) {
                const updatedProduct = {
                    ...products[index],
                    startTime: startTime,
                    endTime: endTime,
                    enabled: true,
                };
                products[index] = updatedProduct;
            }
            return [...products];
        },
        false
    );
}

export function useGetProducts() {
    const URL = [`${endpoints.product}?size=100&page=1`];

    const { data, isLoading } = useSWR(URL, fetcher, options);


    return { productsData: data as ILiveProduct[], productsLoading: isLoading };
}

export async function deleteProductLive(productId: string, liveId: string) {
    await axiosInstance.delete(`${endpoints.live.products}/${productId}`);
    mutate([endpoints.live.stream + '/' + liveId + '/products'], (products: ILiveProduct[] = []) => {
        const index = products.findIndex((product) => product.id === productId);
        if (index > -1) {
            products.splice(index, 1);
        }
        return [...products];
    }, true);
}


export function getProductsLive(liveId: string) {
    const URL = [endpoints.live.stream + '/' + liveId + '/products'];

    const { data, isLoading, error, isValidating } = useSWR(URL, fetcher, options);

    const memoizedValue = useMemo(
        () => ({
            products: (data as ILiveProduct[]) || [],
            productsLoading: isLoading,
            productsError: error,
            productsValidating: isValidating,
        }),
        [data, error, isLoading, isValidating]
    );
    return memoizedValue;


}



/////////////////////////////////Live State API////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

export function stopLive(liveId: string) {
    const URL = `${endpoints.live.stream}/${liveId}/stop`;
    axiosInstance.post(URL);
    mutate([`${endpoints.live.stream}/${liveId}`], (live: ILive | null = null) => {
        if (live) {
            live.status = LiveStatus.REPLAY
        }
        return live;
    }, true);

    mutate([endpoints.live.stream], (lives: ILiveItem[] = []) => {
        const index = lives.findIndex((live) => live.id === liveId);
        if (index > -1) {
            lives[index].status = LiveStatus.REPLAY;
        }
        return [...lives];
    }, true);
}

export function startLive(liveId: string) {
    const URL = `${endpoints.live.stream}/${liveId}/start`;
    axiosInstance.post(URL);
    mutate([`${endpoints.live.stream}/${liveId}`], (live: ILive | null = null) => {
        if (live) {
            live.status = LiveStatus.ONGOING
            live.startedAt = new Date();
        }
        return live;
    }, true);

    mutate([endpoints.live.stream], (lives: ILiveItem[] = []) => {
        const index = lives.findIndex((live) => live.id === liveId);
        if (index > -1) {
            lives[index].status = LiveStatus.ONGOING;
        }
        return [...lives];
    }, true);
}