import React, { useCallback, useMemo, useState } from "react";
import {
  Card,
  CardContent,
  Typography,
  Table,
  TableBody,
  TableCell,
  TableContainer,
  TableRow,
  Avatar,
  ListItemText,
  Box,
  LinearProgress,
  Grid,
  Stack,
  Divider,
  Button,
  Dialog,
  DialogTitle,
  DialogContent,
  TextField,
  DialogActions,
  Chip,
  InputAdornment, // Import de Chip depuis Material-UI
} from "@mui/material";
import CloseIcon from "@mui/icons-material/Close";
import MinimizeIcon from "@mui/icons-material/Minimize";

import {
  IProduct,
  Variation,
  VariationTableFilterValue,
  VariationTableFilters,
  _products,
} from "@/shared/_mock/_cartLive";
import VariationTableToolbar from "./Variation-table-toolbar";
import { _mock } from "@/shared/_mock";
import {
  TableHeadCustom,
  TableSelectedAction,
  getComparator,
  useTable,
} from "@/shared/components/table";
import isEqual from "lodash/isEqual";
import VariationTableFiltersResult from "./Variation-table-filters-result";

type Props = {
  product: IProduct;
};
const defaultFilters: VariationTableFilters = {
  name: "",
  color: [],
  size: [],
};

//----------------------------------------------------------------------------------
const TABLE_HEAD = [
  { id: "Produit", label: "Produit", width: 250 },
  { id: "id", label: "ID", width: 80 },
  { id: "ugs", label: "UGS", width: 80 },
  { id: "couleur", label: "Couleur", width: 80 },
  { id: "taille", label: "Taille US", width: 80 },
  { id: "Qte", label: "Quantité", width: 80 },
  { id: "prix", label: "Prix", width: 80 },
];
//----------------------------------------------------------------------------------
export default function ProductVariationsCard({ product }: Props) {
  const [open, setOpen] = useState(false);
  const [newPrice, setNewPrice] = useState("");
  const [defaultQuantity, setDefaultQuantity] = useState("1");
  const [selectedRow, setSelectedRow] = useState<string | null>(null);

  const handleRowClick = (id: string) => {
    setSelectedRow(id === selectedRow ? null : id);
  };

  const handleQuantityChange = (e: any) => {
    setDefaultQuantity(e.target.value);
  };

  const handleIncreaseQuantity = () => {
    setDefaultQuantity((prevQuantity) => String(parseInt(prevQuantity) + 1));
  };

  const handleDecreaseQuantity = () => {
    if (parseInt(defaultQuantity) > 1) {
      setDefaultQuantity((prevQuantity) => String(parseInt(prevQuantity) - 1));
    }
  };

  const handleOpen = () => {
    setOpen(true);
  };

  const handleClose = () => {
    setOpen(false);
  };

  const handlePriceChange = (e: any) => {
    setNewPrice(e.target.value);
  };

  const handleUpdatePrice = () => {
    handleClose();
  };
  const [filters, setFilters] = useState(defaultFilters);
  const table = useTable();
  const handleFilters = useCallback(
    (name: string, value: VariationTableFilterValue) => {
      table.onResetPage();
      setFilters((prevState) => ({
        ...prevState,
        [name]: value,
      }));
    },
    [table]
  );

  const handleResetFilters = useCallback(() => {
    setFilters(defaultFilters);
  }, []);
  const canReset = !isEqual(defaultFilters, filters);
  const [tableData, setTableData] = useState<Variation[]>(
    _products.flatMap((product) => product.variations)
  );
  const memoizedApplyFilter = useMemo(() => applyFilter, []);

  const dataFiltered = memoizedApplyFilter({
    inputData: tableData,
    comparator: getComparator(table.order, table.orderBy),
    filters,
  });
  return (
    <Grid xs={12}>
      <Card>
        <Stack
          direction="column"
          divider={<Divider orientation="horizontal" />}
          spacing={2}
          sx={{ ml: 2 }}
        >
          <Box
            sx={{
              display: "flex",
              flexDirection: "row",
            }}
          >
            <Typography variant="subtitle1" gutterBottom mt={2}>
              Produit sélectionné
            </Typography>
            <Box
              sx={{
                display: "flex",
                alignItems: "center",
                flexDirection: "row",
                justifyContent: "flex-end",
                flex: 1,
              }}
            >
              <CloseIcon fontSize="small" sx={{ cursor: "pointer" }} />
              <MinimizeIcon
                fontSize="small"
                sx={{ cursor: "pointer", ml: 1 }}
              />
            </Box>
          </Box>

          <Box
            sx={{
              display: "flex",
              flexDirection: "row",
            }}
          >
            <Avatar
              alt={product.title}
              src={product.image}
              variant="rounded"
              sx={{ width: 64, height: 64, mr: 2 }}
            />
            <ListItemText
              disableTypography
              primary={
                <Box
                  component="div"
                  sx={{ typography: "body2", color: "text.disabled" }}
                >
                  {product.title}
                </Box>
              }
              sx={{ display: "flex", flexDirection: "column" }}
            />
            <Box
              sx={{
                display: "flex",
                alignItems: "center",
                flexDirection: "column",
                justifyContent: "flex-end",
              }}
            >
              <Typography variant="body1" gutterBottom mr={2}>
                {product.price} €
              </Typography>

              <Button variant="outlined" onClick={handleOpen} sx={{ mr: 2 }}>
                Modifier le prix
              </Button>
            </Box>
          </Box>
        </Stack>
        <VariationTableToolbar
          filters={filters}
          onFilters={handleFilters}
          //
          ColorOptions={_mock.color}
          SizeOptions={_mock.size}
        />

        {canReset && (
          <VariationTableFiltersResult
            filters={filters}
            onFilters={handleFilters}
            //
            onResetFilters={handleResetFilters}
            //
            results={dataFiltered.length}
            sx={{ p: 2.5, pt: 0 }}
          />
        )}

        <CardContent>
          <TableContainer sx={{ ml: 2, mr: 3 }}>
            <TableSelectedAction
              dense={table.dense}
              numSelected={table.selected.length}
              rowCount={dataFiltered.length}
              onSelectAllRows={(checked) =>
                table.onSelectAllRows(
                  checked,
                  dataFiltered.map((row) => row.id)
                )
              }
            />
            <Table
              size={table.dense ? "small" : "medium"}
              sx={{ minWidth: 960 }}
            >
              <TableHeadCustom
                order={table.order}
                orderBy={table.orderBy}
                headLabel={TABLE_HEAD}
                rowCount={dataFiltered.length}
                numSelected={table.selected.length}
                onSort={table.onSort}
              />

              <TableBody>
                {product.variations.map((variation) => (
                  <TableRow
                    key={variation.id}
                    selected={variation.id === selectedRow}
                    onClick={() => handleRowClick(variation.id)}
                  >
                    <TableCell>
                      <Stack direction="row">
                        <Avatar
                          alt={variation.name}
                          src={variation.image}
                          variant="rounded"
                          sx={{ width: 48, height: 48, mr: 2 }}
                        />
                        <ListItemText
                          primary={
                            <Box
                              component="div"
                              sx={{
                                typography: "body2",
                                color: "text.disabled",
                              }}
                            >
                              {variation.name}
                            </Box>
                          }
                          sx={{ display: "flex", flexDirection: "column" }}
                        />
                      </Stack>
                      <Stack direction="column">
                        <Chip
                          label="en stock"
                          size="small"
                          sx={{
                            backgroundColor: "#e8f5e9",
                            color: "#2e7d32",
                            mt: 0.5,
                            maxWidth: "70px",
                            overflow: "hidden",
                            textOverflow: "ellipsis",
                          }}
                        />
                        <Chip
                          size="small"
                          label="réapprovisionnement autorisé"
                          sx={{
                            backgroundColor: "#f3e5f5",
                            color: "#4a148c",
                            mt: 0.5,
                            maxWidth: "200px",
                            overflow: "hidden",
                            textOverflow: "ellipsis",
                          }}
                        />
                      </Stack>
                    </TableCell>
                    <TableCell>{variation.id}</TableCell>
                    <TableCell>{variation.UGS}</TableCell>
                    <TableCell>{variation.color}</TableCell>
                    <TableCell>{variation.size}</TableCell>
                    <TableCell>{variation.quantity}</TableCell>
                    <TableCell>{variation.price} € </TableCell>
                  </TableRow>
                ))}
              </TableBody>
            </Table>
          </TableContainer>
          <Box mt={2} ml={2} display="flex" alignItems="center">
            <Avatar sx={{ backgroundColor: "#9575cd", width: 35, height: 35 }}>
              <Button onClick={handleDecreaseQuantity}>-</Button>
            </Avatar>
            <TextField
              label="Quantité"
              type="number"
              value={defaultQuantity}
              onChange={handleQuantityChange}
              InputLabelProps={{ shrink: true }}
              sx={{ marginLeft: "8px", marginRight: "8px" }}
            />
            <Avatar sx={{ backgroundColor: "#9575cd", width: 35, height: 35 }}>
              <Button onClick={handleIncreaseQuantity}>+</Button>
            </Avatar>
          </Box>
        </CardContent>
        <Dialog open={open} onClose={handleClose} fullWidth maxWidth="xs">
          <DialogTitle>Modifier le Prix</DialogTitle>
          <DialogContent>
            <TextField
              autoFocus
              margin="normal"
              id="newPrice"
              label="Nouveau Prix (€)"
              type="number"
              fullWidth
              variant="outlined"
              value={newPrice}
              onChange={handlePriceChange}
              InputProps={{
                startAdornment: (
                  <InputAdornment position="start">€</InputAdornment>
                ),
              }}
            />
          </DialogContent>
          <DialogActions>
            <Button onClick={handleClose} color="secondary" variant="outlined">
              Annuler
            </Button>
            <Button
              onClick={handleUpdatePrice}
              color="primary"
              variant="contained"
            >
              Enregistrer
            </Button>
          </DialogActions>
        </Dialog>
      </Card>
    </Grid>
  );
}
//---------------------------------------------------------------------------------------

function applyFilter({
  inputData,
  comparator,
  filters,
}: {
  inputData: Variation[];
  comparator: (a: any, b: any) => number;
  filters: VariationTableFilters;
}) {
  const { name, color, size } = filters;

  const stabilizedThis = inputData.map((el, index) => [el, index] as const);

  stabilizedThis.sort((a, b) => {
    const order = comparator(a[0], b[0]);
    if (order !== 0) return order;
    return a[1] - b[1];
  });

  inputData = stabilizedThis.map((el) => el[0]);

  if (name) {
    inputData = inputData.filter(
      (product) => product.name.toLowerCase().indexOf(name.toLowerCase()) !== -1
    );
  }

  if (color.length) {
    inputData = inputData.filter((produit) => color.includes(produit.color));
  }
  if (size.length) {
    inputData = inputData.filter((produit) => color.includes(produit.size));
  }

  return inputData;
}
