import React, { useState, useCallback } from "react";
import Dialog from "@mui/material/Dialog";
import TextField from "@mui/material/TextField";
import Stack from "@mui/material/Stack";
import Button from "@mui/material/Button";
import InputAdornment from "@mui/material/InputAdornment";
import List from "@mui/material/List";
import ListItem from "@mui/material/ListItem";
import ListItemText from "@mui/material/ListItemText";
import ListItemAvatar from "@mui/material/ListItemAvatar";
import Avatar from "@mui/material/Avatar";
import { IProduct, _products } from "@/shared/_mock/_cartLive";
import Card from "@mui/material/Card";

type Props = {
  open: boolean;
  onClose: () => void;
  onSelect: (product: IProduct) => void;
};

export default function ProductListDialog({ open, onClose, onSelect }: Props) {
  const [searchQuery, setSearchQuery] = useState("");

  const handleSearchQueryChange = useCallback(
    (event: React.ChangeEvent<HTMLInputElement>) => {
      setSearchQuery(event.target.value);
    },
    []
  );

  // Function to handle product selection
  const handleProductSelect = useCallback(
    (product: IProduct) => {
      onSelect(product);
      setSearchQuery(""); // Clear search query after selection
      onClose();
    },
    [onClose, onSelect]
  );

  // Filter products based on search query
  const filteredProducts = _products.filter(
    (product) =>
      product.title.toLowerCase().includes(searchQuery.toLowerCase()) ||
      product.id.toLowerCase().includes(searchQuery.toLowerCase())
  );

  return (
    <Dialog fullWidth maxWidth="xs" open={open} onClose={onClose}>
      <Stack spacing={2} p={2}>
        <TextField
          value={searchQuery}
          onChange={handleSearchQueryChange}
          placeholder="Rechercher un nom de produit ou un UGS..."
          InputProps={{
            startAdornment: (
              <InputAdornment position="start">🔍</InputAdornment>
            ),
          }}
        />

        {searchQuery && (
          <List>
            {filteredProducts.map((product) => (
              <Card
                variant="outlined"
                key={product.id}
                sx={{ marginBottom: 2 }}
              >
                <ListItem
                  key={product.id}
                  button
                  onClick={() => handleProductSelect(product)}
                >
                  <ListItemAvatar>
                    <Avatar
                      variant="rounded"
                      alt={product.title}
                      sx={{ height: 100, width: 100 }}
                      src={product.image}
                    />
                  </ListItemAvatar>
                  <ListItem
                    sx={{
                      display: "flex",
                      flexDirection: "column",
                      alignItems: "start",
                      gap: 1,
                    }}
                  >
                    <ListItemText primary={product.title} />
                    <ListItemText secondary={product.id} />
                    <ListItemText secondary={`Quantité: ${product.quantity}`} />
                    <ListItemText
                      secondary={`Date de Création: ${product.createdAt.toString()}`}
                    />
                  </ListItem>
                </ListItem>
              </Card>
            ))}
          </List>
        )}

        <Button fullWidth onClick={onClose}>
          Cancel
        </Button>
      </Stack>
    </Dialog>
  );
}
