"use client";

import isEqual from "lodash/isEqual";
import { useState, useCallback, useMemo } from "react";

import Tab from "@mui/material/Tab";
import Tabs from "@mui/material/Tabs";
import Card from "@mui/material/Card";
import Table from "@mui/material/Table";
import Button from "@mui/material/Button";
import { alpha } from "@mui/material/styles";
import Container from "@mui/material/Container";
import TableBody from "@mui/material/TableBody";
import TableContainer from "@mui/material/TableContainer";

import { paths } from "@/routes/paths";
import { useRouter } from "@/hooks/use-router";

import { useBoolean } from "@/hooks/use-boolean";

import { _Order, Order_STATUS_OPTIONS } from "@/shared/_mock/_cartLive";

import Label from "@/shared/components/label";
import Iconify from "@/components/iconify";
import Scrollbar from "@/components/scrollbar";
import { useSnackbar } from "@/components/snackbar";
import { ConfirmDialog } from "@/shared/components/custom-dialog";
import { useSettingsContext } from "@/components/settings";
import CustomBreadcrumbs from "@/components/custom-breadcrumbs";
import {
  useTable,
  emptyRows,
  TableNoData,
  getComparator,
  TableEmptyRows,
  TableHeadCustom,
  TableSelectedAction,
  TablePaginationCustom,
} from "@/shared/components/table";
import {
  IOrder,
  OrderTableFilterValue,
  OrderTableFilters,
} from "@/shared/_mock/_cartLive";

import UserTableRow from "./Order-table-row";
import UserTableToolbar from "./Order-table-toolbar";
import UserTableFiltersResult from "./Order-table-filters-result";
import { Box, Divider, Grid, Stack, Typography } from "@mui/material";

// ----------------------------------------------------------------------

const STATUS_OPTIONS = [
  { value: "all", label: "All" },
  ...Order_STATUS_OPTIONS,
];

const TABLE_HEAD = [
  { id: "nameid", label: "Nom d'utilisateur" },
  { id: "name", label: "name", width: 180 },
  { id: "product", label: "Nom Produit", width: 180 },
  { id: "UGS", label: "UGS" },
  { id: "quantity", label: "Quantité" },
  { id: "CreatedAt", label: "Date de création", width: 180 },
  { id: "price", label: "Prix" },
  { id: "statusP", label: "Status Panier", width: 100 },
  { id: "statusC", label: "Status Commande", width: 100 },
  { id: "CreatedAt", label: "Date dernière Modification ", width: 180 },
  { id: "", width: 88 },
];

const defaultFilters: OrderTableFilters = {
  name: "",
  statutPanier: "all",
};

// ----------------------------------------------------------------------

export default function AllOrderView() {
  const { enqueueSnackbar } = useSnackbar();

  const table = useTable();

  const settings = useSettingsContext();

  const router = useRouter();

  const confirm = useBoolean();

  const [tableData, setTableData] = useState<IOrder[]>(_Order);

  const [filters, setFilters] = useState(defaultFilters);
  const memoizedApplyFilter = useMemo(() => applyFilter, []);

  const dataFiltered = memoizedApplyFilter({
    inputData: tableData,
    comparator: getComparator(table.order, table.orderBy),
    filters,
  });

  const dataInPage = dataFiltered.slice(
    table.page * table.rowsPerPage,
    table.page * table.rowsPerPage + table.rowsPerPage
  );

  const denseHeight = table.dense ? 56 : 56 + 20;

  const canReset = !isEqual(defaultFilters, filters);

  const notFound = (!dataFiltered.length && canReset) || !dataFiltered.length;

  const handleFilters = useCallback(
    (name: string, value: OrderTableFilterValue) => {
      table.onResetPage();
      setFilters((prevState) => ({
        ...prevState,
        [name]: value,
      }));
    },
    [table]
  );
  const handleResetFilters = useCallback(() => {
    setFilters(defaultFilters);
  }, []);

  const handleFilterStatus = useCallback(
    (event: React.SyntheticEvent, newValue: string) => {
      handleFilters("statutPanier", newValue);
    },
    [handleFilters]
  );

  return (
    <>
      <Grid xs={12}>
        <Card>
          <Stack
            direction="column"
            divider={<Divider orientation="horizontal" />}
            spacing={2}
            sx={{ ml: 2 }}
          >
            <Box
              sx={{
                display: "flex",
                flexDirection: "row",
              }}
            >
              <Typography variant="subtitle1" gutterBottom mt={2}>
                Commandes
              </Typography>
            </Box>
          </Stack>
          <Tabs
            value={filters.statutPanier}
            onChange={handleFilterStatus}
            sx={{
              px: 2.5,
              boxShadow: (theme) =>
                `inset 0 -2px 0 0 ${alpha(theme.palette.grey[500], 0.08)}`,
            }}
          >
            {STATUS_OPTIONS.map((tab) => (
              <Tab
                key={tab.value}
                iconPosition="end"
                value={tab.value}
                label={tab.label}
                icon={
                  <Label
                    variant={
                      ((tab.value === "all" ||
                        tab.value === filters.statutPanier) &&
                        "filled") ||
                      "soft"
                    }
                    color={
                      (tab.value === "payer" && "success") ||
                      (tab.value === "consulter" && "warning") ||
                      (tab.value === "supprimer" && "error") ||
                      "default"
                    }
                  >
                    {["payer", "consulter", "supprimer"].includes(tab.value)
                      ? tableData.filter(
                          (user) => user.statutPanier === tab.value
                        ).length
                      : tableData.length}
                  </Label>
                }
              />
            ))}
          </Tabs>

          <UserTableToolbar
            filters={filters}
            onFilters={handleFilters}
            //
          />

          {canReset && (
            <UserTableFiltersResult
              filters={filters}
              onFilters={handleFilters}
              //
              onResetFilters={handleResetFilters}
              //
              results={dataFiltered.length}
              sx={{ p: 2.5, pt: 0 }}
            />
          )}

          <TableContainer sx={{ position: "relative", overflow: "unset" }}>
            <TableSelectedAction
              dense={table.dense}
              numSelected={table.selected.length}
              rowCount={dataFiltered.length}
              onSelectAllRows={(checked) =>
                table.onSelectAllRows(
                  checked,
                  dataFiltered.map((row) => row.id)
                )
              }
            />

            <Scrollbar>
              <Table
                size={table.dense ? "small" : "medium"}
                sx={{ minWidth: 960 }}
              >
                <TableHeadCustom
                  order={table.order}
                  orderBy={table.orderBy}
                  headLabel={TABLE_HEAD}
                  rowCount={dataFiltered.length}
                  numSelected={table.selected.length}
                  onSort={table.onSort}
                  onSelectAllRows={(checked) =>
                    table.onSelectAllRows(
                      checked,
                      dataFiltered.map((row) => row.id)
                    )
                  }
                />

                <TableBody>
                  {dataFiltered
                    .slice(
                      table.page * table.rowsPerPage,
                      table.page * table.rowsPerPage + table.rowsPerPage
                    )
                    .map((row) => (
                      <UserTableRow
                        key={row.id}
                        row={row}
                        selected={table.selected.includes(row.id)}
                      />
                    ))}

                  <TableEmptyRows
                    height={denseHeight}
                    emptyRows={emptyRows(
                      table.page,
                      table.rowsPerPage,
                      dataFiltered.length
                    )}
                  />

                  <TableNoData notFound={notFound} />
                </TableBody>
              </Table>
            </Scrollbar>
          </TableContainer>

          <TablePaginationCustom
            count={dataFiltered.length}
            page={table.page}
            rowsPerPage={table.rowsPerPage}
            onPageChange={table.onChangePage}
            onRowsPerPageChange={table.onChangeRowsPerPage}
            //
            dense={table.dense}
            onChangeDense={table.onChangeDense}
          />
        </Card>
      </Grid>
    </>
  );
}

// ----------------------------------------------------------------------

function applyFilter({
  inputData,
  comparator,
  filters,
}: {
  inputData: IOrder[];
  comparator: (a: any, b: any) => number;
  filters: OrderTableFilters;
}) {
  const { name, statutPanier } = filters;

  const stabilizedThis = inputData.map((el, index) => [el, index] as const);

  stabilizedThis.sort((a, b) => {
    const order = comparator(a[0], b[0]);
    if (order !== 0) return order;
    return a[1] - b[1];
  });

  inputData = stabilizedThis.map((el) => el[0]);

  if (name) {
    inputData = inputData.filter(
      (user) => user.name.toLowerCase().indexOf(name.toLowerCase()) !== -1
    );
  }

  if (statutPanier !== "all") {
    inputData = inputData.filter((user) => user.statutPanier === statutPanier);
  }
  return inputData;
}
