import { ApexOptions } from "apexcharts";
import { useState, useCallback } from "react";

import Box from "@mui/material/Box";
import MenuItem from "@mui/material/MenuItem";
import ButtonBase from "@mui/material/ButtonBase";
import CardHeader from "@mui/material/CardHeader";
import Card, { CardProps } from "@mui/material/Card";

import Iconify from "@/components/iconify";
import Chart, { useChart } from "@/components/chart";
import CustomPopover, { usePopover } from "../../../components/custom-popover";

interface Props extends CardProps {
  title?: string;
  subheader?: string;
  chart: {
    categories?: string[];
    colors?: string[];
    series: {
      type: string;
      data: {
        name: string;
        data: number[];
      }[];
    }[];
    options?: ApexOptions;
  };
}

export default function BasketAmountChart({
  title,
  subheader,
  chart,
  ...other
}: Props) {
  const { categories, colors, series, options } = chart;

  const popover = usePopover();

  const [seriesData, setSeriesData] = useState([
    "Montant théorique des paniers",
    "Montant réel des paniers",
  ]);

  const chartOptions = useChart({
    colors,
    stroke: {
      show: true,
      width: 2,
      colors: ["transparent"],
    },
    xaxis: {
      categories,
    },
    tooltip: {
      y: {
        formatter: (value: number, { seriesIndex, dataPointIndex, w }: any) => {
          const theoreticalAmount =
            w.config.series[seriesIndex].data[dataPointIndex];
          const realAmount =
            w.config.series[1 - seriesIndex].data[dataPointIndex];
          const difference = theoreticalAmount - realAmount;
          return `€${value} (Écart : €${difference})`;
        },
      },
    },
    ...options,
  });

  const handleChangeSeries = useCallback(
    (newValue: string) => {
      popover.onClose();
      setSeriesData([newValue, "Montant réel des paniers"]);
    },
    [popover]
  );

  return (
    <>
      <Card {...other}>
        <CardHeader
          title={title}
          subheader={subheader}
          action={
            <ButtonBase
              onClick={popover.onOpen}
              sx={{
                pl: 1,
                py: 0.5,
                pr: 0.5,
                borderRadius: 1,
                typography: "subtitle2",
                bgcolor: "background.neutral",
              }}
            >
              Comparer à
              <Iconify
                width={16}
                icon={
                  popover.open
                    ? "eva:arrow-ios-upward-fill"
                    : "eva:arrow-ios-downward-fill"
                }
                sx={{ ml: 0.5 }}
              />
            </ButtonBase>
          }
        />

        <Box sx={{ mt: 3, mx: 3 }}>
          <Chart
            dir="ltr"
            type="bar"
            series={[
              ...(series.find((item) => item.type === seriesData[0])?.data ||
                []),
              ...(series.find((item) => item.type === seriesData[1])?.data ||
                []),
            ]}
            options={chartOptions}
            width="100%"
            height={364}
          />
        </Box>
      </Card>

      <CustomPopover
        open={popover.open}
        onClose={popover.onClose}
        sx={{ width: 140 }}
      >
        {["Période précédente", "Année précédente", "Mois en cours"].map(
          (option) => (
            <MenuItem
              key={option}
              selected={option === seriesData[0]}
              onClick={() => handleChangeSeries(option)}
            >
              {option}
            </MenuItem>
          )
        )}
      </CustomPopover>
    </>
  );
}
