import { useState, useEffect } from "react";
import ButtonBase from "@mui/material/ButtonBase";
import {
  Card,
  CardHeader,
  MenuItem,
  Menu,
  ListItemButton,
  ListItemText,
} from "@mui/material";
import Chart, { useChart } from "@/components/chart";
import Iconify from "@/components/iconify";
import { ApexOptions } from "apexcharts";

interface SeriesData {
  name: string;
  type: string;
  fill?: string;
  data: number[];
}

interface ChartData {
  labels: string[];
  colors?: string[];
  series: SeriesData[];
  options?: ApexOptions;
}

interface Props {
  title?: string;
  subheader?: string;
  chart: ChartData;
}

export default function AnalyticsNetSales({ title, subheader, chart }: Props) {
  const [selectedChartIndices, setSelectedChartIndices] = useState<number[]>([
    2,
  ]); // Default selection: index of "Mois en cours"
  const [anchorEl, setAnchorEl] = useState<null | HTMLElement>(null);

  const { labels, colors, series, options } = chart;

  const chartOptions = useChart({
    colors,
    plotOptions: {
      bar: {
        columnWidth: "16%",
      },
    },
    fill: {
      type: "solid",
    },
    labels,
    xaxis: {
      type: "datetime",
    },
    tooltip: {
      x: {
        format: "dd/MM/yy",
      },
      shared: true,
      intersect: false,
      y: {
        formatter: (value: number) => {
          if (typeof value !== "undefined") {
            return `${value.toFixed(0)} Produits`;
          }
          return value;
        },
      },
    },
    yaxis: {
      title: {
        text: "Nombre d'ajouts au panier",
      },
      min: 0,
      max: 1000,
    },

    ...options,
  });

  useEffect(() => {
    const defaultIndex = series.findIndex(
      (chart) => chart.name === "Mois en cours"
    );
    setSelectedChartIndices([defaultIndex]);
  }, []);

  const handleClick = (event: React.MouseEvent<HTMLButtonElement>) => {
    setAnchorEl(event.currentTarget);
  };

  const handleClose = () => {
    setAnchorEl(null);
  };

  const handleMenuItemClick = (index: number) => {
    const selectedIndex = selectedChartIndices.indexOf(index);
    let newSelected: number[] = [];

    if (selectedIndex === -1) {
      newSelected = [...selectedChartIndices, index];
    } else if (selectedChartIndices.length > 1) {
      newSelected = selectedChartIndices.filter((i) => i !== index);
    } else {
      return;
    }

    setSelectedChartIndices(newSelected);
  };

  return (
    <Card>
      <CardHeader
        title={title}
        subheader={subheader}
        action={
          <>
            <ButtonBase
              onClick={handleClick}
              sx={{
                pl: 1,
                py: 0.5,
                pr: 0.5,
                borderRadius: 1,
                typography: "subtitle2",
                bgcolor: "background.neutral",
                display: "flex",
                alignItems: "center",
              }}
            >
              Comparer à
              <Iconify
                width={16}
                icon={
                  anchorEl
                    ? "eva:arrow-ios-upward-fill"
                    : "eva:arrow-ios-downward-fill"
                }
                sx={{ ml: 0.5 }}
              />
            </ButtonBase>
            <Menu
              anchorEl={anchorEl}
              open={Boolean(anchorEl)}
              onClose={handleClose}
            >
              {series.map((chart, index) => (
                <MenuItem
                  key={index}
                  onClick={() => handleMenuItemClick(index)}
                >
                  <ListItemButton
                    selected={selectedChartIndices.indexOf(index) !== -1}
                  >
                    <ListItemText primary={chart.name} />
                  </ListItemButton>
                </MenuItem>
              ))}
            </Menu>
          </>
        }
      />
      <Chart
        dir="ltr"
        type="line"
        series={selectedChartIndices.map((index) => series[index])}
        options={chartOptions}
        width="100%"
        height={426}
      />
    </Card>
  );
}
