import { formatDistanceToNowStrict } from 'date-fns';

import Box from '@mui/material/Box';
import Stack from '@mui/material/Stack';
import Avatar from '@mui/material/Avatar';
import IconButton from '@mui/material/IconButton';
import Typography from '@mui/material/Typography';

import { useMockedUser } from '@/hooks/use-mocked-user';

import Iconify from '@/shared/components/iconify';

import { IChatMessage, IChatParticipant } from '@/shared/types/chat';

import { useGetMessage } from '@/hooks';
import { Tooltip } from '@mui/material';

// ----------------------------------------------------------------------

type Props = {
  message: IChatMessage;
  participants: IChatParticipant[];
  onOpenLightbox: (value: string) => void;
};

export default function ChatMessageItem({ message, participants, onOpenLightbox }: Props) {
  const { user } = useMockedUser();

  const { me, senderDetails, hasImage } = useGetMessage({
    message,
    participants,
    currentUserId: `${user?.id}`,
  });

  const { firstName, avatarUrl } = senderDetails;

  const { body, createdAt } = message;

  const renderInfo = (
    <Typography
      noWrap
      variant="caption"
      sx={{
        mb: 1,
        color: 'text.disabled',
        ...(!me && {
          mr: 'auto',
        }),
      }}
    >
      {!me && `${firstName},`} &nbsp;
      {formatDistanceToNowStrict(new Date(createdAt), {
        addSuffix: true,
      })}
    </Typography>
  );

  const renderBody = (
    <Stack
      sx={{
        p: 1.5,
        minWidth: 48,
        maxWidth: 320,
        borderRadius: 1,
        typography: 'body2',
        bgcolor: 'background.neutral',
        ...(me && {
          color: 'grey.800',
          bgcolor: 'primary.lighter',
        }),
        ...(hasImage && {
          p: 0,
          bgcolor: 'transparent',
        }),
        position: 'relative', // Ajoutez cette ligne pour rendre la position relative
      }}
    >
      {hasImage ? (
        <Box>
          <IconButton
            className="message-actions"
            size="small"
            sx={{
              position: 'absolute',
              opacity:0,
              top: -28,
              ...(!me && {
                right: 0,
              }),
              ...(me && {
                left: -10,
              }),
              transition: (theme) =>
                theme.transitions.create(['opacity'], {
                duration: theme.transitions.duration.shorter,
                
              }),
            }}
          >
            <Tooltip title='Replay'>
              <Iconify icon="solar:reply-bold" width={18} />
            </Tooltip>
          </IconButton>
          <Box
            component="img"
            alt="attachment"
            src={body}
            onClick={() => onOpenLightbox(body)}
            sx={{
              minHeight: 220,
              borderRadius: 1.5,
              cursor: 'pointer',
              '&:hover': {
                opacity: 0.9,
              },
            }}
          />
        </Box>
      ) : (
        <Box>
          <IconButton
            className="message-actions"
            size="small"
            sx={{
              position: 'absolute',
              opacity: 0,
              top: -28,
              ...(!me && {
                right: 0,
              }),
              ...(me && {
                left: -10,
              }),
              transition: (theme) =>
                theme.transitions.create(['opacity'], {
                duration: theme.transitions.duration.shorter,
                
              }),
            }}
          >
            <Tooltip title='Replay'>
              <Iconify icon="solar:reply-bold" width={18} />
            </Tooltip>
          </IconButton>
          
          {body}
        </Box>
      )}
    </Stack>
  );
  
  const renderActions = (
    <Stack
      direction="row"
      className="message-actions"
      sx={{
        pt: 0.5,
        opacity: 0,
        top: '100%',
        left: 0,
        position: 'absolute',
        transition: (theme) =>
          theme.transitions.create(['opacity'], {
            duration: theme.transitions.duration.shorter,
          }),
        ...(me && {
          left: 'unset',
          right: 0,
        }),
      }}
    >

      <IconButton size="small">
      <Tooltip title='React'>
        <Iconify icon="eva:smiling-face-fill" width={16} />
      </Tooltip>
      </IconButton>
      <IconButton size="small">
      <Tooltip title='Pin'>
        <Iconify icon="fluent-mdl2:pinned" width={16} />
      </Tooltip>
      </IconButton>
      <IconButton size="small" sx={{left:0}}>
      <Tooltip title='Share'>
        <Iconify icon="carbon:send-alt" width={16} />
      </Tooltip>
      </IconButton>
      <IconButton size="small">
      <Tooltip title='Delete'>
        <Iconify icon="solar:trash-bin-trash-bold" width={16} />
      </Tooltip>
      </IconButton>
    </Stack>
  );

  return (
    <Stack direction="row" justifyContent={me ? 'flex-end' : 'unset'} sx={{ mb: 5 }}>
      {!me && <Avatar alt={firstName} src={avatarUrl} sx={{ width: 32, height: 32, mr: 2 }} />}

      <Stack alignItems="flex-end">
        {renderInfo}

        <Stack
          direction="row"
          alignItems="center"
          sx={{
            position: 'relative',
            '&:hover': {
              '& .message-actions': {
                opacity: 1,
              },
            },
          }}
        >
          {renderBody}
          {renderActions}
        </Stack>
      </Stack>
    </Stack>
  );
}
