'use client';

import { useState, useCallback } from 'react';

import Tab from '@mui/material/Tab';
import Tabs from '@mui/material/Tabs';
import Card from '@mui/material/Card';
import Table from '@mui/material/Table';
import Stack from '@mui/material/Stack';
import Button from '@mui/material/Button';
import Container from '@mui/material/Container';
import TableBody from '@mui/material/TableBody';
import IconButton from '@mui/material/IconButton';
import { alpha, useTheme } from '@mui/material/styles';
import TableContainer from '@mui/material/TableContainer';
import LocalizationProvider from '@/shared/locales/localization-provider';
import { paths } from '@/routes/paths';
import { useRouter } from '@/hooks';
import RouterLink from '@/routes/router-link';
import { useBoolean } from '@/hooks';
import { _lives } from '@/shared/_mock';
import Label from '@/shared/components/label';
import Iconify from '@/shared/components/iconify';
import Scrollbar from '@/shared/components/scrollbar';
import { useSnackbar } from '@/components/snackbar';
import { ConfirmDialog } from '@/shared/components/custom-dialog'
import { useSettingsContext } from '@/shared/components/settings';
import CustomBreadcrumbs from '@/components/custom-breadcrumbs';
import {
  useTable,
  emptyRows,
  TableNoData,
  getComparator,
  TablePaginationCustom,
  TableEmptyRows,
} from '@/shared/components/table';
import { ILiveTableFilter, ILiveTableFilterValues } from '@/shared/types/live';
import { ILiveItem, LiveStatus } from '@/shared/types/live';
import LiveTableRow from './live-table-row';
import LiveSearch from './live-search';
import TableHead from './live-table-head';
import { useGetLives } from '@/shared/api/live';



const defaultFilters: ILiveTableFilter = {
  status: 'all',
  startDate: null,
  tags: [],
};


export default function AllLivesView() {

  const theme = useTheme();

  const settings = useSettingsContext();
  const router = useRouter();
  const table = useTable({ defaultOrderBy: 'startDate' });
  const { livesData } = useGetLives();
  const [filters, setFilters] = useState(defaultFilters);

  const [search, setSearch] = useState<{ query: string; results: ILiveItem[] }>({
    query: '',
    results: [],
  });

  const dataFiltered = applyFilter({
    inputData: livesData,
    comparator: getComparator(table.order, table.orderBy),
    filters,
  });


  const denseHeight = table.dense ? 56 : 56 + 20;


  const canReset = filters.status !== 'all' || !!filters.tags.length || (!!filters.startDate);

  const notFound = (!dataFiltered.length && canReset) || !dataFiltered.length;

  const getLiveLength = (status: string) =>
    livesData.filter((item) => item.status === status).length;


  const TABS = [
    { value: 'all', label: 'Tout', color: 'default', count: livesData.length },
    {
      value: LiveStatus.COMING,
      label: LiveStatus.COMING,
      color: 'error',
      count: getLiveLength(LiveStatus.COMING),
    },
    {
      value: LiveStatus.ONGOING,
      label: LiveStatus.ONGOING,
      color: 'warning',
      count: getLiveLength(LiveStatus.ONGOING),
    },

    {
      value: LiveStatus.REPLAY,
      label: LiveStatus.REPLAY,
      color: 'success',
      count: getLiveLength(LiveStatus.REPLAY),
    },
  ] as const;

  const handleFilters = useCallback(
    (name: string, value: ILiveTableFilterValues) => {
      table.onResetPage();
      setFilters((prevState) => ({
        ...prevState,
        [name]: value,
      }));
    },
    [table]
  );



  const handleSearch = useCallback(
    (inputValue: string) => {
      // setSearch((prevState) => ({
      //   ...prevState,
      //   query: inputValue,
      // }));

      // if (inputValue) {
      //   const results = _lives.filter(
      //     (live) => live.title.toLowerCase().indexOf(search.query.toLowerCase()) !== -1
      //   );

      //   setSearch((prevState) => ({
      //     query:prevState.query,
      //     results,
      //   }));
      // }
    },
    [search.query]
  );
  //----------------------------------------------------------------------------------






  const handleEditRow = useCallback(
    (id: string) => {
      router.push(paths.dashboard.live.edit(id));
    },
    [router]
  );
  const handleViewStats = useCallback(
    (id: string) => {
      router.push(paths.dashboard.live.statistics.single(id));
    },
    [router]
  );
  const handleFilterStatus = useCallback(
    (event: React.SyntheticEvent, newValue: string) => {
      handleFilters('status', newValue);
    },
    [handleFilters]
  );


  //----------------view details--------------


  const handleViewRow = useCallback(
    (id: string) => {
      router.push(paths.dashboard.live.details(id));
    },
    [router]
  );

  const renderSearch = (
    <Stack
      spacing={3}
      justifyContent="space-between"
      alignItems={{ xs: 'flex-end', sm: 'center' }}
      direction={{ xs: 'column', sm: 'row' }}
    >

      <Button
        component={RouterLink}
        href={paths.dashboard.live.new}
        variant="contained"
        startIcon={<Iconify icon="mingcute:add-line" />}
      >
        Nouveau live
      </Button>
      <LiveSearch
        query={search.query}
        results={search.results}
        onSearch={handleSearch}
        hrefItem={(id: string) => paths.dashboard.live.details(id)}
      />


    </Stack>
  );



  /////////////////////////////

  return (
    <>
      <LocalizationProvider >

        <Container maxWidth={settings.themeStretch ? false : 'lg'}>
          <CustomBreadcrumbs
            heading="Tout les lives"
            links={[
              {
                name: 'dashboard',
                href: paths.dashboard.root,
              },
              {
                name: 'lives',
                href: paths.dashboard.live.all_lives,
              },
              {
                name: 'Tout les lives',
              },
            ]}
            sx={{
              mb: { xs: 3, md: 5 },
            }}
          />

          <Stack
            spacing={2.5}
            sx={{
              mb: { xs: 3, md: 5 },
            }}
          >
            {renderSearch}

          </Stack>


          <Card>
            <Tabs
              value={filters.status}
              onChange={handleFilterStatus}
              sx={{
                px: 2.5,
                boxShadow: `inset 0 -2px 0 0 ${alpha(theme.palette.grey[500], 0.08)}`,
              }}
            >
              {TABS.map((tab) => (
                <Tab
                  key={tab.value}
                  value={tab.value}
                  label={tab.label}
                  iconPosition="end"
                  icon={
                    <Label
                      variant={
                        ((tab.value === 'all' || tab.value === filters.status) && 'filled') || 'soft'
                      }
                      color={tab.color}
                    >
                      {tab.count}
                    </Label>
                  }
                />
              ))}
            </Tabs>

            <TableContainer sx={{ position: 'relative', overflow: 'unset' }}>


              <Scrollbar>
                <Table size={table.dense ? 'small' : 'medium'} sx={{ minWidth: 800 }}>
                  <TableHead
                    order={table.order}
                    orderBy={table.orderBy}
                    onSort={table.onSort}

                  />

                  <TableBody>
                    {dataFiltered
                      .slice(
                        table.page * table.rowsPerPage,
                        table.page * table.rowsPerPage + table.rowsPerPage
                      )
                      .map((row) => (
                        <LiveTableRow
                          key={row.id}
                          row={row}
                          selected={table.selected.includes(row.id)}
                          onViewRow={() => handleViewRow(row.id)}
                          onViewStats={() => handleViewStats(row.id)}
                          onEditRow={() => handleEditRow(row.id)}
                        />
                      ))}


                    <TableEmptyRows
                      height={denseHeight}
                      emptyRows={emptyRows(table.page, table.rowsPerPage, dataFiltered.length)}
                    />

                    <TableNoData notFound={notFound} />
                  </TableBody>



                </Table>
              </Scrollbar>
            </TableContainer>

            <TablePaginationCustom
              count={dataFiltered.length}
              page={table.page}
              rowsPerPage={table.rowsPerPage}
              onPageChange={table.onChangePage}
              onRowsPerPageChange={table.onChangeRowsPerPage}
              dense={table.dense}
              onChangeDense={table.onChangeDense}
            />
          </Card>


        </Container>
      </LocalizationProvider>
    </>

  );
}


function applyFilter({
  inputData,
  comparator,
  filters,
}: {
  inputData: ILiveItem[];
  comparator: (a: any, b: any) => number;
  filters: ILiveTableFilter;
}) {
  const { status, tags } = filters;
  const stabilizedThis = inputData.map((el, index) => [el, index] as const);

  stabilizedThis.sort((a, b) => {
    const order = comparator(a[0], b[0]);
    if (order !== 0) return order;
    return a[1] - b[1];
  });

  inputData = stabilizedThis.map((el) => el[0]);



  if (status !== 'all') {
    inputData = inputData.filter((live) => live.status === status);
  }




  if (tags.length) {
    inputData = inputData.filter((live) => tags.includes(live.keyWord[0]));
  }
  return inputData;
}
