import { useCallback, useEffect, useState } from "react";
import { Button, IconButton, Paper, Stack, TextField, Typography } from "@mui/material";
import Iconify from "@/components/iconify";
import Label from "@/shared/components/label";
import { useBoolean, useResponsive, useRouter } from "@/hooks";
import { paths } from "@/routes/paths";
import RouterLink from "@/routes/router-link";
import { fDateTime } from "@/utils/format-time";
import { LiveStatus } from "@/shared/types/live";
import { getRealTimeStats, startLive, stopLive } from "@/shared/api/live";
import useTimer from "@/hooks/use-timer";
import usePolling from "@/hooks/use-polling";
import { useLiveStatistics } from "@/contexts/live-stats";
import { useGetLiveNumberOfviewers } from "@/contexts/live-stats";
import { ConfirmDialog } from "@/shared/components/custom-dialog";

interface HeadSectionProps {
    liveId: string;
    status: LiveStatus | undefined;
    keyStream: string | undefined;
    startedAt: Date | undefined;
}

export default function HeadSection({ liveId, startedAt, status }: HeadSectionProps) {
    const smUp = useResponsive('up', 'sm');
    const [elapsedTime, handleStart] = useTimer(new Date());
    const numberOfViewers = useGetLiveNumberOfviewers();

    useEffect(() => {
        if (status === LiveStatus.ONGOING) {
            handleStart();
        }
    }, [status, handleStart]);

    const startConfirm = useBoolean();
    const stopConfirm = useBoolean();

    const router = useRouter();
    const navigateToLives = useCallback(() => {
        router.push(paths.dashboard.live.root);
    }, [router]);

    return (
        <>
            <Paper
                sx={{
                    position: 'relative',
                    display: 'flex',
                    flexDirection: { xs: "column", lg: "row" },
                    mb: 4,
                    justifyContent: 'space-between',
                    alignItems: { xs: "start", lg: "center" },
                    gap: 2,
                    width: "100%",
                    bgcolor: "transparent",
                }}
            >
                <Stack width={300} direction="row" alignItems="flex-start">
                    <IconButton onClick={navigateToLives}>
                        <Iconify icon="eva:arrow-ios-back-fill" />
                    </IconButton>

                    <Stack spacing={0.5}>
                        <Stack spacing={1} direction="row" alignItems="center">
                            <Typography variant="h4"> Live #{liveId} </Typography>
                            <Label variant="soft" color="success">
                                {status}
                            </Label>
                        </Stack>

                        <Typography variant="body2" sx={{ color: 'text.disabled' }}>
                            {fDateTime(new Date())}
                        </Typography>
                    </Stack>
                </Stack>

                <Stack
                    spacing={3}
                    justifyContent={{ xs: 'center', md: 'flex-end' }}
                    alignItems="center"
                    direction="row"
                    height="100%"
                >
                    {status == LiveStatus.ONGOING && (
                        <>
                            <TextField
                                label="Spectateur"
                                size="small"
                                variant="outlined"
                                sx={{ maxWidth: 100 }}
                                value={numberOfViewers}
                            />
                            {smUp && (
                                <TextField
                                    label="Ajout au panier"
                                    size="small"
                                    variant="outlined"
                                    sx={{ maxWidth: 110 }}
                                    value={0}
                                />
                            )}

                            <TextField
                                value={elapsedTime}
                                label="Durée"
                                size="small"
                                variant="outlined"
                                sx={{ maxWidth: 100 }}
                                InputLabelProps={{ shrink: true }}
                                inputProps={{ step: 1 }}
                            />
                        </>
                    )}
                    {status === LiveStatus.COMING && (
                        <Button
                            onClick={() => { startConfirm.onTrue(); }}
                            variant="contained"
                            startIcon={<Iconify icon="material-symbols:not-started-rounded" />}
                        >
                            {smUp && "Démarrer"}
                        </Button>
                    )}
                    {status === LiveStatus.ONGOING && (
                        <Button
                            onClick={() => { stopConfirm.onTrue(); }}
                            variant="contained"
                            sx={{ color: 'error.main' }}
                            startIcon={<Iconify icon="fluent:record-stop-24-regular" />}
                        >
                            {smUp && "Arrêter"}
                        </Button>
                    )}
                    <Button
                        href={paths.dashboard.live.root}
                        LinkComponent={RouterLink}
                        variant="contained"
                        startIcon={<Iconify icon="material-symbols-light:text-select-move-back-word" />}
                    >
                        {smUp && "Quitter"}
                    </Button>
                </Stack>
            </Paper>
            <ConfirmDialog
                open={stopConfirm.value}
                onClose={stopConfirm.onFalse}
                title="Arrêter"
                content="voulez vous arréter le live?"
                action={
                    <Button variant="contained" color="error" onClick={() => { stopLive(liveId); stopConfirm.onFalse(); }}>
                        Arrêter
                    </Button>
                }
            />
            <ConfirmDialog
                open={startConfirm.value}
                onClose={startConfirm.onFalse}
                title="Démarrer"
                content="voulez vous démarrer le live?"
                action={
                    <Button variant="contained" color="success" onClick={() => { startLive(liveId); handleStart(); startConfirm.onFalse();}}>
                        Démarrer
                    </Button>
                }
            />
        </>
    );
}

