"use client"

import React, { useState } from 'react';
import {
    Avatar,
    Box,
    Button,
    Card,
    Grid,
    ListItemText,
    useTheme,
} from '@mui/material';
import { TimeField } from '@mui/x-date-pickers';
import Iconify from '@/components/iconify';
import { addProductLive, deleteProductLive } from '@/shared/api/live';
import { calculateSeconds, secondsToTime } from '@/utils/format-time';

type Props = {
    liveId: string;
    id: string;
    title: string;
    enabled: boolean;
    sku: string;
    image: string;
    price: number;
    startTime?: number | undefined;
    endTime?: number | undefined;
}

const ProductCard = (product: Props) => {
    const theme = useTheme();
    const [startTime, setStartTime] = useState<Date | null>();
    const [endTime, setEndTime] = useState<Date | null>();

    function addProductToLive() {
        if (startTime && endTime) {
            addProductLive(product.liveId, product.id, calculateSeconds(startTime), calculateSeconds(endTime))
        }
    }

    return (
        <Card variant="outlined" sx={{ marginBottom: 2, bgcolor: theme.palette.background.neutral }}>

            <Grid container >
                <Grid item xs={4} alignItems="flex-start"  >
                    <Avatar variant="rounded" sx={{ height: "100%", width: "100%" }} alt={product.title} src={product.image} />
                </Grid >
                <Grid item xs={8} sx={{ p: 1, gap: 2, display: "flex", flexDirection: "column", }} padding={2}>
                    <ListItemText
                        primary={product.title}
                        secondary={product.sku}
                        primaryTypographyProps={{ typography: 'body2', noWrap: true }}
                        secondaryTypographyProps={{
                            mt: 0.5,
                            component: 'span',
                            typography: 'caption',
                        }}
                    />
                    <Box sx={{ display: 'flex', width: "100%", justifyContent: "space-between", gap: 2 }}>
                        {!product.enabled && <Button
                            variant="contained"
                            fullWidth={true}
                            startIcon={<Iconify icon="mdi:add-bold" />}
                            onClick={addProductToLive}
                        >
                            Ajouter
                        </Button>}
                        <Button
                            onClick={() => { deleteProductLive(product.id, product.liveId) }}
                            variant="contained"
                            fullWidth={true}
                            startIcon={<Iconify icon="ic:round-delete" />}
                        >
                            supprimer
                        </Button>
                    </Box>
                    <Box sx={{ display: 'flex', width: "100%", justifyContent: "space-between", gap: 2 }}>
                        <TimeField
                            value={product.startTime && secondsToTime(product.startTime)}
                            onChange={(date) => setStartTime(date as Date)}
                            disabled={product.enabled}
                            fullWidth
                            size='small'
                            label="Start Time"
                            InputLabelProps={{
                                shrink: true,
                            }}
                            inputProps={{
                                step: 1,
                            }}
                            ampm={false}
                            format="HH:mm:ss"
                        />
                        <TimeField
                            fullWidth
                            size='small'
                            label="End Time"
                            onChange={(date) => setEndTime(date as Date)}
                            value={product.endTime && secondsToTime(product.endTime)}
                            disabled={product.enabled}
                            InputLabelProps={{
                                shrink: true,
                            }}
                            inputProps={{
                                step: 1,
                            }}
                            ampm={false}
                            format="HH:mm:ss"
                        />
                    </Box>
                </Grid>

            </Grid>
        </Card>
    )
};

export default ProductCard;
