import { useState, useEffect, useCallback, useMemo } from 'react';
import Stack from '@mui/material/Stack';
import Grid from '@mui/material/Grid';
import { useSettingsContext } from '@/shared/components/settings';
import Container from '@mui/material/Container';
import FormProvider, {
    RHFUpload,
} from "@/shared/components/hook-form";
import { Typography } from '@mui/material';
import * as Yup from "yup";
import { useForm } from "react-hook-form";
import { yupResolver } from "@hookform/resolvers/yup";
import { fData } from "@/utils/format-number";

// ----------------------------------------------------------------------

interface Props {
    onCreate?: VoidFunction;
    mediaName?: string;
    onChangeMediaName?: (event: React.ChangeEvent<HTMLInputElement>) => void;
    onUploadChangeTab: () => void;
}

export default function MultiMediaUploadView({
    onCreate,
    mediaName,
    onChangeMediaName,
    onUploadChangeTab,
    ...other
}: Props) {
    const [medias, setMedias] = useState<(File | string)[]>([]);

    const settings = useSettingsContext();

    const MediaSchema = Yup.object().shape({
        url: Yup.mixed<any>().nullable().required("Média requis"),
    });

    const defaultValues = useMemo(() => ({
        url: null,
    }),
        []
    );

    const methods = useForm({
        resolver: yupResolver(MediaSchema),
        defaultValues,
    });

    const {
        reset,
        watch,
        control,
        setValue,
        handleSubmit,
        formState: { isSubmitting },
    } = methods;

    const values = watch();

    const onSubmit = handleSubmit(async (data) => {
        try {
            await new Promise((resolve) => setTimeout(resolve, 500));
            reset();
            console.info("DONNÉES", data);
        } catch (error) {
            console.error(error);
        }
    });

    useEffect(() => {
        if (!open) {
            setMedias([]);
        }
    }, [open]);

    const handleDrop = useCallback(
        (acceptedMedias: File[]) => {
            const newMedias = acceptedMedias.map((media) =>
                Object.assign(media, {
                    preview: URL.createObjectURL(media),
                })
            );

            setMedias([...medias, ...newMedias]);
        },
        [medias]
    );

    const handleUpload = () => {
        onUploadChangeTab();
    };

    const handleRemoveMedia = (inputMedia: File | string) => {
        const filtered = medias.filter((media) => media !== inputMedia);
        setMedias(filtered);
    };

    const handleRemoveAllMedias = () => {
        setMedias([]);
    };

    return (
        <Container maxWidth={settings.themeStretch ? false : 'lg'} sx={{ height: '70vh', maxHeight: '80vh', overflowY: 'auto' }}>
            <FormProvider methods={methods} onSubmit={onSubmit}>
                <Stack maxWidth={'lg'} {...other} >
                    <Stack direction="row" alignItems="center" justifyContent="space-between">
                        <Typography variant="h4">Téléverser un média</Typography>
                    </Stack>
                    <Stack justifyContent={'center'} >
                        <Grid xs={12} md={12}>
                            <Stack sx={{ pt: 1, pb: 0, border: 'none' }}>
                                <RHFUpload
                                    multiple
                                    thumbnail
                                    files={medias}
                                    name="images"
                                    maxSize={3145728}
                                    onDrop={handleDrop}
                                    onRemove={handleRemoveMedia}
                                    onRemoveAll={handleRemoveAllMedias}
                                    onUpload={handleUpload}
                                    helperText={
                                        <Typography
                                            variant="caption"
                                            sx={{
                                                mt: 3,
                                                mx: "auto",
                                                display: "block",
                                                textAlign: "center",
                                                color: "text.disabled",
                                            }}
                                        >
                                            Formats autorisés : *.jpeg, *.jpg, *.png
                                            <br /> taille maximale de {fData(3145728)}
                                        </Typography>
                                    }
                                />
                            </Stack>
                        </Grid>
                    </Stack>
                </Stack>
            </FormProvider>
        </Container>
    );
}
