'use client';

import { useState, useCallback } from 'react';
import Stack from '@mui/material/Stack';
import Container from '@mui/material/Container';
import Typography from '@mui/material/Typography';
import { useBoolean } from '@/hooks/use-boolean';
import { isAfter, isBetween } from '@/utils/format-time';
import { _allMedias, MEDIA_TYPE_OPTIONS } from '@/shared/_mock';
import { useSnackbar } from '@/shared/components/snackbar';
import EmptyContent from '@/shared/components/empty-content';
import { fileFormat } from '@/shared/components/file-thumbnail';
import { useSettingsContext } from '@/shared/components/settings';
import { useTable, getComparator } from '@/shared/components/table';
import { useRouter } from '@/hooks';
import { IMedia, IMediaFilters, IMediaFilterValue } from '@/shared/types/media';
import FileManagerFilters from '../../media-manager-filters';
import FileManagerFiltersResult from '../../media-manager-filters-result';
import SingleMediaGrid from './Single-media-grid';


// ----------------------------------------------------------------------

const defaultFilters: IMediaFilters = {
    name: '',
    type: [],
    startDate: null,
    endDate: null,
};

// ----------------------------------------------------------------------

export default function SelectSingleMediaView() {
    const { enqueueSnackbar } = useSnackbar();

    const router = useRouter();

    const table = useTable({ defaultRowsPerPage: 10 });

    const settings = useSettingsContext();

    const openDateRange = useBoolean();

    const confirm = useBoolean();

    const [tableData, setTableData] = useState<IMedia[]>(_allMedias);

    const [filters, setFilters] = useState(defaultFilters);

    const dateError = isAfter(filters.startDate, filters.endDate);

    const dataFiltered = applyFilter({
        inputData: tableData,
        comparator: getComparator(table.order, table.orderBy),
        filters,
        dateError,
    });

    const canReset =
        !!filters.name || !!filters.type.length || (!!filters.startDate && !!filters.endDate);

    const notFound = (!dataFiltered.length && canReset) || !dataFiltered.length;

    const handleFilters = useCallback(
        (name: string, value: IMediaFilterValue) => {
            table.onResetPage();
            setFilters((prevState) => ({
                ...prevState,
                [name]: value,
            }));
        },
        [table]
    );

    const handleResetFilters = useCallback(() => {
        setFilters(defaultFilters);
    }, []);

    const handleAddMedia = (id: string) => {
        console.log("ID of media to add : "+id);
    };

    const renderFilters = (
        <Stack
            spacing={2}
            direction={{ xs: 'column', md: 'row' }}
            alignItems={{ xs: 'flex-end', md: 'center' }}
        >
            <FileManagerFilters
                openDateRange={openDateRange.value}
                onCloseDateRange={openDateRange.onFalse}
                onOpenDateRange={openDateRange.onTrue}
                //
                filters={filters}
                onFilters={handleFilters}
                //
                dateError={dateError}
                typeOptions={MEDIA_TYPE_OPTIONS}
            />
        </Stack>
    );

    const renderResults = (
        <FileManagerFiltersResult
            filters={filters}
            onResetFilters={handleResetFilters}
            //
            canReset={canReset}
            onFilters={handleFilters}
            //
            results={dataFiltered.length}
        />
    );

    return (
        <>
            <Container maxWidth={settings.themeStretch ? false : 'lg'}  sx={{ height: '75vh', overflowY: 'auto' }}> 
                <Stack direction="row" alignItems="center" justifyContent="space-between">
                    <Typography variant="h4">Médiathèque</Typography>
                </Stack>

                <Stack
                    spacing={2.5}
                    sx={{
                        my: { xs: 3, md: 5 },
                    }}
                >
                    {renderFilters}

                    {canReset && renderResults}
                </Stack>

                {notFound ? (
                    <EmptyContent
                        filled
                        title="No Data"
                        sx={{
                            py: 10,
                        }}
                    />
                ) : (
                    <>
                        <SingleMediaGrid
                            table={table}
                            dataFiltered={dataFiltered}
                            onAddMedia={handleAddMedia}
                            onOpenConfirm={confirm.onTrue}
                        />
                    </>
                )}
            </Container>
        </>
    );
}

// ----------------------------------------------------------------------

function applyFilter({
    inputData,
    comparator,
    filters,
    dateError,
}: {
    inputData: IMedia[];
    comparator: (a: any, b: any) => number;
    filters: IMediaFilters;
    dateError: boolean;
}) {
    const { name, type, startDate, endDate } = filters;

    const stabilizedThis = inputData.map((el, index) => [el, index] as const);

    stabilizedThis.sort((a, b) => {
        const order = comparator(a[0], b[0]);
        if (order !== 0) return order;
        return a[1] - b[1];
    });

    inputData = stabilizedThis.map((el) => el[0]);

    if (name) {
        inputData = inputData.filter(
            (file) => file.name.toLowerCase().indexOf(name.toLowerCase()) !== -1
        );
    }

    if (type.length) {
        inputData = inputData.filter((file) => type.includes(fileFormat(file.type)));
    }

    if (!dateError) {
        if (startDate && endDate) {
            inputData = inputData.filter((file) => isBetween(file.createdAt, startDate, endDate));
        }
    }

    return inputData;
}
