import { useState, useEffect, useCallback, useMemo } from 'react';
import Stack from '@mui/material/Stack';
import Button from '@mui/material/Button';
import Box from '@mui/material/Box';
import Iconify from '@/components/iconify';
import { Upload } from '@/shared/components/upload';
import { Typography } from '@mui/material';
import * as Yup from "yup";
import { useForm } from "react-hook-form";
import { yupResolver } from "@hookform/resolvers/yup";
import { fData } from "@/utils/format-number";
import FormProvider from '@/components/hook-form/form-provider';

interface Props {
    onCreate?: VoidFunction;
    mediaName?: string;
    onChangeMediaName?: (event: React.ChangeEvent<HTMLInputElement>) => void;
    onUploadChangeTab: () => void;
}

export default function SingleMediaUploadView({
    onCreate,
    mediaName,
    onChangeMediaName,
    onUploadChangeTab,
    ...other
}: Props) {
    const [medias, setMedias] = useState<(File | string)[]>([]);
    const [previewImage, setPreviewImage] = useState<string | null>(null);

    const MediaSchema = Yup.object().shape({
        url: Yup.mixed<any>().nullable().required("Média requis"),
    });

    const defaultValues = useMemo(() => ({
        url: null,
    }),
        []
    );

    const methods = useForm({
        resolver: yupResolver(MediaSchema),
        defaultValues,
    });

    const {
        reset,
        watch,
        control,
        setValue,
        handleSubmit,
        formState: { isSubmitting },
    } = methods;

    const values = watch();

    const onSubmit = handleSubmit(async (data) => {
        try {
            await new Promise((resolve) => setTimeout(resolve, 500));
            reset();
            console.info("DONNÉES", data);
        } catch (error) {
            console.error(error);
        }
    });

    const handleDrop = useCallback(
        (acceptedFiles: File[]) => {
            const file = acceptedFiles[0];

            const newFile = Object.assign(file, {
                preview: URL.createObjectURL(file),
            });

            if (file) {
                setValue("url", newFile, { shouldValidate: true });
                setPreviewImage(newFile.preview);
            }
        },
        [setValue]
    );

    useEffect(() => {
        setMedias([]);
        setPreviewImage(null);
    }, []);

    const handleUpload = () => {
        onUploadChangeTab();
    };

    const handleRemoveMedia = (inputMedia: File | string) => {
        const filtered = medias.filter((media) => media !== inputMedia);
        setMedias(filtered);
        setPreviewImage(null);
    };

    const handleRemoveAllMedias = () => {
        setMedias([]);
        setPreviewImage(null);
    };

    return (
        <FormProvider methods={methods} onSubmit={onSubmit}>
            <Stack direction="row" alignItems="center" justifyContent="space-between">
                <Typography variant="h4">Téléverser média</Typography>
            </Stack>
            <Stack maxWidth={'lg'} {...other} sx={{ height: '70vh', overflowY: 'auto' }}>
                <Stack justifyContent={'center'}>
                    <Stack sx={{ pt: 1, pb: 0, border: 'none' }}>
                        <Upload
                            files={medias}
                            onDrop={handleDrop}
                            onRemove={handleRemoveMedia}
                            onUpload={handleUpload}
                            helperText={
                                <Typography
                                    variant="caption"
                                    sx={{
                                        mt: 3,
                                        mx: "auto",
                                        display: "block",
                                        textAlign: "center",
                                        color: "text.disabled",
                                    }}
                                >
                                    Formats autorisés : *.jpeg, *.jpg, *.png
                                    <br /> taille maximale de {fData(3145728)}
                                </Typography>
                            }
                        />
                    </Stack>
                    {previewImage && (
                        <Stack direction="row" marginBottom={1}>
                            <Box
                                sx={{
                                    width: '80px',
                                    height: '80px',
                                    borderRadius: '8px',
                                    overflow: 'hidden',
                                    position: 'relative',
                                    backgroundColor: 'white',
                                    boxShadow: '0px 4px 10px rgba(0, 0, 0, 0.1)',
                                }}
                            >
                                <Box
                                    component="div"
                                    sx={{
                                        width: '100%',
                                        height: '100%',
                                        backgroundImage: `url(${previewImage})`,
                                        backgroundSize: 'cover',
                                        backgroundPosition: 'center',
                                    }}
                                />
                            </Box>
                        </Stack>
                    )}
                </Stack>


                {previewImage && (
                    <Stack sx={{ mt: 1 }}>
                        <Stack direction='row' width={'200px'}>
                            <Button size="small" variant="outlined" color="inherit" onClick={handleRemoveAllMedias} sx={{ width: '50%' }}>
                                Remove
                            </Button>
                            <Button
                                type="submit"
                                size="small"
                                variant="contained"
                                startIcon={<Iconify icon="eva:cloud-upload-fill" />}
                                onClick={handleUpload}
                                sx={{ width: '50%', ml: 1 }}
                                disabled={isSubmitting}
                            >
                                Upload
                            </Button>
                        </Stack>
                    </Stack>
                )}
            </Stack>
        </FormProvider>
    );
}
