import React, { useState, useRef, useEffect } from 'react';
import AvatarEditor from 'react-avatar-editor';
import { Modal, Button, Typography, Grid, Stack, ButtonGroup } from '@mui/material';
import { IMedia } from '@/shared/types/media';;
import ZoomOutIcon from '@mui/icons-material/ZoomOut';
import ZoomInIcon from '@mui/icons-material/ZoomIn';
import RotateLeftIcon from '@mui/icons-material/RotateLeft';
import RotateRightIcon from '@mui/icons-material/RotateRight';
import Slider from '@mui/material/Slider';

type ImageEditorProps = {
  open: boolean;
  onClose: () => void;
  image: IMedia | undefined;
};

const EditImageModal: React.FC<ImageEditorProps> = ({ open, onClose, image }) => {
  const [scale, setScale] = useState<number>(1);
  const [rotate, setRotate] = useState<number>(0);
  const [aspectRatio, setAspectRatio] = useState<number | null>(1);
  const editorRef = useRef<AvatarEditor | null>(null);
  const [originalWidth, setOriginalWidth] = useState<number | null>(null);
  const [originalHeight, setOriginalHeight] = useState<number | null>(null);

  const width = aspectRatio ? 300 : undefined;
  const height = aspectRatio ? (300 / aspectRatio) : undefined;

  useEffect(() => {
    if (image) {
      const img = new Image();
      img.src = image.url;
      img.onload = () => {
        setOriginalWidth(img.width);
        setOriginalHeight(img.height);
      };
    }
  }, [image]);

  const handleScaleChange = (event: Event, newValue: number | number[]) => {
    setScale(newValue as number);
  };

  const handleRotateChange = (event: Event, newValue: number | number[]) => {
    setRotate(newValue as number);
  };

  const handleAspectRatioChange = (ratio: number | null) => {
    setAspectRatio(ratio);
    if (ratio === 0) {
      handleAutoDetectAspectRatio();
    }
  };

  const handleAutoDetectAspectRatio = () => {
    if (originalWidth && originalHeight) {
      handleAspectRatioChange(originalWidth / originalHeight);
    }
  };

  const handleZoomOut = () => {
    setScale(prevScale => Math.max(prevScale - 0.1, 1));
  };

  const handleZoomIn = () => {
    setScale(prevScale => Math.min(prevScale + 0.1, 4));
  };

  const handleRotateLeft = () => {
    setRotate(prevRotate => (prevRotate - 90 + 360) % 360);
  };

  const handleRotateRight = () => {
    setRotate(prevRotate => (prevRotate + 90) % 360);
  };

  const handleSave = () => {
    if (editorRef.current && image) {
      console.log('Saving edited image:', editorRef.current.getImage());
      onClose();
    }
  };

  const aspectRatioOptions = [
    { value: 0, label: 'Full' },
    { value: 1, label: 'Square' },
    { value: 2, label: '16:9' },
    { value: 3, label: '4:3' },
    { value: 4, label: '3:2' },
    { value: 5, label: '8:5' },
    { value: 6, label: '5:3' },
    { value: 7, label: '2:1' },
  ];

  
  return (
    <Modal open={open} onClose={onClose}>
      <Grid container justifyContent="center" alignItems="center" style={{ width: '65%', maxWidth: '70%', position: 'absolute', top: '50%', left: '50%', transform: 'translate(-50%, -50%)', backgroundColor: 'white', padding: '20px', borderRadius: '15px' }}>
        <Grid item xs={12} sx={{mb: 3}}>
          <Typography variant="h5" align="center" gutterBottom>
            Modifier image
          </Typography>
        </Grid>
        <Grid item xs={12}>
          <Grid container justifyContent="space-between" alignItems="center">
            <Grid item style={{ marginRight: '-10px', maxWidth: '40%', maxHeight: '40%' }}>
              {image && (
                <div style={{ position: 'relative', width: width, height: height }}>
                <AvatarEditor
                  ref={editorRef}
                  image={image.url}
                  width={width}
                  height={height}
                  scale={scale}
                  rotate={rotate}
                  borderRadius={0}
                  style={{ position: 'absolute', top: 0, left: 0 }}
                />
              </div>
              )}
            </Grid>
            <Grid item xs={5} style={{ marginLeft: '-10px' }}>
              {image && (
                <>
                  <Stack sx={{ marginBottom: '10px', width: '95%' }}>
                    <Stack direction="row" alignItems="center" spacing={1} marginBottom="10px">
                      <Typography variant="h6">Zoom:</Typography>
                      <Typography variant="body1">{Math.round(scale * 100)}%</Typography>
                    </Stack>
                    <ButtonGroup>
                      <Button onClick={handleZoomOut}><ZoomOutIcon /></Button>
                      <Slider
                        value={scale}
                        min={1}
                        max={4}
                        step={0.01}
                        onChange={handleScaleChange}
                        style={{ width: '100%', margin: '0 10px' }}
                      />
                      <Button onClick={handleZoomIn}><ZoomInIcon /></Button>
                    </ButtonGroup>
                  </Stack>
                  <Stack sx={{ width: '95%' }}>
                    <Stack direction="row" alignItems="center" spacing={1} marginBottom="10px">
                      <Typography variant="h6">Rotate:</Typography>
                      <Typography variant="body1">{rotate}°</Typography>
                    </Stack>
                    <ButtonGroup>
                      <Button onClick={handleRotateLeft}><RotateLeftIcon /></Button>
                      <Slider
                        value={rotate}
                        min={0}
                        max={360}
                        step={1}
                        onChange={handleRotateChange}
                        style={{ width: '100%', margin: '0 10px' }}
                      />
                      <Button onClick={handleRotateRight}><RotateRightIcon /></Button>
                    </ButtonGroup>
                  </Stack>
                </>
              )}
              <br />
              <Stack sx={{ width: '95%' }}>
                <Typography variant="h6">Scale:</Typography>
                <Slider
                  value={aspectRatio ? aspectRatioOptions.findIndex(option => option.value === aspectRatio) : 0}
                  min={0}
                  max={7}
                  step={1}
                  marks={aspectRatioOptions.map(option => ({ value: option.value, label: option.label }))}
                  onChange={(event, newValue) => handleAspectRatioChange(aspectRatioOptions[newValue as number].value)}
                />
              </Stack>
            </Grid>
          </Grid>
        </Grid>
        <Grid item xs={12}>
          <Grid container justifyContent="center" style={{ marginTop: '80px' }}>
            <Button variant="outlined" onClick={onClose} style={{ marginRight: '10px' }}>Cancel</Button>
            <Button variant="contained" onClick={handleSave}>Save</Button>
          </Grid>
        </Grid>
      </Grid>
    </Modal>
  );
};

export default EditImageModal;
