'use client';

import { useState } from 'react';
import * as Yup from 'yup';
import { useForm } from 'react-hook-form';
import { yupResolver } from '@hookform/resolvers/yup';
import { useEffect, useMemo } from 'react';
import { useResponsive } from '@/hooks/use-responsive';
import { useSnackbar } from '@/components/snackbar';
import FormProvider, {
    RHFEditor,
    RHFTextField,
} from "@/shared/components/hook-form";
import { paths } from '@/routes/paths';
import LoadingButton from '@mui/lab/LoadingButton';
import { useRouter } from '@/hooks';
import Box from '@mui/material/Box';
import Stack from '@mui/material/Stack';
import Grid from '@mui/material/Grid';
import Button from '@mui/material/Button';
import Divider from '@mui/material/Divider';
import Typography from '@mui/material/Typography';
import { fData } from '@/utils/format-number';
import { fDateTime } from '@/utils/format-time';
import Scrollbar from '@/components/scrollbar';
import FileThumbnail, { fileFormat } from '@/shared/components/file-thumbnail';
import { _allMedias } from '@/shared/_mock';
import CustomBreadcrumbs from "@/shared/components/custom-breadcrumbs";
import { FormLabel } from '@mui/material';
import EditImageModal from './edit-image/EditImageModal';

// ----------------------------------------------------------------------

type Props = {
    id: string;
};
export default function EditMediaView({ id }: Props) {

    const mediaFinded = _allMedias.find(media => media.id === id)

    const router = useRouter();

    const mdUp = useResponsive('up', 'md');

    const { enqueueSnackbar } = useSnackbar();

    const MediaSchema = Yup.object().shape({
        title: Yup.string().required('Titre est requis'),
        description: Yup.string().required("Description est requis"),
        url: Yup.mixed()
            .nullable()
            .test('is-valid-image', 'Image must be a File or a URL string', value =>
                typeof value === 'string' || (value instanceof File)
            ),
    });

    const defaultValues = useMemo(() => ({
        title: mediaFinded?.title || '',
        name: mediaFinded?.name || '',
        size: mediaFinded?.size || '',
        url: mediaFinded?.url || '',
        type: mediaFinded?.type || '',
        modifiedAt: mediaFinded?.modifiedAt || '',
        createdAt: mediaFinded?.createdAt || '',
        author: mediaFinded?.author || '',
        description: mediaFinded?.description || '',
    }), [mediaFinded]);

    const methods = useForm({
        resolver: yupResolver(MediaSchema),
        defaultValues,
    });

    const {
        reset,
        watch,
        setValue,
        handleSubmit,
        formState: { isSubmitting },
    } = methods;

    const values = watch();

    useEffect(() => {
        if (mediaFinded) {
            reset(defaultValues);
        }
        console.log(defaultValues)
    }, [mediaFinded, defaultValues, reset])

    const onSubmit = handleSubmit(async (data) => {
        try {
            await new Promise((resolve) => setTimeout(resolve, 500));
            reset();
            enqueueSnackbar('Edit success!', { variant: 'success' });
            router.push(paths.dashboard.media.root);
            console.info('Form Data:', data);
        } catch (error) {
            console.error(error);
            enqueueSnackbar('Failed to edit this media.', { variant: 'error' });
        }
    });

    const [open, setOpen] = useState<boolean>(false);

    const handleOpen = () => {
        setOpen(true);
    };

    const handleClose = () => {
        setOpen(false);
    };

    const renderDetails = (
        <>
            <Scrollbar sx={{ height: 1 }}>
                <Grid container spacing={2} sx={{ mt: 2 }}>
                    <Grid item xs={6} sx={{ ml: 3, mr: '5%' }}>
                        <CustomBreadcrumbs
                            heading="Modifier média"
                            links={[
                                { name: "Dashboard", href: paths.dashboard.root },
                                { name: "Médiathèque", href: paths.dashboard.media.root },
                                { name: "Modifier", },
                            ]}
                            sx={{
                                mb: {
                                    xs: 3,
                                    md: 5,
                                },
                            }}
                        />
                        <Stack alignItems="center" justifyContent="space-between" sx={{ p: 2.5 }}>
                            <RHFTextField name="title" label="Titre du média" />
                            <FormLabel sx={{ mt: 2, mr: '86%' }}>Description</FormLabel>
                            <RHFEditor simple name="description"
                                onChange={(value) => {
                                    console.log(value);
                                }}
                            />
                            <Button variant="outlined" onClick={handleOpen} sx={{ width: '100%', mt:2 }}>Modifier image</Button>
                            <EditImageModal open={open} onClose={handleClose} image={mediaFinded} />
                        </Stack>

                        <Grid sx={{ display: 'flex', justifyContent: "left" }}>
                            <LoadingButton
                                type="submit"
                                variant="contained"
                                size="large"
                                loading={isSubmitting}
                                sx={{ ml: 2 }}
                            >Enregistrer</LoadingButton>
                        </Grid>
                    </Grid>
                    <Grid item xs={5}>
                        <Stack
                            spacing={2.5}
                            justifyContent="center"
                            sx={{
                                p: 2.5,
                                bgcolor: 'background.neutral',
                            }}
                        >
                            <FileThumbnail
                                imageView
                                file={defaultValues.type === 'folder' ? defaultValues.type : defaultValues.url}
                                sx={{ width: 20, height: 20 }}
                                imgSx={{ borderRadius: 1 }}
                            />

                            <Typography variant="subtitle1" sx={{ wordBreak: 'break-all' }}>
                                {defaultValues.name}
                            </Typography>

                            <Divider sx={{ borderStyle: 'dashed' }} />
                            <Stack spacing={1.5}>
                                <Stack direction="row" sx={{ typography: 'caption', textTransform: 'capitalize' }}>
                                    <Box component="span" sx={{ width: 90, color: 'text.secondary', mr: 2 }}>
                                        Titre du média :
                                    </Box>
                                    {defaultValues.title}
                                </Stack>
                                <Stack direction="row" sx={{ typography: 'caption', textTransform: 'capitalize' }}>
                                    <Box component="span" sx={{ width: 90, color: 'text.secondary', mr: 2 }}>
                                        Description du média :
                                    </Box>
                                    {defaultValues.description}
                                </Stack>
                                <Stack direction="row" sx={{ typography: 'caption', textTransform: 'capitalize' }}>
                                    <Box component="span" sx={{ width: 90, color: 'text.secondary', mr: 2 }}>
                                        Modifié le :
                                    </Box>
                                    {fDateTime(defaultValues.modifiedAt)}
                                </Stack>
                                <Stack direction="row" sx={{ typography: 'caption', textTransform: 'capitalize' }}>
                                    <Box component="span" sx={{ width: 90, color: 'text.secondary', mr: 2 }}>
                                        Téléversé par :
                                    </Box>
                                    {defaultValues.author}
                                </Stack>
                                <Stack direction="row" sx={{ typography: 'caption', textTransform: 'capitalize' }}>
                                    <Box component="span" sx={{ width: 90, color: 'text.secondary', mr: 2 }}>
                                        Téléversé le:
                                    </Box>
                                    {fDateTime(defaultValues.createdAt)}
                                </Stack>
                                <Stack direction="row" sx={{ typography: 'caption', textTransform: 'capitalize' }}>
                                    <Box component="span" sx={{ width: 90, color: 'text.secondary', mr: 2 }}>
                                        Taille :
                                    </Box>
                                    {fData(defaultValues.size)}
                                </Stack>
                                <Stack direction="row" sx={{ typography: 'caption', textTransform: 'capitalize' }}>
                                    <Box component="span" sx={{ width: 90, color: 'text.secondary', mr: 2 }}>
                                        Type :
                                    </Box>
                                    {fileFormat(defaultValues.type)}
                                </Stack>
                            </Stack>
                        </Stack>
                    </Grid>
                </Grid>

            </Scrollbar>
        </>
    );

    return (
        <FormProvider methods={methods} onSubmit={onSubmit}>
            <Grid container spacing={3}>
                {renderDetails}
            </Grid>
        </FormProvider>
    );
}
