import React, { useState } from "react";
import {
  Grid,
  Stack,
  TextField,
  IconButton,
  MenuItem,
  Select,
} from "@mui/material";
import Autocomplete from "@mui/material/Autocomplete";
import AddIcon from "@mui/icons-material/Add";
import DeleteIcon from "@mui/icons-material/Delete";
import LoadingButton from "@mui/lab/LoadingButton";

interface Entry {
  colonnes: string;
  operateur: string;
  valeur: string | Date;
}

const OPERATEURS_PAR_COLONNES: { [key: string]: string[] } = {
  date: ["supérieur à", "inférieur à", "égal à"],
  commande: ["contient", "commence avec", "se termine par"],
  client: ["contient", "commence avec", "se termine par"],
  Article: ["contient", "commence avec", "se termine par"],
  prix: ["supérieur à", "inférieur à", "égal à"],
  origine: ["égal à"],
};

export default function FilterView() {
  const [entries, setEntries] = useState<Entry[]>([
    { colonnes: "", operateur: "", valeur: "" },
  ]);

  const handleAddEntry = () => {
    setEntries([...entries, { colonnes: "", operateur: "", valeur: "" }]);
  };

  const handleRemoveEntry = (index: number) => {
    const newEntries = [...entries];
    newEntries.splice(index, 1);
    setEntries(newEntries);
  };

  const handleInputChange = (
    index: number,
    field: keyof Entry,
    value: string | Date
  ) => {
    const newEntries = [...entries];
    newEntries[index] = { ...newEntries[index], [field]: value };
    if (field === "colonnes") {
      const colonne = value as string;
      const operateurOptions = OPERATEURS_PAR_COLONNES[colonne] || [];
      newEntries[index].operateur = operateurOptions[0] || "";
      if (colonne === "origine") {
        newEntries[index].valeur = "site";
      }
    }
    setEntries(newEntries);
  };

  const handleSubmit = (event: React.FormEvent<HTMLFormElement>) => {
    event.preventDefault();
    console.log("Formulaire soumis !");
  };

  return (
    <form onSubmit={handleSubmit} style={{ maxWidth: "100%", padding: "20px" }}>
      <Stack spacing={1} style={{ marginTop: "40px" }}>
        {entries.map((entry, index) => (
          <Grid container spacing={1} alignItems="center" key={index}>
            <Grid item xs={3}>
              <Autocomplete
                value={entry.colonnes}
                onChange={(_event, newValue: string | null) => {
                  const colonne = newValue !== null ? newValue : "";
                  handleInputChange(index, "colonnes", colonne);
                }}
                options={[
                  "date",
                  "commande",
                  "client",
                  "Article",
                  "prix",
                  "origine",
                ]}
                fullWidth
                renderInput={(params: any) => (
                  <TextField {...params} label="Colonnes" />
                )}
              />
            </Grid>
            <Grid item xs={4}>
              <Autocomplete
                value={entry.operateur}
                onChange={(_event, newValue: string | null) =>
                  handleInputChange(
                    index,
                    "operateur",
                    newValue !== null ? newValue : ""
                  )
                }
                options={OPERATEURS_PAR_COLONNES[entry.colonnes] || []}
                fullWidth
                renderInput={(params: any) => (
                  <TextField {...params} label="Opérateur" />
                )}
              />
            </Grid>
            <Grid item xs={3}>
              {entry.colonnes === "date" ? (
                <TextField
                  value={entry.valeur as Date}
                  type="date"
                  onChange={(event) =>
                    handleInputChange(index, "valeur", event.target.value)
                  }
                  fullWidth
                />
              ) : entry.colonnes === "origine" ? (
                <Select
                  value={entry.valeur as string}
                  onChange={(event) =>
                    handleInputChange(index, "valeur", event.target.value)
                  }
                  fullWidth
                >
                  <MenuItem value="site">Site</MenuItem>
                  <MenuItem value="en direct">En direct</MenuItem>
                </Select>
              ) : (
                <TextField
                  value={entry.valeur as string}
                  onChange={(event) =>
                    handleInputChange(index, "valeur", event.target.value)
                  }
                  fullWidth
                  label="Valeur"
                />
              )}
            </Grid>
            <Grid item xs={2} container alignItems="center">
              <IconButton onClick={handleAddEntry}>
                <AddIcon fontSize="small" />
              </IconButton>
              {index > 0 && (
                <IconButton onClick={() => handleRemoveEntry(index)}>
                  <DeleteIcon fontSize="small" />
                </IconButton>
              )}
            </Grid>
          </Grid>
        ))}
        <Grid item xs={12} style={{ marginTop: "20px" }}>
          <LoadingButton type="submit" variant="contained" fullWidth>
            Filtrer les commandes
          </LoadingButton>
        </Grid>
      </Stack>
    </form>
  );
}
