'use client';

import { useState, useCallback } from 'react';

import Card from '@mui/material/Card';
import Table from '@mui/material/Table';
import Grid from '@mui/material/Grid';
import Container from '@mui/material/Container';
import TableBody from '@mui/material/TableBody';
import TableCell from '@mui/material/TableCell';
import TableHead from '@mui/material/TableHead';
import TableContainer from '@mui/material/TableContainer';
import LocalizationProvider from '@/shared/locales/localization-provider';
import { paths } from '@/routes/paths';
import { useRouter } from '@/hooks';
import Scrollbar from '@/shared/components/scrollbar';
import { useSnackbar } from '@/components/snackbar';
import { useSettingsContext } from '@/shared/components/settings';
import CustomBreadcrumbs from '@/components/custom-breadcrumbs';
import { useTable, emptyRows, TableNoData, TablePaginationCustom, TableEmptyRows, } from '@/shared/components/table';
import PaymentMethodTableRow from './methods-table-row';
import { Reorder } from 'framer-motion';
import { IPaymentMethod } from '@/shared/types/payment';
import { useGetPaymentMethods } from '@/shared/api/payment';




export default function PaymentMethodsView() {
    const settings = useSettingsContext();
    const router = useRouter();
    const table = useTable({ defaultOrderBy: 'title' });
    const [tableData, setTableData] = useState<IPaymentMethod[]>(useGetPaymentMethods().paymentMethodsData);




    const denseHeight = table.dense ? 56 : 56 + 20;


    const notFound = (!tableData.length) || !tableData.length;


    const handleViewRow = useCallback(
        (link: string) => {
            router.push(paths.dashboard.payment.method(link));
        },
        [tableData]
    );

    const handleReorder = (newOrder: number[]) => {
        const reorderedData = newOrder.map(index => tableData[index]);
        setTableData(reorderedData);
    };

    return (
        <>
            <LocalizationProvider >
                <Container maxWidth={settings.themeStretch ? false : 'lg'}>
                    <Grid container alignItems="center" justifyContent="space-between">
                        <Grid item>
                            <CustomBreadcrumbs
                                heading="Méthodes de paiements"
                                links={[
                                    {
                                        name: 'dashboard',
                                        href: paths.dashboard.root,
                                    },
                                    {
                                        name: "Paiement",
                                        href: paths.dashboard.payment.root,
                                    }
                                ]}
                                sx={{
                                    mb: { xs: 3, md: 5 },
                                }}
                            />
                        </Grid>
                    </Grid>

                    <Card>
                        <TableContainer sx={{ position: 'relative', overflow: 'unset', maxWidth: '1420px' }}>
                            <TableHead  >
                                <TableCell sx={{ width: '300px' }} align='center' >Nom</TableCell>
                                <TableCell sx={{ width: '300px', pl: '50px' }} >Activé</TableCell>
                                <TableCell sx={{ width: '520px' }} align='center'>Description</TableCell>
                                <TableCell sx={{ width: '300px' }} align='center' >Actions</TableCell>
                            </TableHead>

                            <Scrollbar>
                                <Table size={table.dense ? 'small' : 'medium'} sx={{ minWidth: 800 }}>
                                    <TableBody>
                                        <Reorder.Group
                                            axis="y" // Vertical reordering
                                            values={tableData.map((_, index) => index)}
                                            onReorder={handleReorder}
                                        >
                                            {tableData
                                                .slice(
                                                    table.page * table.rowsPerPage,
                                                    table.page * table.rowsPerPage + table.rowsPerPage
                                                )
                                                .map((row, index) => (
                                                    <Reorder.Item key={row.id} value={index}> {/* Use index as value for Reorder.Item */}
                                                        <PaymentMethodTableRow
                                                            key={row.id}
                                                            row={row}
                                                            onViewRow={() => handleViewRow(row.id)}
                                                        />
                                                    </Reorder.Item>
                                                ))}
                                        </Reorder.Group>
                                        <TableEmptyRows
                                            height={denseHeight}
                                            emptyRows={emptyRows(table.page, table.rowsPerPage, tableData.length)}
                                        />

                                        <TableNoData notFound={notFound} />
                                    </TableBody>
                                </Table>
                            </Scrollbar>
                        </TableContainer>
                        <TablePaginationCustom
                            count={tableData.length}
                            page={table.page}
                            rowsPerPage={table.rowsPerPage}
                            onPageChange={table.onChangePage}
                            onRowsPerPageChange={table.onChangeRowsPerPage}
                            dense={table.dense}
                            onChangeDense={table.onChangeDense}
                        />
                    </Card>
                </Container>
            </LocalizationProvider>
        </>

    );
}
