import { useCallback } from "react";

import Box from "@mui/material/Box";
import Chip from "@mui/material/Chip";
import Paper from "@mui/material/Paper";
import Button from "@mui/material/Button";
import Stack, { StackProps } from "@mui/material/Stack";

import Iconify from "@/shared/components/iconify";
import { IPromocodeFilter, IPromocodeFilterValue } from "@/contexts/types/promo";



type Props = StackProps & {
  filters: IPromocodeFilter;
  onFilters: (name: string, value: IPromocodeFilterValue) => void;
  
  onResetFilters: VoidFunction;

  results: number;
};

export default function CodePromoResult({
  filters,
  onFilters,
  
  onResetFilters,
  
  results,
  ...other
}: Props) {
  const handleRemoveCategory = useCallback(
    (inputValue: string) => {
      const newValue = filters.Category.filter((item) => item !== inputValue);

      onFilters("Category", newValue);
    },
    [filters.Category, onFilters]
  );



  const handleRemoveTypecode = useCallback(
    (inputValue: string) => {
      const newValue = filters.typecode.filter((item) => item !== inputValue);

      onFilters("typecode", newValue);
    },
    [filters.typecode, onFilters]
  );





  return (
    <Stack spacing={1.5} {...other}>
      <Box sx={{ typography: "body2" }}>
        <strong>{results}</strong>
        <Box component="span" sx={{ color: "text.secondary", ml: 0.25 }}>
          results found
        </Box>
      </Box>

      <Stack
        flexGrow={1}
        spacing={1}
        direction="row"
        flexWrap="wrap"
        alignItems="center"
      >
        {!!filters.Category.length && (
          <Block label="Category:">
            {filters.Category.map((item) => (
              <Chip
                key={item}
                label={item}
                size="small"
                onDelete={() => handleRemoveCategory(item)}
              />
            ))}

          </Block>
        )}



        {!!filters.typecode.length && (
          <Block label="Typecode:">
            {filters.typecode.map((item) => (
              <Chip
                key={item}
                label={item}
                size="small"
                onDelete={() => handleRemoveTypecode(item)}
              />
            ))}
          </Block>
        )}



        <Button
          color="error"
          onClick={onResetFilters}
          startIcon={<Iconify icon="solar:trash-bin-trash-bold" />}
        >
          Clear
        </Button>
      </Stack>
    </Stack>
  );
}

// ----------------------------------------------------------------------

type BlockProps = StackProps & {
  label: string;
};

function Block({ label, children, sx, ...other }: BlockProps) {
  return (
    <Stack
      component={Paper}
      variant="outlined"
      spacing={1}
      direction="row"
      sx={{
        p: 1,
        borderRadius: 1,
        overflow: "hidden",
        borderStyle: "dashed",
        ...sx,
      }}
      {...other}
    >
      <Box component="span" sx={{ typography: "subtitle2" }}>
        {label}
      </Box>

      <Stack spacing={1} direction="row" flexWrap="wrap">
        {children}
      </Stack>
    </Stack>
  );
}
