import { useState, useCallback } from "react";
import MenuItem from "@mui/material/MenuItem";
import Checkbox from "@mui/material/Checkbox";
import Box from "@mui/material/Box";
import InputLabel from "@mui/material/InputLabel";
import FormControl from "@mui/material/FormControl";
import OutlinedInput from "@mui/material/OutlinedInput";
import { DatePicker } from '@mui/x-date-pickers/DatePicker';
import Select, { SelectChangeEvent } from "@mui/material/Select";
import Iconify from "@/shared/components/iconify";
import { formHelperTextClasses } from '@mui/material/FormHelperText';
import CustomPopover, { usePopover } from "@/shared/components/custom-popover";
import { IReferralFilter, IReferralOrderFilterValue } from "@/contexts/types/Referral";


// ----------------------------------------------------------------------

type Props = {
  filters: IReferralFilter;
  onFilters: (name: string, value: IReferralOrderFilterValue) => void;
  dateError: boolean;
  //
  statusOptions: {
    value: string;
    label: string;
  }[];
};

export default function ReferralTableToolbar({
  filters,
  onFilters,
  //
  statusOptions,
  dateError
}: Props) {
  const popover = usePopover();

  const [status, setStatus] = useState<string[]>(filters.status);

  const handleChangeStatus = useCallback(
    (event: SelectChangeEvent<string[]>) => {
      const {
        target: { value },
      } = event;
      setStatus(typeof value === "string" ? value.split(",") : value);
    },
    []
  );

  const handleCloseStatus = useCallback(() => {
    onFilters("status", status);
  }, [onFilters, status]);

  const handleFilterStartDate = useCallback(
    (newValue: Date | null) => {
      onFilters('startDate', newValue);
    },
    [onFilters]
  );

  const handleFilterEndDate = useCallback(
    (newValue: Date | null) => {
      onFilters('endDate', newValue);
    },
    [onFilters]
  );

  const [anchorEl, setAnchorEl] = useState<HTMLElement | null>(null);

  const handleClick = (event: React.MouseEvent<HTMLButtonElement>) => {
    setAnchorEl(event.currentTarget);
  };

  const handleClose = () => {
    setAnchorEl(null);
  };

  return (
    <>
      <Box>

          <FormControl
            sx={{
              flexShrink: 0,
              width: { xs: 1, md: 220 },
              mr: 3,
              
        
            }}
          >
            <InputLabel>Status</InputLabel>

            <Select
              multiple
              value={status}
              onChange={handleChangeStatus}
              input={<OutlinedInput label="Status" />}
              renderValue={(selected) => selected.map((value) => value).join(", ")}
              onClose={handleCloseStatus}
              sx={{ textTransform: "capitalize" }}
            >
              {statusOptions.map((option) => (
                <MenuItem key={option.value} value={option.value}>
                  <Checkbox
                    disableRipple
                    size="small"
                    checked={status.includes(option.value)}
                  />
                  {option.label}
                </MenuItem>
              ))}
            </Select>
          </FormControl>
          <DatePicker
            label="Date début"
            value={filters.startDate}
            onChange={handleFilterStartDate}
            slotProps={{
              textField: {
                fullWidth: true,
              },
            }}
            sx={{
              maxWidth: { md: 220 },
              mr: 3,
            }}
          />
          <DatePicker
            label="Date fin"
            value={filters.endDate}
            onChange={handleFilterEndDate}
            slotProps={{
              textField: {
                fullWidth: true,
                error: dateError,
                helperText: dateError && 'End date must be later than start date',
              },
            }}
            sx={{
              maxWidth: { md: 220 },
              [`& .${formHelperTextClasses.root}`]: {
                position: { md: 'absolute' },
                bottom: { md: -40 },
              },
            }}
          />

      </Box>
      <CustomPopover
        open={popover.open}
        onClose={popover.onClose}
        arrow="right-top"
        sx={{ width: 140 }}
      >
        <MenuItem
          onClick={() => {
            popover.onClose();
          }}
        >
          <Iconify icon="solar:printer-minimalistic-bold" />
          Imprimer
        </MenuItem>

        <MenuItem
          onClick={() => {
            popover.onClose();
          }}
        >
          <Iconify icon="solar:import-bold" />
          Importer
        </MenuItem>

        <MenuItem
          onClick={() => {
            popover.onClose();
          }}
        >
          <Iconify icon="solar:export-bold" />
          Exporter
        </MenuItem>

      </CustomPopover>
    </>
  );
}
