import React, { useState } from 'react';
import Box from '@mui/material/Box';
import Paper from '@mui/material/Paper';
import Stack from '@mui/material/Stack';
import Button from '@mui/material/Button';
import Collapse from '@mui/material/Collapse';
import MenuItem from '@mui/material/MenuItem';
import TableRow from '@mui/material/TableRow';
import Checkbox from '@mui/material/Checkbox';
import TableCell from '@mui/material/TableCell';
import IconButton from '@mui/material/IconButton';
import ListItemText from '@mui/material/ListItemText';

import { useBoolean } from '../../../../hooks/use-boolean';

import { fDate, fTime } from '../../../../utils/format-time';

import Label from '../../../components/label';
import Iconify from '../../../components/iconify';
import { ConfirmDialog } from '../../../components/custom-dialog';
import CustomPopover, { usePopover } from '../../../components/custom-popover';

import { IReviewItem, IReviewStatus } from '../../../types/review';
import {IProductReview}from '../../../types/product';
import { TextField } from '@mui/material';

// ----------------------------------------------------------------------

type Props = {
  row: IReviewItem;
  selected: boolean;
  onViewRow: VoidFunction;
  onSelectRow: VoidFunction;
  onDeleteRow: VoidFunction;
  onEditRow: VoidFunction;
};

export default function ReviewTableRow({
  row,
  selected,
  onViewRow,
  onSelectRow,  
  onDeleteRow,
  onEditRow,
}: Props) {
  const { status: initialStatus, orderNumber, createdAt, customer,  commentaire, items} = row;

  const [status, setStatus] = useState(initialStatus);
  const confirm = useBoolean();

  const [showReplySection, setShowReplySection] = useState(false);

  const popover = usePopover();

  const handleReplyClick = () => {
    setShowReplySection(!showReplySection);
    popover.onClose();
  };

  const handleApprove = () => {
    setStatus(IReviewStatus.Ongoing); // Set to 'Approuvé'
    popover.onClose();
  };

  const handleDisapprove = () => {
    setStatus(IReviewStatus.Cancelled); // Set to 'Désapprouver'
    popover.onClose();
  };

  const renderPrimary = (
    <TableRow hover selected={selected}>
      <TableCell padding="checkbox">
        <Checkbox checked={selected} onClick={onSelectRow} />
      </TableCell>

      <TableCell>
        <Box
          onClick={onViewRow}
          sx={{
            cursor: 'pointer',
            '&:hover': {
              textDecoration: 'underline',
            },
          }}
        >
          {orderNumber}
        </Box>
      </TableCell>
      <TableCell > {commentaire} </TableCell>
      <TableCell sx={{ display: 'flex', alignItems: 'center' }}>
        <ListItemText
          primary={customer.name}
         
          primaryTypographyProps={{ typography: 'body2' }}
          secondaryTypographyProps={{
            component: 'span',
            color: 'text.disabled',
          }}
        />
      </TableCell>
      <TableCell > {items.name}</TableCell>
      <TableCell>
        <ListItemText
          primary={fDate(createdAt)}
          secondary={fTime(createdAt)}
          primaryTypographyProps={{ typography: 'body2', noWrap: true }}
          secondaryTypographyProps={{
            mt: 0.5,
            component: 'span',
            typography: 'caption',
          }}
        />
      </TableCell>

    
      <TableCell>
        <Label
          variant="soft"
          color={
            (status === IReviewStatus.Ongoing && 'success') ||
            (status === IReviewStatus.Pending && 'warning') ||
            (status === IReviewStatus.Cancelled && 'error') ||
             'default'
          }
        >
          {status}
        </Label>
      </TableCell>

      <TableCell align="right" sx={{ px: 1, whiteSpace: 'nowrap' }}>
        <IconButton color={popover.open ? 'inherit' : 'default'} onClick={popover.onOpen}>
          <Iconify icon="eva:more-vertical-fill" />
        </IconButton>
      </TableCell>
    </TableRow>
  );

  const renderSecondary = (
    <TableRow>
      <TableCell sx={{ p: 1, border: 'none' }} colSpan={9}>
        <Collapse
          in={showReplySection}
          timeout="auto"
          unmountOnExit
          sx={{ bgcolor: 'background.neutral' }}
        >
          <Stack component={Paper} sx={{ m: 1.5, padding: 2 }}>
            <Box mb={3}>
              <h5 color="grey.700">Répondre au commentaire</h5>

              <TextField
                rows={8}
                multiline
                fullWidth
                name="comment"
                variant="outlined"
                placeholder=" Écrire une réponse..."
              />
            </Box>

            <Stack
              direction="row"
              alignItems="center"
              spacing={2}
              flexGrow={1}
              sx={{ width: 0.6, margin: 1 }}
            >
              <Button variant="contained" sx={{ width: 0.2 }}>
                Répondre
              </Button>
              <Button variant="contained" sx={{ width: 0.2 }} onClick={() => setShowReplySection(false)}>
                Annuler
              </Button>
            </Stack>
          </Stack>
        </Collapse>
      </TableCell>
    </TableRow>
  );

  return (
    <>
      {renderPrimary}

      {renderSecondary}

      <CustomPopover
        open={popover.open}
        onClose={popover.onClose}
        arrow="right-top"
        sx={{ width: 200 }}
      >
        <MenuItem
          onClick={() => {
            confirm.onTrue();
            popover.onClose();
          }}
          sx={{ color: 'error.main' }}
        >
          <Iconify icon="solar:trash-bin-trash-bold" />
          Mettre à la corbeille
        </MenuItem>

        <MenuItem onClick={handleApprove}>
          <Iconify icon="openmoji:check-mark" />
          
          Approuver
        </MenuItem>

        <MenuItem onClick={handleDisapprove}>
          <Iconify icon="vaadin:close-small"  />
          
          Désapprouver
        </MenuItem>

        <MenuItem onClick={handleReplyClick}>
          <Iconify icon="solar:pen-bold" />
          Répondre
        </MenuItem>
      </CustomPopover>

      <ConfirmDialog
        open={confirm.value}
        onClose={confirm.onFalse}
        title="Mettre à la corbeille"
        content="Etes-vous sûr de vouloir le mettre à la corbeille?"
        action={
          <Button variant="contained" color="error" onClick={onDeleteRow}>
            Mettre à la corbeille
          </Button>
        }
      />
    </>
  );
}
