'use client';

import { useCallback } from 'react';
import Table from '@mui/material/Table';
import Button from '@mui/material/Button';
import Tooltip from '@mui/material/Tooltip';
import Container from '@mui/material/Container';
import TableBody from '@mui/material/TableBody';
import TableCell from '@mui/material/TableCell';
import Grid from '@mui/material/Grid';
import TableHead from '@mui/material/TableHead';
import IconButton from '@mui/material/IconButton';
import { useTheme } from '@mui/material/styles';
import TableContainer from '@mui/material/TableContainer';
import LocalizationProvider from '@/shared/locales/localization-provider';
import { useBoolean } from '@/hooks';
import { shippingAreaList } from '@/shared/_mock/_shipping';
import Iconify from '@/shared/components/iconify';
import Scrollbar from '@/shared/components/scrollbar';
import { useSettingsContext } from '@/shared/components/settings';
import CustomBreadcrumbs from '@/components/custom-breadcrumbs';
import { useTable, emptyRows, TableNoData, TableSelectedAction, TablePaginationCustom, TableEmptyRows, } from '@/shared/components/table';
import ShippingAreaTableRow from './ShippingAreaTableRow';
import { Reorder } from 'framer-motion';
import { Box, InputAdornment, Modal, TextField, Typography } from '@mui/material';
import { useEffect, useState } from 'react';
import * as Yup from 'yup';
import { useForm } from 'react-hook-form';
import { yupResolver } from '@hookform/resolvers/yup';
import { useMemo } from 'react';
import Chip from '@mui/material/Chip';
import Card from '@mui/material/Card';
import Stack from '@mui/material/Stack';
import { useSnackbar } from '@/components/snackbar';
import FormProvider, {
  RHFAutocomplete,
} from '@/components/hook-form';
import { paths } from '@/routes/paths';
import { IShippingAreaItem, ShippingMethod } from '@/contexts/types/Shipping';
import { shippingMethodsList } from '@/shared/_mock/_shipping'
import { countries } from '@/shared/assets/data/countries'
import { useRouter } from '@/hooks';


export default function ShippingAreaTableView() {
  const { enqueueSnackbar } = useSnackbar();

  const theme = useTheme();

  const settings = useSettingsContext();
  const router = useRouter();
  const table = useTable({ defaultOrderBy: 'title' });
  const confirm = useBoolean();
  const [tableData, setTableData] = useState<IShippingAreaItem[]>(shippingAreaList);

  const [shippingMethodData, setShippingMethodData] = useState<ShippingMethod[]>(shippingMethodsList);

  const dataInPage = tableData.slice(
    table.page * table.rowsPerPage,
    table.page * table.rowsPerPage + table.rowsPerPage
  );

  const denseHeight = table.dense ? 56 : 56 + 20;


  const notFound = (!tableData.length) || !tableData.length;

  //----------------------------------------------------------------------------------  

  const [openAddModal, setOpenAddModal] = useState(false);
  const [openEditModal, setOpenEditModal] = useState(false);


  const handleAddClick = () => {
    methods.reset({ region: [] });
    reset();
    setOpenAddModal(true);
  };

  const handleEditClick = (id: string) => {
    const shippingAreaEdited: IShippingAreaItem | undefined = tableData.find((a) => a.id === id);
    if (shippingAreaEdited) {
      setShippingAreaEditing(shippingAreaEdited);
    }
    methods.reset(defaultValues);
    setOpenEditModal(true);
  };

  const [shippingAreaEditing, setShippingAreaEditing] = useState<IShippingAreaItem>();

  const NewShippingAreaSchema = Yup.object().shape({
    name: Yup.string().required('Nom est requis'),
    region: Yup.array().required('Région est requis'),
    shippingMethods: Yup.array().required("Méthode de livraison est requis")
  });

  const defaultValues = useMemo(() => ({
    name: shippingAreaEditing?.name || '',
    region: shippingAreaEditing?.region || [],
    shippingMethods: shippingAreaEditing?.shippingMethodList || []
  }), [shippingAreaEditing]);

  const methods = useForm({
    resolver: yupResolver(NewShippingAreaSchema),
    defaultValues,
  });

  const {
    reset,
    handleSubmit,
    formState: { isSubmitting },
  } = methods;

  useEffect(() => {
    if (shippingAreaEditing) {
      reset(defaultValues);
    }
    console.log(defaultValues);
  }, [shippingAreaEditing, defaultValues, reset])

  const onSubmit = handleSubmit(async (data) => {
    try {
      await new Promise((resolve) => setTimeout(resolve, 500));
      reset();
      enqueueSnackbar('Success!', { variant: 'success' });
      console.info('Form Data:', data);
    } catch (error) {
      console.error(error);
      enqueueSnackbar('Failed', { variant: 'error' });
    }
  });

  const handleCloseEditModal = () => {
    setOpenEditModal(false);
    setSelectedShippingArea(null);
    reset();
  };

  const handleCloseAddModal = () => {
    setOpenAddModal(false);
    reset();
  };

  const handleAddFormSubmit = () => {
    setOpenAddModal(false);
  };

  const handleEditFormSubmit = () => {
    setOpenEditModal(false);
  };

  //------------------------------
  const handleDeleteRow = useCallback(
    (id: string) => {
      const deleteRow = tableData.filter((row) => row.id !== id);

      enqueueSnackbar('Delete success!');

      setTableData(deleteRow);

      table.onUpdatePageDeleteRow(dataInPage.length);
    },
    [dataInPage.length, enqueueSnackbar, table, tableData]
  );

  //----------------view details--------------

  const [selectedShippingArea, setSelectedShippingArea] = useState<IShippingAreaItem | null>(null);
  const handelSelectedShippingArea = (shippingArea: any) => {
    setSelectedShippingArea(shippingArea);
  };
  const handleViewRow = useCallback(
    (id: string) => {
      const shippingArea = tableData.find((a) => a.id === id);
      handelSelectedShippingArea(shippingArea)
    },
    [tableData]
  );

  //----------------Change ordre of row---------------
  const handleReorder = (newOrder: number[]) => {
    // Create a new array with the shipping areas reordered based on the newOrder array
    const reorderedData = newOrder.map(index => tableData[index]);
    setTableData(reorderedData);
  };

  //-----------tab-------------------
  const [value, setValue] = useState('1');

  const handleChange = (event: React.SyntheticEvent, newValue: string) => {
    setValue(newValue);
  };

  /////////////////////////////

  return (
    <>
      <LocalizationProvider >
        <Container maxWidth={settings.themeStretch ? false : 'lg'}>
          <Grid container alignItems="center" justifyContent="space-between">
            <Grid item>
              <CustomBreadcrumbs
                heading="Zones d'expédition"
                links={[
                  {
                    name: 'Dashboard',
                    href: paths.dashboard.root,
                  },
                  {
                    name: "Livraison",
                  }
                ]}
                sx={{
                  mb: { xs: 3, md: 5 },
                }}
              />
            </Grid>
            <Grid item>
              <Stack
                spacing={3}
                justifyContent="space-between"
                alignItems={{ xs: 'flex-end', sm: 'center' }}
                direction={{ xs: 'column', sm: 'row' }}
              >
                <Button
                  variant="contained"
                  color='success'
                  startIcon={<Iconify icon="mingcute:add-line" />}
                  onClick={handleAddClick}
                >
                  Ajouter
                </Button>
              </Stack>
            </Grid>
          </Grid>

          <Card>
            <TableContainer sx={{ position: 'relative', overflow: 'unset' }}>
              <TableSelectedAction
                dense={table.dense}
                numSelected={table.selected.length}
                rowCount={tableData.length}
                onSelectAllRows={(checked) => {
                  table.onSelectAllRows(
                    checked,
                    tableData.map((row) => row.id)
                  );
                }}
                action={
                  <Stack direction="row">
                    <Tooltip title="Archive">
                      <IconButton color="primary" onClick={confirm.onTrue}>
                        <Iconify icon="clarity:archive-solid" />

                      </IconButton>
                    </Tooltip>

                    <Tooltip title="Delete">
                      <IconButton color="primary" onClick={confirm.onTrue}>
                        <Iconify icon="solar:trash-bin-trash-bold" />
                      </IconButton>
                    </Tooltip>
                  </Stack>
                }
              />
              <TableHead sx={{ width: '100%' }} >
                <TableCell width={'30%'}>Nom</TableCell>
                <TableCell width={'30%'}>Region</TableCell>
                <TableCell width={'33%'}>Méthodes de livraison</TableCell>
                <TableCell width={'7%'}>Actions</TableCell>
              </TableHead>

              <Scrollbar>
                <Table size={table.dense ? 'small' : 'medium'} sx={{ minWidth: 800 }}>
                  <TableBody>
                    <Reorder.Group
                      axis="y" // Vertical reordering
                      values={tableData.map((_, index) => index)} // Array of indices representing the current order
                      onReorder={handleReorder} // Callback function to handle reordering
                    >
                      {tableData
                        .slice(
                          table.page * table.rowsPerPage,
                          table.page * table.rowsPerPage + table.rowsPerPage
                        )
                        .map((row, index) => (
                          <Reorder.Item key={row.id} value={index}> {/* Use index as value for Reorder.Item */}
                            <ShippingAreaTableRow
                              key={row.id}
                              row={row}
                              selected={table.selected.includes(row.id)}
                              onSelectRow={() => table.onSelectRow(row.id)}
                              onViewRow={() => handleViewRow(row.id)}
                              selectedShippingArea={selectedShippingArea}
                              onEditRow={() => handleEditClick(row.id)}
                              onDeleteRow={() => handleDeleteRow(row.id)}
                            />
                          </Reorder.Item>
                        ))}
                    </Reorder.Group>


                    <TableEmptyRows
                      height={denseHeight}
                      emptyRows={emptyRows(table.page, table.rowsPerPage, tableData.length)}
                    />

                    <TableNoData notFound={notFound} />
                  </TableBody>



                </Table>
              </Scrollbar>
            </TableContainer>

            <TablePaginationCustom
              count={tableData.length}
              page={table.page}
              rowsPerPage={table.rowsPerPage}
              onPageChange={table.onChangePage}
              onRowsPerPageChange={table.onChangeRowsPerPage}
              dense={table.dense}
              onChangeDense={table.onChangeDense}
            />
          </Card>
        </Container>
      </LocalizationProvider>

      <Modal open={openAddModal} onClose={handleCloseAddModal}>
        <FormProvider methods={methods} onSubmit={onSubmit}>
          <Box sx={{ position: 'absolute', top: '50%', left: '50%', transform: 'translate(-50%, -50%)', width: 500, bgcolor: 'background.paper', boxShadow: 24, p: 4, borderRadius: 3 }}>
            <Typography variant="h6" align="center" gutterBottom sx={{ mt: 2, mb: 3 }}>Ajouter une zone d'expédition</Typography>
            <TextField
              name='name'
              label="Nom de la zone"
              type="text"
              sx={{ m: 1, width: '97%' }}
            />
            <RHFAutocomplete
              name="region"
              label="Région"
              placeholder="+ Région"
              defaultValue={[]}
              sx={{ ml: 1, mr: 1, mb: 1, width: '97%' }}
              multiple
              freeSolo
              disableCloseOnSelect
              options={countries.map(country => country.label)}
              getOptionLabel={(option) => option}
              renderOption={(props, option) => (
                <li {...props} key={option}>
                  {option}
                </li>
              )}
              renderTags={(selected, getTagProps) =>
                selected.map((option, index) => (
                  <Chip
                    {...getTagProps({ index })}
                    key={option}
                    label={option}
                    size="small"
                    color="success"
                    variant="soft"
                  />
                ))
              }
            />
            <RHFAutocomplete
              name="shippingMethodList"
              label="Méthodes de livraison"
              placeholder="+ Méthodes de livraison"
              defaultValue={[]}
              sx={{ ml: 1, mr: 1, width: '97%' }}
              multiple
              freeSolo
              disableCloseOnSelect
              options={shippingMethodData.map((option) => option)}
              getOptionLabel={(option) => typeof option === 'string' ? option : option.name}
              renderOption={(props, option) => (
                <li {...props} key={option.id}>
                  {option.name}
                </li>
              )}
              renderTags={(selected, getTagProps) =>
                selected.map((option, index) => (
                  <Chip
                    {...getTagProps({ index })}
                    key={option.id}
                    label={option.name}
                    size="small"
                    color="success"
                    variant="soft"
                  />
                ))
              }
            />
            <Box sx={{ display: 'flex', justifyContent: 'center', mt: 3 }}>
              <Button onClick={() => setOpenAddModal(false)} sx={{ mt: 2, mb: 2, mr: 1 }} variant="outlined" color="inherit">Annuler</Button>
              <Button onClick={handleAddFormSubmit} sx={{ mt: 2, mb: 2 }} variant="contained" color="inherit">Ajouter</Button>
            </Box>
          </Box>
        </FormProvider>
      </Modal>

      <Modal open={openEditModal} onClose={handleCloseEditModal}>
        <FormProvider methods={methods} onSubmit={onSubmit}>
          <Box sx={{ position: 'absolute', top: '50%', left: '50%', transform: 'translate(-50%, -50%)', width: 500, bgcolor: 'background.paper', boxShadow: 24, p: 4, borderRadius: 3 }}>
            <Typography variant="h6" align="center" gutterBottom sx={{ mt: 2, mb: 3 }}>Modifier une zone d'expédition</Typography>
            <TextField
              name='name'
              label="Nom de la zone"
              type="text"
              defaultValue={shippingAreaEditing?.name}
              sx={{ m: 1, width: '97%' }}
            />
            <RHFAutocomplete
              name="region"
              label="Région"
              placeholder="+ Région"
              defaultValue={shippingAreaEditing?.region}
              sx={{ ml: 1, mr: 1, mb: 1, width: '97%' }}
              multiple
              freeSolo
              disableCloseOnSelect
              options={countries.map(country => country.label)}
              getOptionLabel={(option) => option}
              renderOption={(props, option) => (
                <li {...props} key={option}>
                  {option}
                </li>
              )}
              renderTags={(selected, getTagProps) =>
                selected.map((option, index) => (
                  <Chip
                    {...getTagProps({ index })}
                    key={option}
                    label={option}
                    size="small"
                    color="success"
                    variant="soft"
                  />
                ))
              }
            />
            <RHFAutocomplete
              name="shippingMethodList"
              label="Méthodes de livraison"
              placeholder="+ Méthodes de livraison"
              defaultValue={shippingAreaEditing?.shippingMethodList}
              sx={{ ml: 1, mr: 1, width: '97%' }}
              multiple
              freeSolo
              disableCloseOnSelect
              options={shippingMethodData.map((option) => option)}
              getOptionLabel={(option) => typeof option === 'string' ? option : option.name}
              renderOption={(props, option) => (
                <li {...props} key={option.id}>
                  {option.name}
                </li>
              )}
              renderTags={(selected, getTagProps) =>
                selected.map((option, index) => (
                  <Chip
                    {...getTagProps({ index })}
                    key={option.id}
                    label={option.name}
                    size="small"
                    color="success"
                    variant="soft"
                  />
                ))
              }
            />
            <Box sx={{ display: 'flex', justifyContent: 'center', mt: 3 }}>
              <Button onClick={() => setOpenEditModal(false)} sx={{ mt: 2, mb: 2, mr: 1 }} variant="outlined" color="inherit">Annuler</Button>
              <Button onClick={handleEditFormSubmit} sx={{ mt: 2, mb: 2 }} variant="contained" color="inherit">Modifier</Button>
            </Box>
          </Box>
        </FormProvider>
      </Modal>
    </>
  );
}
