"use client"

import { useState } from 'react';
import * as Yup from 'yup';
import { useForm } from 'react-hook-form';
import { yupResolver } from '@hookform/resolvers/yup';
import { useMemo } from 'react';
import Chip from '@mui/material/Chip';
import Grid from '@mui/material/Unstable_Grid2';
import Card from '@mui/material/Card';
import Stack from '@mui/material/Stack';
import CardHeader from '@mui/material/CardHeader';
import Typography from '@mui/material/Typography';
import Switch from '@mui/material/Switch';
import { AutocompleteChangeReason } from '@mui/material/Autocomplete';
import { useResponsive } from '@/hooks/use-responsive';
import { useSnackbar } from '@/components/snackbar';
import FormProvider, {
    RHFTextField,
    RHFAutocomplete,
} from '@/components/hook-form';
import { paths } from '@/routes/paths';
import LoadingButton from '@mui/lab/LoadingButton';
import { ShippingMethodType } from '@/contexts/types/Shipping'
import DetailsFreeShipping from './DetailsFreeShipping';
import DetailsFlatRate from './DetailsFlatRate';
import DetailsMondialRelay from './DetailsMondialRelay.tsx';
import DetailsPointOfSale from './DetailsPointOfSale';
import CustomBreadcrumbs from '@/components/custom-breadcrumbs/custom-breadcrumbs';



export default function CreateShippingMethodView() {
    const shippingMethodTypes = Object.values(ShippingMethodType);

    const mdUp = useResponsive('up', 'md');

    const { enqueueSnackbar } = useSnackbar();

    const NewShippingMethodSchema = Yup.object().shape({
        title: Yup.string().required('Title is required'),
        description: Yup.string().required('Description is required'),
        active: Yup.boolean(),
        shippingMethodType: Yup.string().oneOf(Object.values(ShippingMethodType)).required(),
    });


    const defaultValues = useMemo(() => ({
        title: '',
        description: '',
        active: true,
        shippingMethodType: ShippingMethodType.FreeShipping
    }), []);

    const methods = useForm({
        resolver: yupResolver(NewShippingMethodSchema),
        defaultValues,
    });


    const {
        watch,
        control,
        setValue,
        handleSubmit,
        formState: { isSubmitting },
    } = methods;

    const values = watch();

    const onSubmit = handleSubmit(async (data) => {
        try {
            await new Promise((resolve) => setTimeout(resolve, 500));
            enqueueSnackbar('Create success!', { variant: 'success' });
            console.info('Form Data:', data);
        } catch (error) {
            console.error(error);
            enqueueSnackbar('Failed to create this new shipping method.', { variant: 'error' });
        }
    });

    const [selectedType, setSelectedType] = useState<string>(ShippingMethodType.FreeShipping);

    const handleChange = (
        event: React.SyntheticEvent,
        value: string | string[],
        reason: AutocompleteChangeReason
    ) => {
        const newValue = Array.isArray(value) ? value[0] : value;

        if (shippingMethodTypes.includes(newValue as ShippingMethodType)) {
            const validatedValue = newValue as ShippingMethodType;
            setSelectedType(validatedValue);
            setValue('shippingMethodType', validatedValue);
            console.log('Type sélectionné :', validatedValue);
        } else {
            console.error('Invalid shipping method type:', newValue);
        }
    };

    const renderBasic = (
        <>
            <Grid md={12}>
                <CustomBreadcrumbs
                    heading="Ajouter une méthode de livraison"
                    links={[
                        { name: 'Dashboard', href: paths.dashboard.root, },
                        { name: "Méthodes de livraison", href: paths.dashboard.shipping.general.shippingMethod.shippingMethodList, },
                        { name: "Nouvelle méthode de livraison" },
                    ]}
                    sx={{
                        mb: { xs: 3, md: 5 },
                    }}
                />
            </Grid>
            {mdUp && (
                <Grid md={4}>
                    <Typography variant="body2" sx={{ color: 'text.secondary' }}>
                    Saisissez les informations de la nouvelle méthode de livraison : nom de la méthode, son type et d'autres informations sa dépendent de chaque type.                    </Typography>
                </Grid>
            )}

            <Grid xs={12} md={8}>
                <Card>
                    {!mdUp && <CardHeader title="Nouvelle méthode de livraison" />}

                    <Stack spacing={3} sx={{ p: 3 }}>
                        <RHFTextField name="name" label="Nom de la méthode" />

                        <RHFTextField name="description" label="Description" placeholder="Description..." multiline rows={4} />
                        <div style={{ display: 'flex', alignItems: 'center' }}><label>Activé</label> <Switch name='active' inputProps={{ 'aria-label': 'controlled' }} /></div>
                        <RHFAutocomplete
                            name="shippingMethodType"
                            label="Type du méthode"
                            placeholder="Type du méthode"
                            disableClearable
                            options={shippingMethodTypes}
                            getOptionLabel={(option) => option}
                            onChange={handleChange}
                            value={selectedType}
                            renderOption={(props, option) => (
                                <li {...props} key={option}>
                                    {option}
                                </li>
                            )}
                            renderTags={(selected, getTagProps) =>
                                selected.map((option, index) => (
                                    <Chip
                                        {...getTagProps({ index })}
                                        key={option}
                                        label={option}
                                        size="small"
                                        color="success"
                                        variant="soft"
                                    />
                                ))
                            }
                        />
                    </Stack>
                </Card>
            </Grid>
        </>
    );

    const renderActions = (
        <>
            {mdUp && <Grid md={4} />}
            <Grid xs={12} md={8} sx={{ display: 'flex', justifyContent: 'end', py: 4 }}>
                <LoadingButton
                    type="submit"
                    variant="contained"
                    size="large"
                    loading={isSubmitting}
                    sx={{ ml: 2 }}
                >Enregistrer</LoadingButton>
            </Grid>
        </>
    );



    const renderDetails = () => {
        switch (selectedType) {
            case ShippingMethodType.FreeShipping:
                return <><DetailsFreeShipping />{renderActions}</>;
            case ShippingMethodType.FlatRate:
                return <><DetailsFlatRate />{renderActions}</>;
            case ShippingMethodType.MondialRelay:
                return <><DetailsMondialRelay />{renderActions}</>;
            case ShippingMethodType.ColissimoWithoutSignature:
                return renderActions;
            case ShippingMethodType.ColissimoWithSignature:
                return renderActions;
            case ShippingMethodType.PointOfSale:
                return <><DetailsPointOfSale /> {renderActions}</>;
        }
    };


    return (
        <FormProvider methods={methods} onSubmit={onSubmit}>
            <Grid container spacing={3}>
                {renderBasic}
                {renderDetails()}
            </Grid>
        </FormProvider>
    );
}
