"use client"

import { useState } from 'react';
import * as Yup from 'yup';
import { useForm } from 'react-hook-form';
import { yupResolver } from '@hookform/resolvers/yup';
import { useMemo } from 'react';
import Grid from '@mui/material/Unstable_Grid2';
import Card from '@mui/material/Card';
import Stack from '@mui/material/Stack';
import Chip from '@mui/material/Chip';
import Button from '@mui/material/Button';
import CardHeader from '@mui/material/CardHeader';
import Typography from '@mui/material/Typography';
import Switch from '@mui/material/Switch';
import Box from '@mui/material/Box';
import DeleteIcon from '@mui/icons-material/Delete';
import InputAdornment from '@mui/material/InputAdornment';
import Table from '@mui/material/Table';
import TableBody from '@mui/material/TableBody';
import EditIcon from '@mui/icons-material/Edit';
import TableCell from '@mui/material/TableCell';
import AddIcon from '@mui/icons-material/Add';
import TableContainer from '@mui/material/TableContainer';
import TableHead from '@mui/material/TableHead';
import TableRow from '@mui/material/TableRow';
import Paper from '@mui/material/Paper';
import GetAppIcon from '@mui/icons-material/GetApp';
import PublishIcon from '@mui/icons-material/Publish';
import Input from '@mui/material/Input';
import { useResponsive } from '@/hooks/use-responsive';
import { useSnackbar } from '@/components/snackbar';
import FormProvider, {
    RHFTextField,
    RHFAutocomplete,
} from '@/components/hook-form';
import { paths } from '@/routes/paths';
import LoadingButton from '@mui/lab/LoadingButton';
import { CalculationType, MondialRelayMode, ShippingClass, ShippingMethod, ShippingMethodType } from '@/contexts/types/Shipping'
import { shippingClassesList, shippingMethodsList } from '@/shared/_mock/_shipping';
import { Checkbox, Container, FormLabel, IconButton, Modal, TablePagination, Tooltip } from '@mui/material';
import HelpOutlineIcon from '@mui/icons-material/HelpOutline';
import CustomBreadcrumbs from '@/shared/components/custom-breadcrumbs/custom-breadcrumbs';
import ConfirmDialog from '@/shared/components/confirm-dialog';
import { useBoolean } from '@/hooks';

type Props = {
    id: String;
}

interface RowDataCost {
    fromWeight: number | null;
    toWeight: number | null;
    fromCartPrice: number | null;
    toCartPrice: number | null;
    shippingClassLinked: ShippingClass | null;
    cost: number | null;
}

interface RowDataReduction {
    nbrProducts: number | null;
    percentageReduction: number | null;
}

export default function EditColissimoWithoutSignature({ id }: Props) {
    const shippingMethodTypes = Object.values(ShippingMethodType);

    const [shippingClassList, setShippingClassList] = useState<ShippingClass[]>(shippingClassesList);

    const shippingMethod: ShippingMethod | undefined = shippingMethodsList.find(method => method.id === id);

    const mdUp = useResponsive('up', 'md');

    const { enqueueSnackbar } = useSnackbar();

    const NewShippingMethodSchema = Yup.object().shape({
        name: Yup.string().required('Name is required'),
        description: Yup.string().required('Description is required'),
    });

    interface DefaultValues {
        name: string;
        description: string;
        additionalAttributes?: {
            [key: string]: any;
        };
    }

    const defaultValues: DefaultValues = useMemo(() => {
        let additionalAttributes: any = {};

        switch (shippingMethod?.type) {
            case ShippingMethodType.FreeShipping:
                additionalAttributes = {
                    requires: shippingMethod.data.requires || '',
                    minimumOrderAmount: shippingMethod.data.minimumOrderAmount || 0,
                    applyMinOrderRuleBeforePromoCodeDiscount: shippingMethod.data.applyMinOrderRuleBeforePromoCodeDiscount || ''
                };
                break;
            case ShippingMethodType.FlatRate:
                additionalAttributes = {
                    tvaStatut: shippingMethod.data.tvaStatut || false,
                    cost: shippingMethod.data.cost || 0,
                    costHeavyShippingClass: shippingMethod.data.costHeavyShippingClass || 0,
                    CostNoShippingClass: shippingMethod.data.CostNoShippingClass || 0,
                    calculationType: shippingMethod.data.calculationType || CalculationType.ByClass
                };
                break;
            case ShippingMethodType.MondialRelay:
                additionalAttributes = {
                    amountFreeShipping: shippingMethod.data.amountFreeShipping || 0,
                    nameFreeShipping: shippingMethod.data.nameFreeShipping || '',
                    modeMondialRelay: shippingMethod.data.modeMondialRelay || MondialRelayMode.MED,
                    listCostDelivery: shippingMethod.data.listCostDelivery || []
                };
                break;
            default:
                break;
        }

        return {
            name: shippingMethod?.name || '',
            description: shippingMethod?.description || '',
            type: shippingMethod?.type || '',
            additionalAttributes: additionalAttributes
        };
    }, [shippingMethod]);

    const methods = useForm({
        resolver: yupResolver(NewShippingMethodSchema),
        defaultValues,
    });

    const {
        watch,
        control,
        setValue,
        handleSubmit,
        formState: { isSubmitting },
    } = methods;

    const values = watch();

    const onSubmit = handleSubmit(async (data) => {
        try {
            await new Promise((resolve) => setTimeout(resolve, 500));
            enqueueSnackbar('Create success!', { variant: 'success' });
            console.info('Form Data:', data);
        } catch (error) {
            console.error(error);
            enqueueSnackbar('Failed to create this new shipping method.', { variant: 'error' });
        }
    });

    const [selectedType, setSelectedType] = useState<string | null>(null);

    const mondialRelayMode = Object.values(MondialRelayMode);
    const calculationType = Object.values(CalculationType);

    const [checkedStillFree, setCheckedStillFree] = useState(true);

    const [checkedConditionFree, setCheckedConditionFree] = useState(true);

    const handleChangeStillFree = (event: React.ChangeEvent<HTMLInputElement>) => {
        setCheckedStillFree(event.target.checked);
    };

    const handleChangeConditionFree = (event: React.ChangeEvent<HTMLInputElement>) => {
        setCheckedConditionFree(event.target.checked);
    };

    const [dataListCostDelivery, setDataListCostDelivery] = useState<RowDataCost[]>([
        { fromWeight: 2, toWeight: 5, fromCartPrice: 50.0, toCartPrice: 150, shippingClassLinked: shippingClassList[0], cost: 15 },
        { fromWeight: 1, toWeight: 3, fromCartPrice: 80.0, toCartPrice: 250, shippingClassLinked: shippingClassList[1], cost: 18 },
        { fromWeight: 4, toWeight: 7, fromCartPrice: 120.0, toCartPrice: 300, shippingClassLinked: shippingClassList[2], cost: 22 },
        { fromWeight: 3, toWeight: 6, fromCartPrice: 70.0, toCartPrice: 200, shippingClassLinked: shippingClassList[3], cost: 17 },
        { fromWeight: 2.5, toWeight: 4.5, fromCartPrice: 90.0, toCartPrice: 280, shippingClassLinked: shippingClassList[4], cost: 19 },
        { fromWeight: 3.7, toWeight: 8, fromCartPrice: 100.0, toCartPrice: 350, shippingClassLinked: shippingClassList[5], cost: 25 }
    ]);

    const [editIndexCost, setEditIndexCost] = useState<number | null>(null);
    const [formDataCost, setFormDataCost] = useState<RowDataCost>({
        fromWeight: null as number | null,
        toWeight: null as number | null,
        fromCartPrice: null as number | null,
        toCartPrice: null as number | null,
        shippingClassLinked: {} as ShippingClass | null,
        cost: null as number | null
    });

    const [dataListReductionDelivery, setDataListReductionDelivery] = useState<RowDataReduction[]>([
        { nbrProducts: 10, percentageReduction: 8 },
        { nbrProducts: 15, percentageReduction: 10 },
        { nbrProducts: 25, percentageReduction: 12 },
        { nbrProducts: 35, percentageReduction: 14 },
        { nbrProducts: 45, percentageReduction: 16 },
        { nbrProducts: 60, percentageReduction: 20 },
    ]);

    const [editIndexReduction, setEditIndexReduction] = useState<number | null>(null);
    const [formDataReduction, setFormDataReduction] = useState<RowDataReduction>({
        nbrProducts: null as number | null,
        percentageReduction: null as number | null,
    });


    const [openAddCostModal, setOpenAddCostModal] = useState(false);
    const [openEditCostModal, setOpenEditCostModal] = useState(false);
    const [openAddReductionModal, setOpenAddReductionModal] = useState(false);
    const [openEditReductionModal, setOpenEditReductionModal] = useState(false);



    const handleAddCostClick = () => {
        setFormDataCost({
            fromWeight: null,
            toWeight: null,
            fromCartPrice: null,
            toCartPrice: null,
            shippingClassLinked: null,
            cost: null
        });
        setOpenAddCostModal(true);
    };

    const handleEditCostClick = (index: number) => {
        setEditIndexCost(index);
        setOpenEditCostModal(true);
        const row = dataListCostDelivery[index];
        if (row) {
            setFormDataCost({
                fromWeight: row.fromWeight,
                toWeight: row.toWeight,
                fromCartPrice: row.fromCartPrice,
                toCartPrice: row.toCartPrice,
                shippingClassLinked: row.shippingClassLinked,
                cost: row.cost
            });
        }
    };

    const handleCloseEditCostModal = () => {
        setOpenEditCostModal(false);
    };


    const handleCloseAddCostModal = () => {
        setOpenAddCostModal(false);
    };

    //************ */


    const handleAddReductionClick = () => {
        setFormDataReduction({
            nbrProducts: null,
            percentageReduction: null
        })
        setOpenAddReductionModal(true);
    };

    const handleEditReductionClick = (index: number) => {
        setEditIndexReduction(index);
        const row = dataListReductionDelivery[index];
        if (row) {
            setFormDataReduction({
                nbrProducts: row.nbrProducts,
                percentageReduction: row.percentageReduction
            });
        }
        setOpenEditReductionModal(true);
    };

    const handleCloseEditReductionModal = () => {
        setOpenEditReductionModal(false);
    };


    const handleCloseAddReductionModal = () => {
        setOpenAddReductionModal(false);
    };

    const [dataListCost, setDataListCost] = useState(dataListCostDelivery);

    const [selectedFileCost, setSelectedFileCost] = useState<File | null>(null);
    const handleFileCostInputChange = (event: React.ChangeEvent<HTMLInputElement>) => {
        const file = event.target.files?.[0];
        if (file && file.type === 'text/csv') {
            setSelectedFileCost(file);
        } else {
            setSelectedFileCost(null);
        }
    };

    const [selectedRowsCost, setSelectedRowsCost] = useState<number[]>([]);
    const [selectAllCost, setSelectAllCost] = useState(false);

    const handleRowSelectCost = (index: number) => {
        if (selectedRowsCost.includes(index)) {
            setSelectedRowsCost(selectedRowsCost.filter((rowIndex) => rowIndex !== index));
        } else {
            setSelectedRowsCost([...selectedRowsCost, index]);
        }
    };

    const handleSelectAllCost = () => {
        if (selectAllCost) {
            setSelectedRowsCost([]);
        } else {
            setSelectedRowsCost(Array.from({ length: dataListCostDelivery.length }, (_, index) => index));
        }
        setSelectAllCost(!selectAllCost);
    };

    const [selectedCostRows, setSelectedCostRows] = useState<RowDataCost[]>([]);
    const [costPage, setCostPage] = useState(0);
    const [costRowsPerPage, setCostRowsPerPage] = useState(5);
    const [selectedCostCount, setSelectedCostCount] = useState(0);

    const handleChangeCostPage = (event: React.MouseEvent<HTMLButtonElement> | null, newPage: number) => {
        setCostPage(newPage);
    };

    const handleChangeCostRowsPerPage = (event: React.ChangeEvent<HTMLInputElement>) => {
        setCostRowsPerPage(parseInt(event.target.value, 10));
        setCostPage(0);
    };

    const handleCostRowCheckboxChange = (event: React.ChangeEvent<HTMLInputElement>, row: RowDataCost) => {
        const selectedIndex = selectedCostRows.indexOf(row);
        let newSelectedCost: RowDataCost[] = [];

        if (selectedIndex === -1) {
            newSelectedCost = newSelectedCost.concat(selectedCostRows, row);
        } else if (selectedIndex === 0) {
            newSelectedCost = newSelectedCost.concat(selectedCostRows.slice(1));
        } else if (selectedIndex === selectedCostRows.length - 1) {
            newSelectedCost = newSelectedCost.concat(selectedCostRows.slice(0, -1));
        } else if (selectedIndex > 0) {
            newSelectedCost = newSelectedCost.concat(
                selectedCostRows.slice(0, selectedIndex),
                selectedCostRows.slice(selectedIndex + 1)
            );
        }

        setSelectedCostRows(newSelectedCost);
        setSelectedCostCount(newSelectedCost.length);
    };

    const handleSelectCostAllClick = (event: React.ChangeEvent<HTMLInputElement>) => {
        if (event.target.checked) {
            const newSelecteds = [...dataListCostDelivery];
            setSelectedCostRows(newSelecteds);
            setSelectedCostCount(newSelecteds.length)
            return;
        }
        setSelectedCostRows([]);
        setSelectedCostCount(0);
    };



    //--------------------

    const [selectedRowsReduction, setSelectedRowsReduction] = useState<number[]>([]);
    const [selectAllReduction, setSelectAllReduction] = useState(false);

    const handleRowSelectReduction = (index: number) => {
        if (selectedRowsReduction.includes(index)) {
            setSelectedRowsReduction(selectedRowsReduction.filter((rowIndex) => rowIndex !== index));
        } else {
            setSelectedRowsReduction([...selectedRowsReduction, index]);
        }
    };

    const handleSelectAllReduction = () => {
        if (selectAllReduction) {
            setSelectedRowsReduction([]);
        } else {
            setSelectedRowsReduction(Array.from({ length: dataListReductionDelivery.length }, (_, index) => index));
        }
        setSelectAllReduction(!selectAllReduction);
        setSelectedReductionCount(0);
    };

    const [selectedReductionRows, setSelectedReductionRows] = useState<RowDataReduction[]>([]);
    const [reductionPage, setReductionPage] = useState(0);
    const [reductionRowsPerPage, setReductionRowsPerPage] = useState(5);
    const [selectedReductionCount, setSelectedReductionCount] = useState(0);


    const handleChangeReductionPage = (event: React.MouseEvent<HTMLButtonElement> | null, newPage: number) => {
        setReductionPage(newPage);
    };

    const handleChangeReductionRowsPerPage = (event: React.ChangeEvent<HTMLInputElement>) => {
        setReductionRowsPerPage(parseInt(event.target.value, 10));
        setReductionPage(0);
    };

    const handleReductionRowCheckboxChange = (event: React.ChangeEvent<HTMLInputElement>, row: RowDataReduction) => {
        const selectedIndex = selectedReductionRows.indexOf(row);
        let newSelectedReduction: RowDataReduction[] = [];

        if (selectedIndex === -1) {
            newSelectedReduction = newSelectedReduction.concat(selectedReductionRows, row);
        } else if (selectedIndex === 0) {
            newSelectedReduction = newSelectedReduction.concat(selectedReductionRows.slice(1));
        } else if (selectedIndex === selectedReductionRows.length - 1) {
            newSelectedReduction = newSelectedReduction.concat(selectedReductionRows.slice(0, -1));
        } else if (selectedIndex > 0) {
            newSelectedReduction = newSelectedReduction.concat(
                selectedReductionRows.slice(0, selectedIndex),
                selectedReductionRows.slice(selectedIndex + 1)
            );
        }

        setSelectedReductionRows(newSelectedReduction);
        setSelectedReductionCount(newSelectedReduction.length)
    };

    const handleSelectReductionAllClick = (event: React.ChangeEvent<HTMLInputElement>) => {
        if (event.target.checked) {
            const newSelecteds = dataListReductionDelivery.map((row) => row);
            setSelectedReductionRows(newSelecteds);
            setSelectedReductionCount(newSelecteds.length)
            return;
        }
        setSelectedReductionRows([]);
        setSelectedReductionCount(0);
    };

    const confirmDeleteCost = useBoolean();

    const confirmDeleteReduction = useBoolean();


    const renderForm = (
        <>
            <Container>
                <CustomBreadcrumbs
                    heading="Modifier la méthode de livraison"
                    links={[
                        { name: "Dashboard", href: paths.dashboard.root },
                        { name: "Méthodes de livraison", href: paths.dashboard.shipping.general.shippingMethod.shippingMethodList },
                        { name: shippingMethod?.type }
                    ]}
                    sx={{
                        mb: {
                            xs: 3,
                            md: 5,
                        },
                    }}
                />
                <Grid direction={'row'} display={'flex'} xs={12} md={12}>
                    <Grid xs={12} md={6}>
                        <Card>
                            <Stack spacing={3} sx={{ p: 3 }}>
                                <RHFTextField name="name" label="Nom de la méthode" />
                                <RHFTextField name="description" label="Description" placeholder="Description..." multiline rows={4} />
                                <Box style={{ display: 'flex', alignItems: 'center' }}><Typography>Activé</Typography> <Switch name='active' inputProps={{ 'aria-label': 'controlled' }} /></Box>
                                <RHFAutocomplete
                                    disabled
                                    name="type"
                                    label="Type de méthode"
                                    placeholder="+ Type de méthode"
                                    disableClearable
                                    options={shippingMethodTypes}
                                    getOptionLabel={(option) => option}
                                    renderOption={(props, option) => (
                                        <li {...props} key={option}>
                                            {option}
                                        </li>
                                    )}
                                    renderTags={(selected, getTagProps) =>
                                        selected.map((option, index) => (
                                            <Chip
                                                {...getTagProps({ index })}
                                                key={option}
                                                label={option}
                                                size="small"
                                                color="success"
                                                variant="soft"
                                            />
                                        ))
                                    }
                                />
                            </Stack>
                        </Card>
                    </Grid>
                    <Grid xs={12} md={6}>
                        <Card>
                            <Stack spacing={3} sx={{ p: 3 }}>
                                <Grid container alignItems="center" spacing={1}>
                                    <Grid>
                                        <Checkbox
                                            name='stillFree'
                                            checked={checkedStillFree}
                                            onChange={handleChangeStillFree}
                                            color='success'
                                            inputProps={{ 'aria-label': 'primary checkbox' }}
                                        />
                                    </Grid>
                                    <Grid>
                                        <Typography variant='body2'>Toujours gratuit ?</Typography>
                                    </Grid>
                                </Grid>
                                <RHFTextField name="nameFreeShipping" label="Nom si la livraison est offerte" />
                                <RHFAutocomplete
                                    name="excludedShippingClasses"
                                    label="Classes d’expédition exclues"
                                    placeholder="+ Classes d’expédition exclues"
                                    multiple
                                    freeSolo
                                    disableCloseOnSelect
                                    options={shippingClassList.map((option) => option.name)}
                                    getOptionLabel={(option) => option}
                                    renderOption={(props, option) => (
                                        <li {...props} key={option}>
                                            {option}
                                        </li>
                                    )}
                                    renderTags={(selected, getTagProps) =>
                                        selected.map((option, index) => (
                                            <Chip
                                                {...getTagProps({ index })}
                                                key={option}
                                                label={option}
                                                size="small"
                                                color="success"
                                                variant="soft"
                                            />
                                        ))
                                    }
                                />
                                <RHFAutocomplete
                                    name="excludedShippingClasses"
                                    label="Classes d’expédition gratuites"
                                    placeholder="+ Classes d’expédition gratuites"
                                    multiple
                                    freeSolo
                                    disableCloseOnSelect
                                    options={shippingClassList.map((option) => option.name)}
                                    getOptionLabel={(option) => option}
                                    renderOption={(props, option) => (
                                        <li {...props} key={option}>
                                            {option}
                                        </li>
                                    )}
                                    renderTags={(selected, getTagProps) =>
                                        selected.map((option, index) => (
                                            <Chip
                                                {...getTagProps({ index })}
                                                key={option}
                                                label={option}
                                                size="small"
                                                color="success"
                                                variant="soft"
                                            />
                                        ))
                                    }
                                />
                                <Grid container alignItems="center" spacing={1}>
                                    <Grid md={1} xs={2}>
                                        <Checkbox
                                            name='stillFree'
                                            checked={checkedConditionFree}
                                            onChange={handleChangeConditionFree}
                                            color='success'
                                            inputProps={{ 'aria-label': 'primary checkbox' }}
                                        />
                                    </Grid>
                                    <Grid md={11} xs={10}>
                                        <Typography variant='body2'>Gratuit si au moins un des produits du panier a une des classes d’expédition gratuite ci-dessus</Typography>
                                    </Grid>
                                </Grid>
                            </Stack>
                        </Card>
                    </Grid>
                </Grid>

                {mdUp && (
                    <Grid md={12} sx={{ mt: 1 }}>
                        <Typography variant="h6">
                            Frais de livraison
                        </Typography>
                    </Grid>
                )}

                <Grid xs={12} md={12}>
                    <Card>
                        {!mdUp && <CardHeader title="Frais de livraison" />}
                        <Container sx={{ mt: 2 }}>
                            <Box sx={{ display: 'flex', alignItems: 'center', gap: 1 }}>
                                <Button variant="outlined" size="small" sx={{ color: '#808080', width: '15%' }} onClick={handleAddCostClick} startIcon={<AddIcon />}>
                                    Ajouter
                                </Button>
                                <Button variant="outlined" size="small" sx={{ color: '#808080', width: '15%' }} startIcon={<DeleteIcon />}>
                                    Supprimer
                                </Button>
                                <Button variant="outlined" size="small" sx={{ color: '#808080', width: '15%' }} startIcon={<GetAppIcon />}>
                                    Exporter
                                </Button>
                                <Input
                                    type="file"
                                    id="importFileInput"
                                    sx={{ display: 'none' }}
                                    onChange={handleFileCostInputChange}
                                />
                                <FormLabel htmlFor="importFileInput" sx={{ width: '20%' }}>
                                    <Button size="small" sx={{ color: '#808080' }} component="span">
                                        Choisir un fichier
                                    </Button>
                                </FormLabel>
                                {selectedFileCost ? (
                                    <Typography sx={{ fontSize: '12px', width: '20%' }}>{selectedFileCost.name}</Typography>
                                ) : (
                                    <Typography sx={{ fontSize: '12px', width: '20%' }}>Aucun fichier choisi</Typography>
                                )}
                                <Button variant="outlined" size="small" sx={{ color: '#808080', width: '15%' }} startIcon={<PublishIcon />}>
                                    Importer
                                </Button>
                                <Tooltip title={<Typography variant="caption">Le fichier que vous importez doit obligatoirement un CSV avec les 6 colonnes suivants: min_weight, max_weight, min_price, max_price, shipping_class, price</Typography>}>
                                    <IconButton sx={{ p: 1 }}>
                                        <HelpOutlineIcon sx={{ fontSize: 16 }} />
                                    </IconButton>
                                </Tooltip>
                            </Box>
                        </Container>
                        {selectedCostCount > 0 && (
                            <Box sx={{ display: 'flex', justifyContent: 'flex-start', marginTop: 1, backgroundColor: '#acebbc', padding: 1 }}>
                                <Typography variant="body2" sx={{ fontWeight: 'bold', color: "green" }}>
                                    {`${selectedCostCount} élément(s) sélectionné(s)`}
                                </Typography>
                            </Box>
                        )}
                        <TableContainer component={Paper} sx={{ mt: 1, maxHeight: '500px' }}>
                            <Table sx={{ minWidth: 650 }} aria-label="simple table">
                                <TableHead>
                                    <TableRow>
                                        <TableCell align='center'>
                                            <Checkbox
                                                indeterminate={
                                                    selectedCostRows.length > 0 &&
                                                    selectedCostRows.length < dataListCostDelivery.length
                                                }
                                                checked={
                                                    selectedCostRows.length === dataListCostDelivery.length
                                                }
                                                onChange={handleSelectCostAllClick}
                                            />
                                        </TableCell>
                                        <TableCell align='center'>À partir du poids (g)</TableCell>
                                        <TableCell align='center'>Jusqu’au poids (g)</TableCell>
                                        <TableCell align='center'>À partir du prix du panier (EUR €)</TableCell>
                                        <TableCell align='center'>Jusqu’au prix panier (EUR €)</TableCell>
                                        <TableCell align='center'>Classe d’expédition</TableCell>
                                        <TableCell align='center' width={'100px'}>Prix (EUR €)</TableCell>
                                        <TableCell align='center' width={'115px'}>Actions</TableCell>
                                    </TableRow>
                                </TableHead>
                                <TableBody>
                                    {dataListCostDelivery
                                        .slice(costPage * costRowsPerPage, costPage * costRowsPerPage + costRowsPerPage)
                                        .map((row, index) => (
                                            <TableRow key={index} sx={{ '&:last-child td, &:last-child th': { border: 0 } }}>
                                                <TableCell component="th" scope="row" align='center'>
                                                    <Checkbox
                                                        onChange={(event) => handleCostRowCheckboxChange(event, row)}
                                                        checked={selectedCostRows.indexOf(row) !== -1}
                                                    />
                                                </TableCell>
                                                <TableCell align='center'>{row.fromWeight} g</TableCell>
                                                <TableCell align='center'>{row.toWeight} g</TableCell>
                                                <TableCell align='center'>{row.fromCartPrice} €</TableCell>
                                                <TableCell align='center'>{row.toCartPrice} €</TableCell>
                                                <TableCell align='center'>{row.shippingClassLinked?.name}</TableCell>
                                                <TableCell align='center'>{row.cost} €</TableCell>
                                                <TableCell align='center'>
                                                    <Box>
                                                        <IconButton>
                                                            <EditIcon sx={{ color: 'green' }} onClick={() => handleEditCostClick(index)} />
                                                        </IconButton>
                                                        <IconButton>
                                                            <DeleteIcon sx={{ color: 'red' }} onClick={() => { confirmDeleteCost.onTrue(); }} />
                                                        </IconButton>
                                                    </Box>
                                                </TableCell>
                                            </TableRow>
                                        ))}
                                </TableBody>
                            </Table>
                        </TableContainer>
                        <TablePagination
                            rowsPerPageOptions={[5, 10, 25]}
                            component="div"
                            count={dataListCostDelivery.length}
                            rowsPerPage={costRowsPerPage}
                            page={costPage}
                            onPageChange={handleChangeCostPage}
                            onRowsPerPageChange={handleChangeCostRowsPerPage}
                        />
                    </Card>
                </Grid>

                <ConfirmDialog
                    open={confirmDeleteCost.value}
                    onClose={confirmDeleteCost.onFalse}
                    title="Supprimer frais"
                    content="Voulez-vous vraiment supprimer?"
                    action={
                        <Button variant="contained" color="error" onClick={() => confirmDeleteCost.onFalse}>
                            Supprimer
                        </Button>
                    }
                />


                {mdUp && (
                    <Grid md={12} sx={{ mt: 1 }}>
                        <Typography variant="h6">
                            Réduction sur la livraison
                        </Typography>
                    </Grid>
                )}

                <Grid xs={12} md={12}>
                    <Card>
                        {!mdUp && <CardHeader title="Réduction sur la livraison" />}

                        <Container sx={{ mt: 2 }}>
                            <Box sx={{ display: 'flex', alignItems: 'center', gap: 1 }}>
                                <Button variant="outlined" size="small" sx={{ color: '#808080', width: '15%' }} onClick={handleAddReductionClick} startIcon={<AddIcon />}>
                                    Ajouter
                                </Button>
                                <Button variant="outlined" size="small" sx={{ color: '#808080', width: '15%' }} startIcon={<DeleteIcon />}>
                                    Supprimer
                                </Button>
                            </Box>
                        </Container>
                        {selectedReductionCount > 0 && (
                            <Box sx={{ display: 'flex', justifyContent: 'flex-start', marginTop: 1, backgroundColor: '#acebbc', padding: 1 }}>
                                <Typography variant="body2" sx={{ fontWeight: 'bold', color: "green" }}>
                                    {`${selectedReductionCount} élément(s) sélectionné(s)`}
                                </Typography>
                            </Box>
                        )}
                        <TableContainer component={Paper} sx={{ mt: 1, maxHeight: '435px' }}>
                            <Table sx={{ minWidth: 650 }} aria-label="simple table">
                                <TableHead>
                                    <TableRow>
                                        <TableCell align='center'>
                                            <Checkbox
                                                indeterminate={
                                                    selectedReductionRows.length > 0 &&
                                                    selectedReductionRows.length < dataListReductionDelivery.length
                                                }
                                                checked={
                                                    selectedReductionRows.length === dataListReductionDelivery.length
                                                }
                                                onChange={handleSelectReductionAllClick}
                                            />
                                        </TableCell>
                                        <TableCell align='center'>Nombre de produits</TableCell>
                                        <TableCell align='center'>Réduction en pourcentage</TableCell>
                                        <TableCell align='center'>Actions</TableCell>
                                    </TableRow>
                                </TableHead>
                                <TableBody>
                                    {dataListReductionDelivery
                                        .slice(reductionPage * reductionRowsPerPage, reductionPage * reductionRowsPerPage + reductionRowsPerPage)
                                        .map((row, index) => (
                                            <TableRow key={index} sx={{ '&:last-child td, &:last-child th': { border: 0 } }}>
                                                <TableCell component="th" scope="row" align='center'>
                                                    <Checkbox
                                                        onChange={(event) => handleReductionRowCheckboxChange(event, row)}
                                                        checked={selectedReductionRows.indexOf(row) !== -1}
                                                    />
                                                </TableCell>
                                                <TableCell component="th" scope="row" align='center'>{row.nbrProducts}</TableCell>
                                                <TableCell align='center'>{row.percentageReduction} %</TableCell>
                                                <TableCell align='center'>
                                                    <Box>
                                                        <IconButton><EditIcon sx={{ color: 'green' }} onClick={() => handleEditReductionClick(index)} /></IconButton>
                                                        <IconButton><DeleteIcon sx={{ color: 'red' }} onClick={() => { confirmDeleteReduction.onTrue(); }} /></IconButton>
                                                    </Box>
                                                </TableCell>
                                            </TableRow>
                                        ))}
                                </TableBody>
                            </Table>
                        </TableContainer>
                        <TablePagination
                            rowsPerPageOptions={[5, 10, 25]}
                            component="div"
                            count={dataListReductionDelivery.length}
                            rowsPerPage={reductionRowsPerPage}
                            page={reductionPage}
                            onPageChange={handleChangeReductionPage}
                            onRowsPerPageChange={handleChangeReductionRowsPerPage}
                        />
                    </Card>
                </Grid>

                <ConfirmDialog
                    open={confirmDeleteReduction.value}
                    onClose={confirmDeleteReduction.onFalse}
                    title="Supprimer réduction"
                    content="Voulez-vous vraiment supprimer?"
                    action={
                        <Button variant="contained" color="error" onClick={() => confirmDeleteReduction.onFalse}>
                            Supprimer
                        </Button>
                    }
                />
            </Container>

            <Modal open={openAddCostModal} onClose={handleCloseAddCostModal}>
                <Box sx={{ position: 'absolute', top: '50%', left: '50%', transform: 'translate(-50%, -50%)', width: 600, bgcolor: 'background.paper', boxShadow: 24, p: 4, borderRadius: 3 }}>
                    <Typography variant="h6" align="center" sx={{ mt: 2, mb: 3 }} gutterBottom>Ajouter frais de livraison</Typography>
                    <RHFTextField
                        name="fromWeight"
                        label="À partir du poids (g)"
                        placeholder="0.00"
                        type="number"
                        value={formDataCost.fromWeight}
                        onChange={(e) => setFormDataCost({ ...formDataCost, fromWeight: parseInt(e.target.value, 10) })}
                        sx={{ m: 1, width: '25ch' }}
                        InputLabelProps={{ shrink: true }}
                        InputProps={{
                            startAdornment: (
                                <InputAdornment position="start">
                                    <Box component="span" sx={{ color: 'text.disabled' }}>
                                        g
                                    </Box>
                                </InputAdornment>
                            ),
                        }}
                    />
                    <RHFTextField
                        name="toWeight"
                        label="Jusqu’au poids (g)"
                        placeholder="0.00"
                        type="number"
                        value={formDataCost.toWeight}
                        onChange={(e) => setFormDataCost({ ...formDataCost, toWeight: parseInt(e.target.value, 10) })}
                        sx={{ m: 1, width: '25ch' }}
                        InputLabelProps={{ shrink: true }}
                        InputProps={{
                            startAdornment: (
                                <InputAdornment position="start">
                                    <Box component="span" sx={{ color: 'text.disabled' }}>
                                        g
                                    </Box>
                                </InputAdornment>
                            ),
                        }}
                    />
                    <RHFTextField
                        name="fromCartPrice"
                        label="À partir du prix du panier (EUR €)"
                        placeholder="0.00"
                        type="number"
                        value={formDataCost.fromCartPrice}
                        onChange={(e) => setFormDataCost({ ...formDataCost, fromCartPrice: parseInt(e.target.value, 10) })}
                        sx={{ m: 1, width: '25ch' }}
                        InputLabelProps={{ shrink: true }}
                        InputProps={{
                            startAdornment: (
                                <InputAdornment position="start">
                                    <Box component="span" sx={{ color: 'text.disabled' }}>
                                        €
                                    </Box>
                                </InputAdornment>
                            ),
                        }}
                    />
                    <RHFTextField
                        name="toCartPrice"
                        label="Jusqu’au prix panier (EUR €)"
                        placeholder="0.00"
                        type="number"
                        value={formDataCost.toCartPrice}
                        onChange={(e) => setFormDataCost({ ...formDataCost, toCartPrice: parseInt(e.target.value, 10) })}
                        sx={{ m: 1, width: '25ch' }}
                        InputLabelProps={{ shrink: true }}
                        InputProps={{
                            startAdornment: (
                                <InputAdornment position="start">
                                    <Box component="span" sx={{ color: 'text.disabled' }}>
                                        €
                                    </Box>
                                </InputAdornment>
                            ),
                        }}
                    />
                    <Box sx={{ display: 'flex', flexDirection: 'row', alignItems: 'center', width: '100%' }}>
                        <RHFAutocomplete
                            name="shippingClass"
                            label="Classe d'expédition"
                            sx={{ m: 1, width: '30ch' }}
                            freeSolo
                            disableCloseOnSelect
                            value={formDataCost.shippingClassLinked ? formDataCost.shippingClassLinked.name : ''}
                            onChange={(e, value) => {
                                const selectedShippingClass = shippingClassList.find(option => option.name === value);
                                setFormDataCost({ ...formDataCost, shippingClassLinked: selectedShippingClass || null });
                            }}
                            options={shippingClassList.map((option) => option.name)}
                            getOptionLabel={(option) => option}
                            renderOption={(props, option) => (
                                <li {...props} key={option}>
                                    {option}
                                </li>
                            )}
                            renderTags={(selected, getTagProps) =>
                                selected.map((option, index) => (
                                    <Chip
                                        {...getTagProps({ index })}
                                        key={option}
                                        label={option}
                                        size="small"
                                        color="success"
                                        variant="soft"
                                    />
                                ))
                            }
                        />
                        <RHFTextField
                            name="cost"
                            label="Prix"
                            placeholder="0.00"
                            type="number"
                            value={formDataCost.cost}
                            onChange={(e) => setFormDataCost({ ...formDataCost, cost: parseInt(e.target.value, 10) })}
                            sx={{ m: 1, width: '20ch' }}
                            InputLabelProps={{ shrink: true }}
                            InputProps={{
                                startAdornment: (
                                    <InputAdornment position="start">
                                        <Box component="span" sx={{ color: 'text.disabled' }}>
                                            €
                                        </Box>
                                    </InputAdornment>
                                ),
                            }}
                        />
                    </Box>
                    <Box sx={{ display: 'flex', justifyContent: 'center', mt: 4 }}>
                        <Button onClick={() => handleCloseAddCostModal} sx={{ mr: 1 }} variant="outlined" color="inherit">Annuler</Button>
                        <Button onClick={() => handleCloseAddCostModal} variant="contained" color="inherit">Ajouter</Button>
                    </Box>
                </Box>
            </Modal>


            <Modal open={openEditCostModal} onClose={handleCloseEditCostModal}>
                <Box sx={{ position: 'absolute', top: '50%', left: '50%', transform: 'translate(-50%, -50%)', width: 600, bgcolor: 'background.paper', boxShadow: 24, p: 4, borderRadius: 3 }}>
                    <Typography variant="h6" align="center" gutterBottom sx={{ mt: 2, mb: 3 }}>Modifier frais de livraison</Typography>
                    <RHFTextField
                        name="fromWeight"
                        label="À partir du poids (g)"
                        placeholder="0.00"
                        type="number"
                        value={formDataCost.fromWeight}
                        onChange={(e) => setFormDataCost({ ...formDataCost, fromWeight: parseInt(e.target.value, 10) })}
                        sx={{ m: 1, width: '25ch' }}
                        InputLabelProps={{ shrink: true }}
                        InputProps={{
                            startAdornment: (
                                <InputAdornment position="start">
                                    <Box component="span" sx={{ color: 'text.disabled' }}>
                                        g
                                    </Box>
                                </InputAdornment>
                            ),
                        }}
                    />
                    <RHFTextField
                        name="toWeight"
                        label="Jusqu’au poids (g)"
                        placeholder="0.00"
                        type="number"
                        value={formDataCost.toWeight}
                        onChange={(e) => setFormDataCost({ ...formDataCost, toWeight: parseInt(e.target.value, 10) })}
                        sx={{ m: 1, width: '25ch' }}
                        InputLabelProps={{ shrink: true }}
                        InputProps={{
                            startAdornment: (
                                <InputAdornment position="start">
                                    <Box component="span" sx={{ color: 'text.disabled' }}>
                                        g
                                    </Box>
                                </InputAdornment>
                            ),
                        }}
                    />
                    <RHFTextField
                        name="fromCartPrice"
                        label="À partir du prix du panier (EUR €)"
                        placeholder="0.00"
                        type="number"
                        value={formDataCost.fromCartPrice}
                        onChange={(e) => setFormDataCost({ ...formDataCost, fromCartPrice: parseInt(e.target.value, 10) })}
                        sx={{ m: 1, width: '25ch' }}
                        InputLabelProps={{ shrink: true }}
                        InputProps={{
                            startAdornment: (
                                <InputAdornment position="start">
                                    <Box component="span" sx={{ color: 'text.disabled' }}>
                                        €
                                    </Box>
                                </InputAdornment>
                            ),
                        }}
                    />
                    <RHFTextField
                        name="toCartPrice"
                        label="Jusqu’au prix panier (EUR €)"
                        placeholder="0.00"
                        type="number"
                        value={formDataCost.toCartPrice}
                        onChange={(e) => setFormDataCost({ ...formDataCost, toCartPrice: parseInt(e.target.value, 10) })}
                        sx={{ m: 1, width: '25ch' }}
                        InputLabelProps={{ shrink: true }}
                        InputProps={{
                            startAdornment: (
                                <InputAdornment position="start">
                                    <Box component="span" sx={{ color: 'text.disabled' }}>
                                        €
                                    </Box>
                                </InputAdornment>
                            ),
                        }}
                    />
                    <Box sx={{ display: 'flex', flexDirection: 'row', alignItems: 'center', width: '100%' }}>
                        <RHFAutocomplete
                            name="shippingClass"
                            label="Classe d'expédition"
                            sx={{ m: 1, width: '30ch' }}
                            freeSolo
                            disableCloseOnSelect
                            value={formDataCost.shippingClassLinked ? formDataCost.shippingClassLinked.name : ''}
                            onChange={(e, value) => {
                                const selectedShippingClass = shippingClassList.find(option => option.name === value);
                                setFormDataCost({ ...formDataCost, shippingClassLinked: selectedShippingClass || null });
                            }}
                            options={shippingClassList.map((option) => option.name)}
                            getOptionLabel={(option) => option}
                            renderOption={(props, option) => (
                                <li {...props} key={option}>
                                    {option}
                                </li>
                            )}
                            renderTags={(selected, getTagProps) =>
                                selected.map((option, index) => (
                                    <Chip
                                        {...getTagProps({ index })}
                                        key={option}
                                        label={option}
                                        size="small"
                                        color="success"
                                        variant="soft"
                                    />
                                ))
                            }
                        />
                        <RHFTextField
                            name="cost"
                            label="Prix"
                            placeholder="0.00"
                            type="number"
                            value={formDataCost.cost}
                            onChange={(e) => setFormDataCost({ ...formDataCost, cost: parseInt(e.target.value, 10) })}
                            sx={{ m: 1, width: '20ch' }}
                            InputLabelProps={{ shrink: true }}
                            InputProps={{
                                startAdornment: (
                                    <InputAdornment position="start">
                                        <Box component="span" sx={{ color: 'text.disabled' }}>
                                            €
                                        </Box>
                                    </InputAdornment>
                                ),
                            }}
                        />
                    </Box>
                    <Box sx={{ display: 'flex', justifyContent: 'center', mt: 4 }}>
                        <Button onClick={() => handleCloseEditCostModal} sx={{ mr: 1 }} variant="outlined" color="inherit">Annuler</Button>
                        <Button onClick={() => handleCloseEditCostModal} variant="contained" color="inherit">Modifier</Button>
                    </Box>
                </Box>
            </Modal>

            <Modal open={openAddReductionModal} onClose={handleCloseAddReductionModal}>
                <Box sx={{ position: 'absolute', top: '50%', left: '50%', transform: 'translate(-50%, -50%)', width: 550, bgcolor: 'background.paper', boxShadow: 24, p: 4, borderRadius: 3 }}>
                    <Typography variant="h6" align="center" gutterBottom sx={{ mt: 2, mb: 3 }}>Ajouter réduction sur la livraison</Typography>
                    <Box sx={{ width: '80%', ml: 'auto', mr: 'auto' }}>
                        <RHFTextField
                            name="nbrProducts"
                            label="Nombre de produits"
                            type="number"
                            placeholder="0"
                            value={formDataReduction.nbrProducts}
                            onChange={(e) => setFormDataReduction({ ...formDataReduction, nbrProducts: parseInt(e.target.value, 10) })}
                            sx={{ m: 1 }}
                            InputLabelProps={{ shrink: true }}
                        />
                        <RHFTextField
                            name="percentageReduction"
                            label="Réduction en pourcentage"
                            placeholder="0.00"
                            type="number"
                            value={formDataReduction.percentageReduction}
                            onChange={(e) => setFormDataReduction({ ...formDataReduction, percentageReduction: parseInt(e.target.value, 10) })}
                            sx={{ m: 1 }}
                            InputLabelProps={{ shrink: true }}
                            InputProps={{
                                startAdornment: (
                                    <InputAdornment position="start">
                                        <Box component="span" sx={{ color: 'text.disabled' }}>
                                            %
                                        </Box>
                                    </InputAdornment>
                                ),
                            }}
                        />
                    </Box>
                    <Box sx={{ display: 'flex', justifyContent: 'center', mt: 4 }}>
                        <Button onClick={() => handleCloseAddReductionModal} sx={{ mr: 1 }} variant="outlined" color="inherit">Annuler</Button>
                        <Button onClick={() => handleCloseAddReductionModal} variant="contained" color="inherit">Ajouter</Button>
                    </Box>
                </Box>
            </Modal>


            <Modal open={openEditReductionModal} onClose={handleCloseEditReductionModal}>
                <Box sx={{ position: 'absolute', top: '50%', left: '50%', transform: 'translate(-50%, -50%)', width: 550, bgcolor: 'background.paper', boxShadow: 24, p: 4, borderRadius: 3 }}>
                    <Typography variant="h6" align="center" gutterBottom sx={{ mt: 2, mb: 3 }}>Modifier réduction sur la livraison</Typography>
                    <Box sx={{ width: '80%', ml: 'auto', mr: 'auto' }}>
                        <RHFTextField
                            name="nbrProducts"
                            label="Nombre de produits"
                            placeholder='0'
                            type="number"
                            value={formDataReduction.nbrProducts}
                            onChange={(e) => setFormDataReduction({ ...formDataReduction, nbrProducts: parseInt(e.target.value, 10) })}
                            sx={{ m: 1 }}
                            InputLabelProps={{ shrink: true }}
                        />
                        <RHFTextField
                            name="percentageReduction"
                            label="Réduction en pourcentage"
                            placeholder="0.00"
                            type="number"
                            value={formDataReduction.percentageReduction}
                            onChange={(e) => setFormDataReduction({ ...formDataReduction, percentageReduction: parseInt(e.target.value, 10) })}
                            sx={{ m: 1 }}
                            InputLabelProps={{ shrink: true }}
                            InputProps={{
                                startAdornment: (
                                    <InputAdornment position="start">
                                        <Box component="span" sx={{ color: 'text.disabled' }}>
                                            %
                                        </Box>
                                    </InputAdornment>
                                ),
                            }}
                        />
                    </Box>
                    <Box sx={{ display: 'flex', justifyContent: 'center', mt: 4 }}>
                        <Button onClick={() => handleCloseEditReductionModal} sx={{ mr: 1 }} variant="outlined" color="inherit">Annuler</Button>
                        <Button onClick={() => handleCloseEditReductionModal} variant="contained" color="inherit">Modifier</Button>
                    </Box>
                </Box>
            </Modal>
        </>
    );

    const renderActions = (
        <>
            {mdUp && <Grid md={4} />}
            <Grid xs={12} md={8} sx={{ display: 'flex', justifyContent: 'end', py: 4 }}>
                <LoadingButton
                    type="submit"
                    variant="contained"
                    size="large"
                    loading={isSubmitting}
                    sx={{ ml: 2 }}
                >Enregistrer</LoadingButton>
            </Grid>
        </>
    );

    return (
        <FormProvider methods={methods} onSubmit={onSubmit}>
            <Grid container spacing={3}>
                {renderForm}
                {renderActions}
            </Grid>
        </FormProvider>
    );
}
