"use client"

import * as Yup from 'yup';
import { useForm } from 'react-hook-form';
import { yupResolver } from '@hookform/resolvers/yup';
import { useEffect, useMemo } from 'react';
import { useState } from 'react';
import Chip from '@mui/material/Chip';
import Box from '@mui/material/Box';
import Grid from '@mui/material/Unstable_Grid2';
import Card from '@mui/material/Card';
import Stack from '@mui/material/Stack';
import CardHeader from '@mui/material/CardHeader';
import Typography from '@mui/material/Typography';
import Radio from '@mui/material/Radio';
import RadioGroup from '@mui/material/RadioGroup';
import FormControlLabel from '@mui/material/FormControlLabel';
import FormControl from '@mui/material/FormControl';
import FormLabel from '@mui/material/FormLabel';
import InputAdornment from '@mui/material/InputAdornment';
import { useResponsive } from '@/hooks/use-responsive';
import {
    RHFTextField,
    RHFAutocomplete,
} from '@/components/hook-form';
import { RequiresFreeShipping, CalculationType, ShippingMethodType, ShippingMethod } from '@/contexts/types/Shipping';
import CustomBreadcrumbs from '@/components/custom-breadcrumbs/custom-breadcrumbs';
import { useRouter } from '@/hooks';
import { useSnackbar } from 'notistack';
import { paths } from '@/routes/paths';
import { shippingMethodsList } from '@/shared/_mock/_shipping';
import LoadingButton from '@mui/lab/LoadingButton';
import FormProvider from '@/components/hook-form/form-provider';
import { Switch } from '@mui/material';


type Props = {
    id: String;
}

export default function EditFlatRate({ id }: Props) {
    const calculationType = Object.values(CalculationType);

    const shippingMethodTypes = Object.values(ShippingMethodType);

    const shippingMethod: ShippingMethod | undefined = shippingMethodsList.find(method => method.id === id);

    const router = useRouter();

    const mdUp = useResponsive('up', 'md');
    const { enqueueSnackbar } = useSnackbar();

    const NewshippingMethodSchema = Yup.object().shape({
        name: Yup.string().required('Nom est requis'),
        description: Yup.string().required("Description est requis"),
    });

    const defaultValues = useMemo(() => ({
        name: shippingMethod?.name || '',
        description: shippingMethod?.description || '',
    }), [shippingMethod]);

    const methods = useForm({
        resolver: yupResolver(NewshippingMethodSchema),
        defaultValues,
    });

    const {
        reset,
        watch,
        setValue,
        handleSubmit,
        formState: { isSubmitting },
    } = methods;

    const values = watch();

    useEffect(() => {
        if (shippingMethod) {
            reset(defaultValues);
        }
        console.log(defaultValues)
    }, [shippingMethod, defaultValues, reset])

    const onSubmit = handleSubmit(async (data) => {
        try {
            await new Promise((resolve) => setTimeout(resolve, 500));
            reset();
            enqueueSnackbar('Create success!', { variant: 'success' });
            router.push(paths.dashboard.shipping.general.shippingMethod.shippingMethodList);
            console.info('Form Data:', data);
        } catch (error) {
            console.error(error);
            enqueueSnackbar('Failed to create this new shipping method.', { variant: 'error' });
        }
    });

    const [checked, setChecked] = useState(true);

    const handleChange = (event: React.ChangeEvent<HTMLInputElement>) => {
        setChecked(event.target.checked);
    };

    const renderHead = (
        <Grid md={12}>
            <CustomBreadcrumbs
                heading="Modifier méthode de livraison"
                links={[
                    { name: 'Dashboard', href: paths.dashboard.root, },
                    { name: "Méthodes de livraison", href: paths.dashboard.shipping.general.shippingMethod.shippingMethodList, },
                    { name: shippingMethod?.type },
                ]}
                sx={{
                    mb: { xs: 3, md: 5 },
                }}
            />
        </Grid>
    );


    const renderForm = (
        <>
            {
                mdUp && (
                    <Grid md={4}>
                        <Typography variant="body2" sx={{ color: 'text.secondary' }}>
                            Saisissez les informations du méthode de livraison de type Forfait
                        </Typography>
                    </Grid>
                )
            }
            <Grid xs={12} md={8}>
                <Card>
                    <Stack spacing={3} sx={{ p: 3 }}>
                        <RHFTextField name="name" label="Nom de la méthode" />
                        <RHFTextField name="description" label="Description" placeholder="Description..." multiline rows={4} />
                        <Box style={{ display: 'flex', alignItems: 'center' }}><Typography>Activé</Typography> <Switch name='active' inputProps={{ 'aria-label': 'controlled' }} /></Box>
                        <RHFAutocomplete
                            disabled
                            name="type"
                            label="Type de méthode"
                            placeholder="Type de méthode"
                            value={shippingMethod?.type}
                            disableClearable
                            options={shippingMethodTypes}
                            getOptionLabel={(option) => option}
                            renderOption={(props, option) => (
                                <li {...props} key={option}>
                                    {option}
                                </li>
                            )}
                            renderTags={(selected, getTagProps) =>
                                selected.map((option, index) => (
                                    <Chip
                                        {...getTagProps({ index })}
                                        key={option}
                                        label={option}
                                        size="small"
                                        color="success"
                                        variant="soft"
                                    />
                                ))
                            }
                        />
                    </Stack>
                </Card>

                <Card sx={{mt: 3}}>
                    <Stack spacing={3} sx={{ p: 3 }}>
                        <FormControl>
                            <FormLabel id="demo-row-radio-buttons-group-label">État de la TVA</FormLabel>
                            <RadioGroup
                                row
                                aria-labelledby="demo-row-radio-buttons-group-label"
                                defaultValue="taxable"
                                name="row-radio-buttons-group"
                            >
                                <FormControlLabel value="taxable" control={<Radio />} label="Taxable" defaultChecked/>
                                <FormControlLabel value="aucune" control={<Radio />} label="Aucune" />
                            </RadioGroup>
                        </FormControl>
                        <RHFTextField
                            name="cost"
                            label="Coût"
                            placeholder="0.00"
                            type="number"
                            InputLabelProps={{ shrink: true }}
                            InputProps={{
                                startAdornment: (
                                    <InputAdornment position="start">
                                        <Box component="span" sx={{ color: 'text.disabled' }}>
                                            €
                                        </Box>
                                    </InputAdornment>
                                ),
                            }}
                        />
                        <FormLabel>Coûts par classe d’expédition</FormLabel>
                        <RHFTextField
                            name="costHeavyShippingClass"
                            label="Coût pour la classe d’expédition « Lourd »"
                            placeholder="0.00"
                            type="number"
                            InputLabelProps={{ shrink: true }}
                            InputProps={{
                                startAdornment: (
                                    <InputAdornment position="start">
                                        <Box component="span" sx={{ color: 'text.disabled' }}>
                                            €
                                        </Box>
                                    </InputAdornment>
                                ),
                            }}
                        />
                        <RHFTextField
                            name="CostNoShippingClass"
                            label="Coût pour aucune classe d’expédition"
                            placeholder="0.00"
                            type="number"
                            InputLabelProps={{ shrink: true }}
                            InputProps={{
                                startAdornment: (
                                    <InputAdornment position="start">
                                        <Box component="span" sx={{ color: 'text.disabled' }}>
                                            €
                                        </Box>
                                    </InputAdornment>
                                ),
                            }}
                        />
                        <RHFAutocomplete
                            name="calculationType"
                            label="Type de calcul"
                            placeholder="+ Type de calcul"
                            disableClearable
                            options={calculationType}
                            getOptionLabel={(option) => option}
                            renderOption={(props, option) => (
                                <li {...props} key={option}>
                                    {option}
                                </li>
                            )}
                            renderTags={(selected, getTagProps) =>
                                selected.map((option, index) => (
                                    <Chip
                                        {...getTagProps({ index })}
                                        key={option}
                                        label={option}
                                        size="small"
                                        color="info"
                                        variant="soft"
                                    />
                                ))
                            }
                        />
                    </Stack>
                </Card>
            </Grid>
        </>
    );


    const renderActions = (
        <>
            {mdUp && <Grid md={4} />}
            <Grid xs={12} md={8} sx={{ display: 'flex', justifyContent: 'end', py: 4 }}>
                <LoadingButton
                    type="submit"
                    variant="contained"
                    size="large"
                    loading={isSubmitting}
                    sx={{ ml: 2 }}
                >Enregistrer</LoadingButton>
            </Grid>
        </>
    );


    return (
        <>
            <FormProvider methods={methods} onSubmit={onSubmit}>
                <Grid container spacing={3}>
                    {renderHead}
                    {renderForm}
                    {renderActions}
                </Grid>
            </FormProvider>
        </>
    );
}
