"use client"

import { useState } from 'react';
import * as Yup from 'yup';
import { useForm } from 'react-hook-form';
import { yupResolver } from '@hookform/resolvers/yup';
import { useMemo } from 'react';
import Grid from '@mui/material/Unstable_Grid2';
import Card from '@mui/material/Card';
import Stack from '@mui/material/Stack';
import Chip from '@mui/material/Chip';
import Button from '@mui/material/Button';
import Typography from '@mui/material/Typography';
import Switch from '@mui/material/Switch';
import Box from '@mui/material/Box';
import DeleteIcon from '@mui/icons-material/Delete';
import InputAdornment from '@mui/material/InputAdornment';
import TextField from '@mui/material/TextField';
import Table from '@mui/material/Table';
import TableBody from '@mui/material/TableBody';
import EditIcon from '@mui/icons-material/Edit';
import TableCell from '@mui/material/TableCell';
import AddIcon from '@mui/icons-material/Add';
import TableContainer from '@mui/material/TableContainer';
import TableHead from '@mui/material/TableHead';
import TableRow from '@mui/material/TableRow';
import Paper from '@mui/material/Paper';
import { useResponsive } from '@/hooks/use-responsive';
import { useSnackbar } from '@/components/snackbar';
import FormProvider, {
    RHFTextField,
    RHFAutocomplete,
} from '@/components/hook-form';
import { paths } from '@/routes/paths';
import LoadingButton from '@mui/lab/LoadingButton';
import { CalculationType, MondialRelayMode, ShippingMethod, ShippingMethodType } from '@/contexts/types/Shipping'

import { shippingMethodsList } from '@/shared/_mock/_shipping';
import { IconButton, Modal, TablePagination } from '@mui/material';
import CustomBreadcrumbs from '@/components/custom-breadcrumbs/custom-breadcrumbs';


interface RowDataCostDelivery {
    weightCart: number | null;
    weightPackaging: number | null;
    cost: number | null;
}

type Props = {
    id: String;
}

export default function EditMondialRelay({ id }: Props) {
    const shippingMethodTypes = Object.values(ShippingMethodType);

    const shippingMethod: ShippingMethod | undefined = shippingMethodsList.find(method => method.id === id);

    const mdUp = useResponsive('up', 'md');
    const { enqueueSnackbar } = useSnackbar();

    const NewShippingMethodSchema = Yup.object().shape({
        name: Yup.string().required('Name is required'),
        description: Yup.string().required('Description is required'),
    });

    interface DefaultValues {
        name: string;
        description: string;
        additionalAttributes?: {
            [key: string]: any;
        };
    }

    const defaultValues: DefaultValues = useMemo(() => {
        let additionalAttributes: any = {};

        switch (shippingMethod?.type) {
            case ShippingMethodType.FreeShipping:
                additionalAttributes = {
                    requires: shippingMethod.data.requires || '',
                    minimumOrderAmount: shippingMethod.data.minimumOrderAmount || 0,
                    applyMinOrderRuleBeforePromoCodeDiscount: shippingMethod.data.applyMinOrderRuleBeforePromoCodeDiscount || ''
                };
                break;
            case ShippingMethodType.FlatRate:
                additionalAttributes = {
                    tvaStatut: shippingMethod.data.tvaStatut || false,
                    cost: shippingMethod.data.cost || 0,
                    costHeavyShippingClass: shippingMethod.data.costHeavyShippingClass || 0,
                    CostNoShippingClass: shippingMethod.data.CostNoShippingClass || 0,
                    calculationType: shippingMethod.data.calculationType || CalculationType.ByClass
                };
                break;
            case ShippingMethodType.MondialRelay:
                additionalAttributes = {
                    amountFreeShipping: shippingMethod.data.amountFreeShipping || 0,
                    nameFreeShipping: shippingMethod.data.nameFreeShipping || '',
                    modeMondialRelay: shippingMethod.data.modeMondialRelay || MondialRelayMode.MED,
                    listCostDelivery: shippingMethod.data.listCostDelivery || []
                };
                break;
            default:
                break;
        }

        return {
            name: shippingMethod?.name || '',
            description: shippingMethod?.description || '',
            type: shippingMethod?.type || '',
            additionalAttributes: additionalAttributes
        };
    }, [shippingMethod]);

    const methods = useForm({
        resolver: yupResolver(NewShippingMethodSchema),
        defaultValues,
    });

    const {
        watch,
        control,
        setValue,
        handleSubmit,
        formState: { isSubmitting },
    } = methods;

    const values = watch();

    const onSubmit = handleSubmit(async (data) => {
        try {
            await new Promise((resolve) => setTimeout(resolve, 500));
            enqueueSnackbar('Create success!', { variant: 'success' });
            console.info('Form Data:', data);
        } catch (error) {
            console.error(error);
            enqueueSnackbar('Failed to create this new shipping method.', { variant: 'error' });
        }
    });

    const [selectedType, setSelectedType] = useState<string | null>(null);

    const mondialRelayMode = Object.values(MondialRelayMode);
    const calculationType = Object.values(CalculationType);

    const [checked, setChecked] = useState(true);

    const handleChange = (event: React.ChangeEvent<HTMLInputElement>) => {
        setChecked(event.target.checked);
    };

    const dataListCostDelivery: { weightCart: number; weightPackaging: number; cost: number }[] = [
        { weightCart: 3.5, weightPackaging: 1, cost: 100.0 },
        { weightCart: 6, weightPackaging: 2, cost: 420.0 },
        { weightCart: 10, weightPackaging: 2.4, cost: 850.0 },
        { weightCart: 2, weightPackaging: 1, cost: 50.0 },
        { weightCart: 8, weightPackaging: 3, cost: 700.0 },
        { weightCart: 12, weightPackaging: 5, cost: 1100.0 }
    ];

    const [costPage, setCostPage] = useState(0);
    const [costRowsPerPage, setCostRowsPerPage] = useState(5);

    const handleChangeCostPage = (event: React.MouseEvent<HTMLButtonElement> | null, newPage: number) => {
        setCostPage(newPage);
    };

    const handleChangeCostRowsPerPage = (event: React.ChangeEvent<HTMLInputElement>) => {
        setCostRowsPerPage(parseInt(event.target.value, 10));
        setCostPage(0);
    };

    const [openAddModal, setOpenAddModal] = useState(false);
    const [openEditModal, setOpenEditModal] = useState(false);
    const [editIndex, setEditIndex] = useState<number | null>(null);
    const [formData, setFormData] = useState<RowDataCostDelivery>({
        weightCart: null as number | null,
        weightPackaging: null as number | null,
        cost: null as number | null,
    });

    const handleAddClick = () => {
        setFormData({
            weightCart: null,
            weightPackaging: null,
            cost: null
        });
        setOpenAddModal(true);
    };

    const handleEditClick = (index: number) => {
        setEditIndex(index);
        setOpenEditModal(true);
        const row = dataListCostDelivery[index];
        if (row) {
            setFormData({
                weightCart: row.weightCart,
                weightPackaging: row.weightPackaging,
                cost: row.cost
            });
        }
    };

    const handleCloseEditModal = () => {
        setOpenEditModal(false);
    };


    const handleCloseAddModal = () => {
        setOpenAddModal(false);
    };


    const handleAddFormSubmit = () => {
        setOpenAddModal(false);
    };

    const handleEditFormSubmit = () => {
        setOpenEditModal(false);

    };
    const [dataListCost, setDataListCost] = useState(dataListCostDelivery);

    const renderHead = (
        <Grid md={12}>
            <CustomBreadcrumbs
                heading="Modifier méthode de livraison"
                links={[
                    { name: 'Dashboard', href: paths.dashboard.root, },
                    { name: "Méthodes de livraison", href: paths.dashboard.shipping.general.shippingMethod.shippingMethodList, },
                    { name: shippingMethod?.type },
                ]}
                sx={{
                    mb: { xs: 3, md: 5 },
                }}
            />
        </Grid>
    );

    const renderBasic = (
        <>
            {mdUp && (
                <Grid md={4}>
                    <Typography variant="body2" sx={{ color: 'text.secondary' }}>
                        Saisissez les informations du méthode de livraison de type Modial Relay - InPost
                    </Typography>
                </Grid>
            )}

            <Grid xs={12} md={8}>
                <Card>
                    <Stack spacing={3} sx={{ p: 3 }}>
                        <RHFTextField name="name" label="Nom de la méthode" />

                        <RHFTextField name="description" label="Description" placeholder="Description..." multiline rows={4} />
                        <Box style={{ display: 'flex', alignItems: 'center' }}><Typography>Activé</Typography> <Switch name='active' inputProps={{ 'aria-label': 'controlled' }} /></Box>
                        <RHFAutocomplete
                            disabled
                            name="type"
                            label="Type de méthode"
                            placeholder="+ Type de méthode"
                            disableClearable
                            options={shippingMethodTypes}
                            getOptionLabel={(option) => option}
                            renderOption={(props, option) => (
                                <li {...props} key={option}>
                                    {option}
                                </li>
                            )}
                            renderTags={(selected, getTagProps) =>
                                selected.map((option, index) => (
                                    <Chip
                                        {...getTagProps({ index })}
                                        key={option}
                                        label={option}
                                        size="small"
                                        color="success"
                                        variant="soft"
                                    />
                                ))
                            }
                        />
                    </Stack>
                </Card>

                <Card sx={{ mt: 3 }}>
                    <Stack spacing={3} sx={{ p: 3 }}>
                        <RHFTextField
                            name="amountFreeShipping"
                            label="Offrir la livraison à partir de ce montant (optionnel)"
                            placeholder="0.00"
                            type="number"
                            InputLabelProps={{ shrink: true }}
                            InputProps={{
                                startAdornment: (
                                    <InputAdornment position="start">
                                        <Box component="span" sx={{ color: 'text.disabled' }}>
                                            €
                                        </Box>
                                    </InputAdornment>
                                ),
                            }}
                        />
                        <RHFTextField name="nameFreeShipping" label="Nom si la livraison est offerte" />

                        <RHFAutocomplete
                            name="mondialRelayMode"
                            label="Mode de livraison Mondial Relay - InPost"
                            disableClearable
                            options={mondialRelayMode}
                            defaultValue={MondialRelayMode.MED}
                            getOptionLabel={(option) => option}
                            renderOption={(props, option) => (
                                <li {...props} key={option}>
                                    {option}
                                </li>
                            )}
                            renderTags={(selected, getTagProps) =>
                                selected.map((option, index) => (
                                    <Chip
                                        {...getTagProps({ index })}
                                        key={option}
                                        label={option}
                                        size="small"
                                        color="info"
                                        variant="soft"
                                    />
                                ))
                            }
                        />
                        <Button variant="outlined" size='small' sx={{ color: '#808080', width: '20%' }} onClick={handleAddClick}>
                            <AddIcon /> Frais de livraison
                        </Button>
                        <TableContainer component={Paper} sx={{ maxHeight: 420 }}>
                            <Table sx={{ minWidth: 650 }} aria-label="simple table">
                                <TableHead>
                                    <TableRow>
                                        <TableCell align='center'>Poids du panier jusqu’à</TableCell>
                                        <TableCell align='center'>Poids d’emballage (optionnel)</TableCell>
                                        <TableCell align='center'>Prix</TableCell>
                                        <TableCell align='center'>Actions</TableCell>
                                    </TableRow>
                                </TableHead>
                                <TableBody>
                                    {dataListCost
                                        .slice(costPage * costRowsPerPage, costPage * costRowsPerPage + costRowsPerPage)
                                        .map((row, index) => (
                                            <TableRow
                                                key={index}
                                                sx={{ '&:last-child td, &:last-child th': { border: 0 } }}
                                            >
                                                <TableCell component="th" scope="row" align='center'>
                                                    {row.weightCart} Kg
                                                </TableCell>
                                                <TableCell align='center'>{row.weightPackaging} Kg</TableCell>
                                                <TableCell align='center'>{row.cost} €</TableCell>
                                                <TableCell align='center'><Box><IconButton><EditIcon sx={{ color: 'green' }} onClick={() => handleEditClick(index)} /></IconButton><IconButton><DeleteIcon sx={{ color: 'red' }} /></IconButton></Box></TableCell>
                                            </TableRow>
                                        ))}
                                </TableBody>
                            </Table>
                        </TableContainer>
                        <TablePagination
                            rowsPerPageOptions={[5, 10, 25]}
                            component="div"
                            count={dataListCost.length}
                            rowsPerPage={costRowsPerPage}
                            page={costPage}
                            onPageChange={handleChangeCostPage}
                            onRowsPerPageChange={handleChangeCostRowsPerPage}
                        />
                    </Stack>
                    <Grid md={12}>
                        <Typography variant='caption' sx={{ color: 'text.secondary' }}>Indiquez ci-dessus le prix de livraison que vous facturez au client en fonction du poids total des produits dans son panier. Les poids doivent être saisis de manière croissante. Le poids d’emballage est ajouté au poids des produits avant de calculer le prix. Par exemple, si la marchandise pèse 0,950kg et le poids de l’emballage est de 0,1kg, le prix sera calculé sur la base de 1,050kg.</Typography>
                    </Grid>
                </Card>
            </Grid>
            <Modal open={openAddModal} onClose={handleCloseAddModal}>
                <Box sx={{ position: 'absolute', top: '50%', left: '50%', transform: 'translate(-50%, -50%)', width: 500, bgcolor: 'background.paper', boxShadow: 24, p: 4, borderRadius: 3 }}>
                    <Typography variant="h6" align="center" gutterBottom sx={{ mt: 2, mb: 3 }}>Ajouter frais de livraison</Typography>

                    <TextField
                        name='weightCart'
                        label="Poids du panier jusqu'à"
                        type="number"
                        placeholder="0.00"
                        value={formData.weightCart}
                        onChange={(e) => setFormData({ ...formData, weightCart: parseInt(e.target.value, 10) })}
                        sx={{ m: 1, width: '97%' }}
                        InputProps={{
                            startAdornment: <InputAdornment position="start">kg</InputAdornment>,
                        }}
                    />
                    <TextField
                        name="weightPackaging"
                        label="Poids d'emballage (optionnel)"
                        type="number"
                        placeholder="0.00"
                        value={formData.weightPackaging}
                        onChange={(e) => setFormData({ ...formData, weightPackaging: parseInt(e.target.value, 10) })}
                        sx={{ m: 1, width: '97%' }}
                        InputProps={{
                            startAdornment: <InputAdornment position="start">kg</InputAdornment>,
                        }}
                    />
                    <RHFTextField
                        name="cost"
                        label="Prix"
                        placeholder="0.00"
                        type="number"
                        value={formData.cost}
                        onChange={(e) => setFormData({ ...formData, cost: parseInt(e.target.value, 10) })}
                        sx={{ m: 1, width: '97%' }}
                        InputLabelProps={{ shrink: true }}
                        InputProps={{
                            startAdornment: (
                                <InputAdornment position="start">
                                    <Box component="span" sx={{ color: 'text.disabled' }}>
                                        €
                                    </Box>
                                </InputAdornment>
                            ),
                        }}
                    />
                    <Box sx={{ display: 'flex', justifyContent: 'center', mt: 3 }}>
                        <Button onClick={() => setOpenAddModal(false)} sx={{ mt: 2, mb: 2, mr: 1 }} variant="outlined" color="inherit">Annuler</Button>
                        <Button onClick={handleAddFormSubmit} sx={{ mt: 2, mb: 2 }} variant="contained" color="inherit">Ajouter</Button>
                    </Box>
                </Box>
            </Modal>


            <Modal open={openEditModal} onClose={handleCloseEditModal}>
                <Box sx={{ position: 'absolute', top: '50%', left: '50%', transform: 'translate(-50%, -50%)', width: 500, bgcolor: 'background.paper', boxShadow: 24, p: 4, borderRadius: 3 }}>
                    <Typography variant="h6" align="center" gutterBottom sx={{ mt: 2, mb: 3 }}>Modifier frais de livraison</Typography>
                    <TextField
                        name='weightCart'
                        label="Poids du panier jusqu'à"
                        type="number"
                        placeholder="0.00"
                        value={formData.weightCart}
                        onChange={(e) => setFormData({ ...formData, weightCart: parseInt(e.target.value, 10) })}
                        sx={{ m: 1, width: '97%' }}
                        InputProps={{
                            startAdornment: <InputAdornment position="start">kg</InputAdornment>,
                        }}
                    />
                    <TextField
                        name='weightPackaging'
                        label="Poids d'emballage (optionnel)"
                        type="number"
                        placeholder="0.00"
                        value={formData.weightPackaging}
                        onChange={(e) => setFormData({ ...formData, weightPackaging: parseInt(e.target.value, 10) })}
                        sx={{ m: 1, width: '97%' }}
                        InputProps={{
                            startAdornment: <InputAdornment position="start">kg</InputAdornment>,
                        }}
                    />
                    <RHFTextField
                        name="cost"
                        label="Prix"
                        placeholder="0.00"
                        type="number"
                        value={formData.cost}
                        onChange={(e) => setFormData({ ...formData, cost: parseInt(e.target.value, 10) })}
                        sx={{ m: 1, width: '97%' }}
                        InputLabelProps={{ shrink: true }}
                        InputProps={{
                            startAdornment: (
                                <InputAdornment position="start">
                                    <Box component="span" sx={{ color: 'text.disabled' }}>
                                        €
                                    </Box>
                                </InputAdornment>
                            ),
                        }}
                    />
                    <Box sx={{ display: 'flex', justifyContent: 'center', mt: 3 }}>
                        <Button onClick={() => setOpenEditModal(false)} sx={{ mt: 2, mb: 2, mr: 1 }} variant="outlined" color="inherit">Annuler</Button>
                        <Button onClick={handleEditFormSubmit} sx={{ mt: 2, mb: 2 }} variant="contained" color="inherit">Modifier</Button>
                    </Box>
                </Box>
            </Modal>
        </>
    );

    const renderActions = (
        <>
            {mdUp && <Grid md={4} />}
            <Grid xs={12} md={8} sx={{ display: 'flex', justifyContent: 'end', py: 4 }}>
                <LoadingButton
                    type="submit"
                    variant="contained"
                    size="large"
                    loading={isSubmitting}
                    sx={{ ml: 2 }}
                >Enregistrer</LoadingButton>
            </Grid>
        </>
    );

    return (
        <FormProvider methods={methods} onSubmit={onSubmit}>
            <Grid container spacing={3}>
                {renderHead}
                {renderBasic}
                {renderActions}
            </Grid>
        </FormProvider>
    );
}
