import Button from '@mui/material/Button';
import Divider from '@mui/material/Divider';
import MenuItem from '@mui/material/MenuItem';
import TableRow from '@mui/material/TableRow';
import TableCell from '@mui/material/TableCell';
import Switch from '@mui/material/Switch';
import IconButton from '@mui/material/IconButton';
import Checkbox from '@mui/material/Checkbox';
import Typography from '@mui/material/Typography';
import ListItemText from '@mui/material/ListItemText';

import { useBoolean } from '@/hooks';

import Iconify from '@/shared/components/iconify';
import ConfirmDialog from '@/shared/components/confirm-dialog';
import CustomPopover, { usePopover } from '@/shared/components/custom-popover';
import { ShippingMethod } from '@/contexts/types/Shipping';
import React, { useState } from 'react';


type Props = {
  row: ShippingMethod;
  selected: boolean;
  selectedShippingMethod: ShippingMethod | null;
  onSelectRow: VoidFunction;
  onViewRow: VoidFunction;
  onEditRow: VoidFunction;
  onDeleteRow: VoidFunction;
};

export default function ShippingMethodTableRow({
  row,
  selected,
  onSelectRow,
  onViewRow,
  onEditRow,
  onDeleteRow,
}: Props) {
  const { name, active, description, type } = row;
  const confirm = useBoolean();
  const popover = usePopover();
  const openDetails = useBoolean();
  const [isChecked, setIsChecked] = useState(active);

  const handleToggle = () => {
    setIsChecked(!isChecked);
    row.active = !row.active;
  };


  return (
    <>
      <TableRow hover selected={selected} onClick={() => { onViewRow(); }} sx={{ width: '100%' }}>
        <TableCell sx={{ display: 'flex', alignItems: 'center', width: '280px' }}>
          <ListItemText
            disableTypography
            primary={
              <Typography variant="body2">
                {name}
              </Typography>
            }
          />
        </TableCell>

        <TableCell sx={{ width: '220px' }}>
          <Switch
            checked={isChecked}
            onChange={handleToggle}
            inputProps={{ 'aria-label': 'controlled' }}
          />
        </TableCell>

        <TableCell sx={{ width: '520px' }}>{description}</TableCell>

        <TableCell sx={{ display: 'flex', justifyContent: 'flex-end', alignItems: 'center', px: 1, textAlign: 'right' }}>
          <IconButton color={popover.open ? 'inherit' : 'default'} onClick={popover.onOpen}>
            <Iconify icon="eva:more-vertical-fill" />
          </IconButton>
        </TableCell>

      </TableRow>

      <CustomPopover
        open={popover.open}
        onClose={popover.onClose}
        arrow="right-top"
        sx={{ width: 160 }}
      >
        <MenuItem
          onClick={() => {
            onEditRow();
            popover.onClose();
          }}
          style={{ color: "green" }}
        >

          <Iconify icon="solar:pen-bold" />
          Edit
        </MenuItem>
        <MenuItem
          onClick={() => {
            confirm.onTrue();
            popover.onClose();
          }}
          sx={{ color: 'error.main' }}
        >
          <Iconify icon="solar:trash-bin-trash-bold" />
          Delete
        </MenuItem>

      </CustomPopover>


      <ConfirmDialog
        open={confirm.value}
        onClose={confirm.onFalse}
        title="Delete"
        content="Are you sure want to delete?"
        action={
          <Button variant="contained" color="error" onClick={onDeleteRow}>
            Delete
          </Button>
        }
      />
    </>
  );
}
