'use client';

import { useState, useCallback } from 'react';

import Card from '@mui/material/Card';
import Table from '@mui/material/Table';
import Stack from '@mui/material/Stack';
import Button from '@mui/material/Button';
import Grid from '@mui/material/Grid';
import Container from '@mui/material/Container';
import TableBody from '@mui/material/TableBody';
import TableCell from '@mui/material/TableCell';
import TableHead from '@mui/material/TableHead';
import { useTheme } from '@mui/material/styles';
import TableContainer from '@mui/material/TableContainer';
import LocalizationProvider from '@/shared/locales/localization-provider';
import { paths } from '@/routes/paths';
import { useRouter } from '@/hooks';

import { useBoolean } from '@/hooks';

import { shippingMethodsList } from '@/shared/_mock/_shipping';

import Iconify from '@/shared/components/iconify';
import Scrollbar from '@/shared/components/scrollbar';
import { useSnackbar } from '@/components/snackbar';
import { useSettingsContext } from '@/shared/components/settings';
import CustomBreadcrumbs from '@/components/custom-breadcrumbs';
import { useTable, emptyRows, TableNoData, TablePaginationCustom, TableEmptyRows, } from '@/shared/components/table';

import { ShippingMethod } from '@/contexts/types/Shipping';
import ShippingMethodTableRow from './ShippingMethodTableRow';
import RouterLink from '@/routes/router-link';

import { Reorder } from 'framer-motion';



export default function ShippingMethodTableView() {
    const { enqueueSnackbar } = useSnackbar();

    const theme = useTheme();

    const settings = useSettingsContext();
    const router = useRouter();
    const table = useTable({ defaultOrderBy: 'title' });
    const confirm = useBoolean();
    const [tableData, setTableData] = useState<ShippingMethod[]>(shippingMethodsList);


    const dataInPage = tableData.slice(
        table.page * table.rowsPerPage,
        table.page * table.rowsPerPage + table.rowsPerPage
    );

    const denseHeight = table.dense ? 56 : 56 + 20;


    const notFound = (!tableData.length) || !tableData.length;

    //----------------------------------------------------------------------------------  

    const handleDeleteRow = useCallback(
        (id: string) => {
            const deleteRow = tableData.filter((row) => row.id !== id);

            enqueueSnackbar('Delete success!');

            setTableData(deleteRow);

            table.onUpdatePageDeleteRow(dataInPage.length);
        },
        [dataInPage.length, enqueueSnackbar, table, tableData]
    );

    const handleEditRow = useCallback(
        (id: string) => {
            router.push(paths.dashboard.shipping.general.shippingMethod.edit(id));
        },
        [router]
    );

    //----------------view details--------------

    const [selectedShippingMethod, setSelectedShippingMethod] = useState<ShippingMethod | null>(null);
    const handelSelectedShippingMethod = (shippingArea: any) => {
        setSelectedShippingMethod(shippingArea);
    };
    const handleViewRow = useCallback(
        (id: string) => {
            const shippingArea = tableData.find((a) => a.id === id);
            handelSelectedShippingMethod(shippingArea)
        },
        [tableData]
    );

    //----------------Change ordre of row---------------
    const handleReorder = (newOrder: number[]) => {
        // Create a new array with the shipping areas reordered based on the newOrder array
        const reorderedData = newOrder.map(index => tableData[index]);
        setTableData(reorderedData);
    };

    //-----------tab-------------------
    const [value, setValue] = useState('1');

    const handleChange = (event: React.SyntheticEvent, newValue: string) => {
        setValue(newValue);
    };

    /////////////////////////////

    return (
        <>
            <LocalizationProvider >
                <Container maxWidth={settings.themeStretch ? false : 'lg'}>
                    <Grid container alignItems="center" justifyContent="space-between">
                        <Grid item>
                            <CustomBreadcrumbs
                                heading="Méthodes de livraison"
                                links={[
                                    {
                                        name: 'Dashboard',
                                        href: paths.dashboard.root,
                                    },
                                    {
                                        name: "Livraison",
                                    }
                                ]}
                                sx={{
                                    mb: { xs: 3, md: 5 },
                                }}
                            />
                        </Grid>
                        <Grid item>
                            <Stack
                                spacing={3}
                                justifyContent="space-between"
                                alignItems={{ xs: 'flex-end', sm: 'center' }}
                                direction={{ xs: 'column', sm: 'row' }}
                            >
                                <Button
                                    component={RouterLink}
                                    href={paths.dashboard.shipping.general.shippingMethod.createShippingMethod}
                                    variant="contained"
                                    color='success'
                                    startIcon={<Iconify icon="mingcute:add-line" />}
                                >
                                    Ajouter
                                </Button>
                            </Stack>
                        </Grid>
                    </Grid>

                    <Card>
                        <TableContainer sx={{ position: 'relative', overflow: 'unset' }}>
                            <TableHead sx={{ width: '100%' }} >
                                <TableCell width={'30%'}>Nom</TableCell>
                                <TableCell width={'20%'}>Activé</TableCell>
                                <TableCell width={'45%'}>Description</TableCell>
                                <TableCell width={'5%'}>Actions</TableCell>
                            </TableHead>

                            <Scrollbar>
                                <Table size={table.dense ? 'small' : 'medium'} sx={{ minWidth: 800 }}>
                                    <TableBody>
                                        <Reorder.Group
                                            axis="y" // Vertical reordering
                                            values={tableData.map((_, index) => index)} // Array of indices representing the current order
                                            onReorder={handleReorder} // Callback function to handle reordering
                                        >
                                            {tableData
                                                .slice(
                                                    table.page * table.rowsPerPage,
                                                    table.page * table.rowsPerPage + table.rowsPerPage
                                                )
                                                .map((row, index) => (
                                                    <Reorder.Item key={row.id} value={index}> {/* Use index as value for Reorder.Item */}
                                                        <ShippingMethodTableRow
                                                            key={row.id}
                                                            row={row}
                                                            selected={table.selected.includes(row.id)}
                                                            onSelectRow={() => table.onSelectRow(row.id)}
                                                            onViewRow={() => handleViewRow(row.id)}
                                                            selectedShippingMethod={selectedShippingMethod}
                                                            onEditRow={() => handleEditRow(row.id)}
                                                            onDeleteRow={() => handleDeleteRow(row.id)}
                                                        />
                                                    </Reorder.Item>
                                                ))}
                                        </Reorder.Group>

                                        <TableEmptyRows
                                            height={denseHeight}
                                            emptyRows={emptyRows(table.page, table.rowsPerPage, tableData.length)}
                                        />

                                        <TableNoData notFound={notFound} />
                                    </TableBody>
                                </Table>
                            </Scrollbar>
                        </TableContainer>

                        <TablePaginationCustom
                            count={tableData.length}
                            page={table.page}
                            rowsPerPage={table.rowsPerPage}
                            onPageChange={table.onChangePage}
                            onRowsPerPageChange={table.onChangeRowsPerPage}
                            dense={table.dense}
                            onChangeDense={table.onChangeDense}
                        />
                    </Card>


                </Container>
            </LocalizationProvider>
        </>

    );
}
