"use client";
import * as Yup from "yup";
import { useMemo, useCallback } from "react";
import { useForm, Controller } from "react-hook-form";
import { yupResolver } from "@hookform/resolvers/yup";

import Box from "@mui/material/Box";
import Card from "@mui/material/Card";
import Stack from "@mui/material/Stack";
import Button from "@mui/material/Button";
import Switch from "@mui/material/Switch";
import Grid from "@mui/material/Unstable_Grid2";
import Typography from "@mui/material/Typography";
import LoadingButton from "@mui/lab/LoadingButton";
import FormControlLabel from "@mui/material/FormControlLabel";

import { paths } from "@/routes/paths";
import { useRouter } from "@/hooks";

import { fData } from "@/utils/format-number";

import { countries } from "@/shared/assets/data";

import Label from "@/shared/components/label";
import { useSnackbar } from "@/components/snackbar";
import FormProvider, {
  RHFTextField,
  RHFUploadAvatar,
  RHFAutocomplete,
} from "@/shared/components/hook-form";

import { IUserItem } from "@/shared/types/user";

// ----------------------------------------------------------------------

type Props = {
  currentUser?: IUserItem;
};

export default function SupplierNewEditForm({ currentUser }: Props) {
  const router = useRouter();

  const { enqueueSnackbar } = useSnackbar();

  const NewUserSchema = Yup.object().shape({
    name: Yup.string().required("Nom requis"),
    email: Yup.string()
      .required("Email requis")
      .email("L'email doit être une adresse email valide"),
    phoneNumber: Yup.string().required("Numéro de téléphone requis"),
    address: Yup.string().required("Adresse requise"),
    country: Yup.string().required("Pays requis"),
    state: Yup.string().required("État requis"),
    city: Yup.string().required("Ville requise"),
    zipCode: Yup.string().required("Code postal requis"),
    avatarUrl: Yup.mixed<any>().nullable().required("Avatar requis"),

    // not required
    status: Yup.string(),
    isVerified: Yup.boolean(),
  });

  const defaultValues = useMemo(
    () => ({
      name: currentUser?.name || "",
      city: currentUser?.city || "",
      email: currentUser?.email || "",
      state: currentUser?.state || "",
      status: currentUser?.status || "",
      address: currentUser?.address || "",
      country: currentUser?.country || "",
      zipCode: currentUser?.zipCode || "",

      avatarUrl: currentUser?.avatarUrl || null,
      phoneNumber: currentUser?.phoneNumber || "",
      isVerified: currentUser?.isVerified || true,
    }),
    [currentUser]
  );

  const methods = useForm({
    resolver: yupResolver(NewUserSchema),
    defaultValues,
  });

  const {
    reset,
    watch,
    control,
    setValue,
    handleSubmit,
    formState: { isSubmitting },
  } = methods;

  const values = watch();

  const onSubmit = handleSubmit(async (data) => {
    try {
      await new Promise((resolve) => setTimeout(resolve, 500));
      reset();
      enqueueSnackbar(
        currentUser ? "Mise à jour réussie !" : "Création réussie !"
      );
      router.push(paths.dashboard.supplier.root);
      console.info("DONNÉES", data);
    } catch (error) {
      console.error(error);
    }
  });

  return (
    <FormProvider methods={methods} onSubmit={onSubmit}>
      <Grid container spacing={3}>
        <Grid xs={12} md={8}>
          <Card sx={{ p: 3 }}>
            <Box
              rowGap={3}
              columnGap={2}
              display="grid"
              gridTemplateColumns={{
                xs: "repeat(1, 1fr)",
                sm: "repeat(2, 1fr)",
              }}
            >
              <RHFTextField name="name" label="Nom complet" />
              <RHFTextField name="email" label="Adresse email" />
              <RHFTextField name="phoneNumber" label="Numéro de téléphone" />

              <RHFAutocomplete
                name="country"
                type="country"
                label="Pays"
                placeholder="Choisissez un pays"
                fullWidth
                options={countries.map((option) => option.label)}
                getOptionLabel={(option) => option}
              />
              <RHFTextField name="secteur" label="Sécteur" />
              <RHFTextField name="zone" label="Zone Fournisseur" />
              <RHFTextField name="city" label="Référence Fournisseur" />
              <RHFTextField name="state" label="État/Région" />
              <RHFTextField name="city" label="Ville" />
              <RHFTextField name="address" label="Adresse" />
              <RHFTextField name="zipCode" label="Code postal" />
            </Box>

            <Stack alignItems="flex-end" sx={{ mt: 3 }}>
              <LoadingButton
                type="submit"
                variant="contained"
                loading={isSubmitting}
              >
                {!currentUser
                  ? "Créer un utilisateur"
                  : "Sauvegarder les modifications"}
              </LoadingButton>
            </Stack>
          </Card>
        </Grid>
      </Grid>
    </FormProvider>
  );
}
