import sum from "lodash/sum";
import { useEffect, useCallback } from "react";
import { useFieldArray, useFormContext } from "react-hook-form";

import Box from "@mui/material/Box";
import Stack from "@mui/material/Stack";
import Button from "@mui/material/Button";
import Divider from "@mui/material/Divider";

import Typography from "@mui/material/Typography";
import InputAdornment from "@mui/material/InputAdornment";
import { inputBaseClasses } from "@mui/material/InputBase";

import { fCurrency } from "@/utils/format-number";

import { INVOICE_SERVICE_OPTIONS } from "@/shared/_mock";

import Iconify from "@/components/iconify";
import { RHFTextField } from "@/shared/components/hook-form";

import { IInvoiceItem } from "@/shared/types/invoice";
import Autocomplete from "@mui/material/Autocomplete";
import TextField from "@mui/material/TextField";
// ----------------------------------------------------------------------

export default function OrderNewEditDetails() {
  const { control, setValue, watch, resetField } = useFormContext();

  const { fields, append, remove } = useFieldArray({
    control,
    name: "items",
  });

  const values = watch();

  const totalOnRow = values.items.map(
    (item: IInvoiceItem) => item.quantity * item.price
  );

  const subTotal = sum(totalOnRow);

  const totalAmount = subTotal - values.discount + values.taxes;

  useEffect(() => {
    setValue("totalAmount", totalAmount);
  }, [setValue, totalAmount]);

  const handleAdd = () => {
    append({
      description: "",
      service: "",
      quantity: 1,
      price: 0,
      total: 0,
    });
  };

  const handleRemove = (index: number) => {
    remove(index);
  };

  const handleSelectProduct = useCallback(
    (index: number, option: string) => {
      if (option) {
        // Product selected
        const selectedService = INVOICE_SERVICE_OPTIONS.find(
          (service) => service.name === option
        );
        if (selectedService) {
          setValue(`items[${index}].price`, selectedService.price || 0);
          setValue(`items[${index}].service`, option);
          setValue(
            `items[${index}].total`,
            values.items.map(
              (item: IInvoiceItem) => item.quantity * item.price
            )[index]
          );
          [setValue, values.items];
        }
      } else {
        // Deselect product
        resetField(`items[${index}].quantity`);
        resetField(`items[${index}].price`);
        resetField(`items[${index}].total`);
        [resetField];
      }

      // Recalculate total amount
      const updatedItems = values.items.map((item: IInvoiceItem) => ({
        ...item,
        total: item.quantity * item.price,
      }));
      const newSubTotal = sum(
        updatedItems.map((item: IInvoiceItem) => item.total)
      );
      const newTotalAmount = newSubTotal - values.discount + values.taxes;
      setValue("totalAmount", newTotalAmount);
    },
    [setValue, values.items, values.discount, values.taxes]
  );

  const handleChangeQuantity = useCallback(
    (
      event: React.ChangeEvent<HTMLInputElement | HTMLTextAreaElement>,
      index: number
    ) => {
      setValue(`items[${index}].quantity`, Number(event.target.value));
      setValue(
        `items[${index}].total`,
        values.items.map((item: IInvoiceItem) => item.quantity * item.price)[
          index
        ]
      );
    },
    [setValue, values.items]
  );

  const handleChangePrice = useCallback(
    (
      event: React.ChangeEvent<HTMLInputElement | HTMLTextAreaElement>,
      index: number
    ) => {
      setValue(`items[${index}].price`, Number(event.target.value));
      setValue(
        `items[${index}].total`,
        values.items.map((item: IInvoiceItem) => item.quantity * item.price)[
          index
        ]
      );
    },
    [setValue, values.items]
  );

  const renderTotal = (
    <Stack
      spacing={2}
      alignItems="flex-end"
      sx={{ mt: 3, textAlign: "right", typography: "body2" }}
    >
      <Stack direction="row">
        <Box sx={{ color: "text.secondary" }}>Sous-total</Box>
        <Box sx={{ width: 160, typography: "subtitle2" }}>
          {fCurrency(subTotal) || "-"}
        </Box>
      </Stack>

      <Stack direction="row">
        <Box sx={{ color: "text.secondary" }}>Remise</Box>
        <Box
          sx={{
            width: 160,
            ...(values.discount && { color: "error.main" }),
          }}
        >
          {values.discount ? `- ${fCurrency(values.discount)}` : "-"}
        </Box>
      </Stack>

      <Stack direction="row" sx={{ typography: "subtitle1" }}>
        <Box>Total</Box>
        <Box sx={{ width: 160 }}>{fCurrency(totalAmount) || "-"}</Box>
      </Stack>
    </Stack>
  );

  return (
    <Box sx={{ p: 3 }}>
      <Typography variant="h6" sx={{ color: "text.disabled", mb: 3 }}>
        Détails :
      </Typography>

      <Stack
        divider={<Divider flexItem sx={{ borderStyle: "dashed" }} />}
        spacing={3}
      >
        {fields.map((item, index) => (
          <Stack key={item.id} alignItems="flex-end" spacing={1.5}>
            <Stack
              direction={{ xs: "column", md: "row" }}
              spacing={2}
              sx={{ width: 1 }}
            >
              <Autocomplete
                id="produit"
                autoHighlight
                size="small"
                disablePortal
                options={INVOICE_SERVICE_OPTIONS}
                getOptionLabel={(option) => option.name || option.id}
                isOptionEqualToValue={(option, value) =>
                  option.name === value.name || option.id === value.id
                }
                onChange={(event, option) =>
                  handleSelectProduct(index, option?.name || "")
                }
                renderOption={(props, option) => (
                  <Box
                    component="li"
                    sx={{ "& > img": { mr: 2, flexShrink: 0 } }}
                    {...props}
                  >
                    <img loading="lazy" width="40" src={option.image} alt="" />
                    {option.name} +{option.id}
                  </Box>
                )}
                renderInput={(params) => (
                  <TextField
                    {...params}
                    name={`items[${index}].service`}
                    label="Produit"
                    InputLabelProps={{ shrink: true }}
                    inputProps={{
                      ...params.inputProps,
                      autoComplete: "new-password", // disable autocomplete and autofill
                    }}
                  />
                )}
                sx={{ width: "100%", maxWidth: { md: 400 } }}
              />
              <RHFTextField
                size="small"
                name={`items[${index}].description`}
                label="Description"
                InputLabelProps={{ shrink: true }}
              />
              <RHFTextField
                size="small"
                type="number"
                name={`items[${index}].quantity`}
                label="Quantité"
                placeholder="0"
                onChange={(event) => handleChangeQuantity(event, index)}
                InputLabelProps={{ shrink: true }}
                sx={{ maxWidth: { md: 96 } }}
              />

              <RHFTextField
                size="small"
                type="number"
                name={`items[${index}].price`}
                label="Prix"
                placeholder="0.00"
                onChange={(event) => handleChangePrice(event, index)}
                InputProps={{
                  startAdornment: (
                    <InputAdornment position="start">
                      <Box
                        sx={{ typography: "subtitle2", color: "text.disabled" }}
                      >
                        €
                      </Box>
                    </InputAdornment>
                  ),
                }}
                sx={{ maxWidth: { md: 96 } }}
              />

              <RHFTextField
                disabled
                size="small"
                type="number"
                name={`items[${index}].total`}
                label="Total"
                placeholder="0.00"
                value={
                  values.items[index].total === 0
                    ? ""
                    : values.items[index].total
                }
                onChange={(event) => handleChangePrice(event, index)}
                InputProps={{
                  startAdornment: (
                    <InputAdornment position="start">
                      <Box
                        sx={{ typography: "subtitle2", color: "text.disabled" }}
                      >
                        €
                      </Box>
                    </InputAdornment>
                  ),
                }}
                sx={{
                  maxWidth: { md: 104 },
                  [`& .${inputBaseClasses.input}`]: {
                    textAlign: { md: "right" },
                  },
                }}
              />
            </Stack>

            <Button
              size="small"
              color="error"
              startIcon={<Iconify icon="solar:trash-bin-trash-bold" />}
              onClick={() => handleRemove(index)}
            >
              Supprimer
            </Button>
          </Stack>
        ))}
      </Stack>

      <Divider sx={{ my: 3, borderStyle: "dashed" }} />

      <Stack
        spacing={3}
        direction={{ xs: "column", md: "row" }}
        alignItems={{ xs: "flex-end", md: "center" }}
      >
        <Button
          size="small"
          color="primary"
          startIcon={<Iconify icon="mingcute:add-line" />}
          onClick={handleAdd}
          sx={{ flexShrink: 0 }}
        >
          Ajouter un élément
        </Button>

        <Stack
          spacing={2}
          justifyContent="flex-end"
          direction={{ xs: "column", md: "row" }}
          sx={{ width: 1 }}
        >
          <RHFTextField
            size="small"
            label="Remise (€)"
            name="discount"
            type="number"
            sx={{ maxWidth: { md: 120 } }}
          />

          <RHFTextField
            size="small"
            label="Taxes (%)"
            name="taxes"
            type="number"
            sx={{ maxWidth: { md: 120 } }}
          />
        </Stack>
      </Stack>

      {renderTotal}
    </Box>
  );
}
