import * as Yup from "yup";
import { useMemo } from "react";
import { useForm } from "react-hook-form";
import { yupResolver } from "@hookform/resolvers/yup";

import Card from "@mui/material/Card";
import Stack from "@mui/material/Stack";
import LoadingButton from "@mui/lab/LoadingButton";

import { paths } from "@/routes/paths";
import { useRouter } from "@/hooks";

import { useBoolean } from "@/hooks/use-boolean";

import { _addressBooks } from "@/shared/_mock";

import FormProvider from "@/shared/components/hook-form";

import { IInvoice } from "@/shared/types/invoice";

import OrderNewEditDetails from "./order-new-edit-details";
import OrderNewEditAddress from "./Order-new-edit-address";
import OrdereNewEditStatusDate from "./order-new-edit-status-date";

// ----------------------------------------------------------------------

type Props = {
  currentInvoice?: IInvoice;
};

export default function OrdereNewEditForm({ currentInvoice }: Props) {
  const router = useRouter();

  const loadingSave = useBoolean();

  const loadingSend = useBoolean();

  const NewInvoiceSchema = Yup.object().shape({
    invoiceFrom: Yup.mixed<any>().nullable().required("Fournisseur requis"),
    invoiceTo: Yup.mixed<any>().nullable().required("Chauffeur requis"),

    createDate: Yup.mixed<any>()
      .nullable()
      .required("Date de création requise"),
    dueDate: Yup.mixed<any>()
      .required("Date d'échéance requise")
      .test(
        "date-min",
        "La date d'échéance doit être postérieure à la date de création",
        (value, { parent }) => value.getTime() > parent.createDate.getTime()
      ),
    items: Yup.lazy(() =>
      Yup.array().of(
        Yup.object({
          service: Yup.string().required("Produit requis"),
          quantity: Yup.number()
            .required("Quantité requise")
            .min(1, "La quantité doit être supérieure à 0"),
        })
      )
    ),
    // non requis
    taxes: Yup.number(),
    status: Yup.string(),
    discount: Yup.number(),
    totalAmount: Yup.number(),
  });
  const defaultValues = useMemo(
    () => ({
      createDate: currentInvoice?.createDate || new Date(),
      dueDate: currentInvoice?.dueDate || null,
      taxes: currentInvoice?.taxes || 0,
      status: currentInvoice?.status || "Attendue",
      discount: currentInvoice?.discount || 0,
      invoiceFrom: currentInvoice?.invoiceFrom || _addressBooks[0],
      invoiceTo: currentInvoice?.invoiceTo || null,
      items: currentInvoice?.items || [
        {
          description: "",
          service: "",
          quantity: 1,
          price: 0,
          total: 0,
        },
      ],
      totalAmount: currentInvoice?.totalAmount || 0,
    }),
    [currentInvoice]
  );

  const methods = useForm({
    resolver: yupResolver(NewInvoiceSchema),
    defaultValues,
  });

  const {
    reset,

    handleSubmit,
    formState: { isSubmitting },
  } = methods;

  const handleSaveAsDraft = handleSubmit(async (data) => {
    loadingSave.onTrue();

    try {
      await new Promise((resolve) => setTimeout(resolve, 500));
      reset();
      loadingSave.onFalse();
      router.push(paths.dashboard.supplier.list);
      console.info("DATA", JSON.stringify(data, null, 2));
    } catch (error) {
      console.error(error);
      loadingSave.onFalse();
    }
  });

  const handleCreateAndSend = handleSubmit(async (data) => {
    loadingSend.onTrue();

    try {
      await new Promise((resolve) => setTimeout(resolve, 500));
      reset();
      loadingSend.onFalse();
      router.push(paths.dashboard.supplier.list);
      console.info("DATA", JSON.stringify(data, null, 2));
    } catch (error) {
      console.error(error);
      loadingSend.onFalse();
    }
  });

  return (
    <FormProvider methods={methods}>
      <Card>
        <OrderNewEditAddress />

        <OrdereNewEditStatusDate />

        <OrderNewEditDetails />
      </Card>

      <Stack
        justifyContent="flex-end"
        direction="row"
        spacing={2}
        sx={{ mt: 3 }}
      >
        <LoadingButton
          color="inherit"
          size="large"
          variant="outlined"
          loading={loadingSave.value && isSubmitting}
          onClick={handleSaveAsDraft}
        >
          Save as Draft
        </LoadingButton>

        <LoadingButton
          size="large"
          variant="contained"
          loading={loadingSend.value && isSubmitting}
          onClick={handleCreateAndSend}
        >
          {currentInvoice ? "Update" : "Create"} & Send
        </LoadingButton>
      </Stack>
    </FormProvider>
  );
}
