"use client";

import { useState } from "react";
import Box from "@mui/material/Box";
import * as Yup from "yup";
import { useForm } from "react-hook-form";
import { yupResolver } from "@hookform/resolvers/yup";
import { useMemo } from "react";

import Grid from "@mui/material/Unstable_Grid2";
import Card from "@mui/material/Card";
import Stack from "@mui/material/Stack";
import Button from "@mui/material/Button";
import CardHeader from "@mui/material/CardHeader";
import Typography from "@mui/material/Typography";
import Divider from "@mui/material/Divider";
import KeyboardBackspaceIcon from "@mui/icons-material/KeyboardBackspace";

import Autocomplete from "@mui/material/Autocomplete";
import TextField from "@mui/material/TextField";
import { useResponsive } from "@/hooks/use-responsive";
import { useSnackbar } from "@/components/snackbar";
import FormProvider, { RHFTextField } from "@/components/hook-form";
import { paths } from "@/routes/paths";
import RouterLink from "@/routes/router-link";
import LoadingButton from "@mui/lab/LoadingButton";
import { tagsDeviation } from "@/contexts/types/deviation";
import { _orders } from "@/shared/_mock";
import { tagsChoice } from "@/contexts/types/deviation";

export default function CreateShippingMethodView() {
  const TagsDeviation = Object.values(tagsDeviation);

  const mdUp = useResponsive("up", "md");
  const { enqueueSnackbar } = useSnackbar();

  const DeviationSchema = Yup.object().shape({
    description: Yup.string().required("Description is required"),
    active: Yup.boolean(),
    tagsChoice: Yup.array(),
  });

  const defaultValues = useMemo(
    () => ({
      description: "",
      active: false,
      tagsChoice: [],
    }),
    []
  );

  const methods = useForm({
    resolver: yupResolver(DeviationSchema),
    defaultValues,
  });

  const {
    watch,
    control,
    setValue,
    handleSubmit,
    formState: { isSubmitting },
  } = methods;

  const values = watch();

  const onSubmit = handleSubmit(async (data) => {
    try {
      await new Promise((resolve) => setTimeout(resolve, 500));
      enqueueSnackbar("Create success!", { variant: "success" });
      console.info("Form Data:", data);
    } catch (error) {
      console.error(error);
      enqueueSnackbar("Failed to create this new shipping area.", {
        variant: "error",
      });
    }
  });

  const [selectedType, setSelectedType] = useState<string | null>(null);
  const renderDetails = () => {
    switch (selectedType) {
      case tagsDeviation.MoreTags:
        return (
          <Grid container spacing={3}>
            <Grid xs={12} md={6}>
              <RHFTextField
                name="deviation"
                label="Déclarer l'écart"
                type="number"
                InputLabelProps={{ shrink: true }}
                placeholder="0"
                fullWidth
              />
            </Grid>
            <Grid
              xs={12}
              md={6}
              sx={{ display: "flex", alignItems: "flex-end" }}
            >
              <Button
                variant="contained"
                color="primary"
                sx={{ height: "100%" }}
                component={RouterLink}
                href={paths.dashboard.supplier.SupplierOrder.new}
              >
                Créer une commande
              </Button>
            </Grid>
          </Grid>
        );
      case tagsDeviation.lessTags:
        return (
          <Grid container spacing={3}>
            <Grid xs={12} md={8}>
              <Typography variant="body1">
                Les produits sont plus que les étiquettes
              </Typography>
            </Grid>
            <Grid xs={12} md={8}>
              <RHFTextField
                name="deviation"
                label="Déclarer l'écart"
                type="number"
                InputLabelProps={{ shrink: true }}
                placeholder="0"
                fullWidth
              />
            </Grid>
          </Grid>
        );
      case tagsDeviation.tagsCorrect:
        return (
          <Grid container spacing={3}>
            <Grid xs={12} md={8}>
              <Typography variant="body1">
                Produits et quantité exacts
              </Typography>
            </Grid>
            <Grid xs={12} md={8}>
              <RHFTextField
                name="deviation"
                label="l'écart est Null"
                type="number"
                InputLabelProps={{ shrink: true }}
                placeholder="0"
                fullWidth
                disabled
              />
            </Grid>
          </Grid>
        );
      default:
        return null;
    }
  };

  const renderBasic = (
    <>
      {mdUp && (
        <Grid md={4}>
          <Button
            variant="text"
            color="success"
            component={RouterLink}
            href={paths.dashboard.supplier.SupplierOrder.root}
            sx={{ marginBottom: 2 }}
          >
            <KeyboardBackspaceIcon />
            Liste des Commandes
          </Button>
          <Typography variant="h6" sx={{ mb: 0.5 }}>
            Déclarer les Écarts
          </Typography>
          <Typography variant="body2" sx={{ color: "text.secondary" }}>
            Vérifiez les produits reçus en déclarant l'écart calculé à partir
            des étiquettes
          </Typography>
        </Grid>
      )}

      <Grid xs={12} md={8}>
        <Card>
          <Stack spacing={3} sx={{ p: 3 }}>
            <Autocomplete
              id="Commande"
              autoHighlight
              disablePortal
              options={_orders}
              getOptionLabel={(option) => option.orderNumber}
              renderOption={(props, option) => (
                <Box component="li" {...props}>
                  {option.orderNumber} + {option.customer.name}
                </Box>
              )}
              renderInput={(params) => (
                <TextField
                  {...params}
                  label="Commande"
                  placeholder="Choisir une Commande"
                  InputLabelProps={{ shrink: true }}
                />
              )}
            />

            <RHFTextField
              name="description"
              label="Description"
              placeholder="Description..."
              multiline
              rows={4}
            />
            <Autocomplete
              id="type"
              autoHighlight
              disablePortal
              options={TagsDeviation}
              getOptionLabel={(option: string) => option}
              value={selectedType}
              onChange={(event, newValue) => {
                setSelectedType(newValue);
              }}
              renderInput={(params) => (
                <TextField
                  {...params}
                  label="Cas possible"
                  placeholder="Choisir les cas possibles"
                  InputLabelProps={{ shrink: true }}
                />
              )}
            />

            {renderDetails()}
          </Stack>
        </Card>
      </Grid>
    </>
  );

  const renderActions = (
    <>
      {mdUp && <Grid md={4} />}
      <Grid
        xs={12}
        md={8}
        sx={{ display: "flex", justifyContent: "end", py: 4 }}
      >
        <LoadingButton
          type="submit"
          variant="contained"
          size="large"
          loading={isSubmitting}
          sx={{ ml: 2 }}
        >
          Enregistrer
        </LoadingButton>
      </Grid>
    </>
  );

  return (
    <FormProvider methods={methods} onSubmit={onSubmit}>
      <Grid container spacing={3}>
        {renderBasic}

        {renderActions}
      </Grid>
    </FormProvider>
  );
}
