import * as Yup from "yup";
import { useMemo, useCallback, useState } from "react";
import { useForm, Controller } from "react-hook-form";
import { yupResolver } from "@hookform/resolvers/yup";

import Box from "@mui/material/Box";
import Card from "@mui/material/Card";
import Stack from "@mui/material/Stack";
import Button from "@mui/material/Button";
import Switch from "@mui/material/Switch";
import Grid from "@mui/material/Unstable_Grid2";
import Typography from "@mui/material/Typography";
import LoadingButton from "@mui/lab/LoadingButton";
import FormControlLabel from "@mui/material/FormControlLabel";

import { paths } from "@/routes/paths";
import { useRouter } from "../../../../hooks/use-router";

import { fData } from "@/utils/format-number";

import { countries } from "../../../assets/data";

import Label from "../../../components/label";
import { useSnackbar } from "@/components/snackbar";
import FormProvider, {
  RHFTextField,
  RHFUploadAvatar,
  RHFAutocomplete,
} from "../../../components/hook-form";

import { IUserItem } from "../../../types/user";
import { RHFSelect } from "@/components/hook-form";
import { IconButton, InputAdornment, MenuItem, TextField } from "@mui/material";
import { Visibility, VisibilityOff } from "@mui/icons-material";
import RouterLink from "@/routes/router-link";
// ----------------------------------------------------------------------

type Props = {
  currentUser?: IUserItem;
  roleOptions: string[];
};

const generateStrongPassword = () => {
  const length = 12;
  const charset =
    "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789!@#$%^&*()_-+=";
  let password = "";
  for (let i = 0; i < length; ++i) {
    const randomIndex = Math.floor(Math.random() * charset.length);
    password += charset[randomIndex];
  }
  return password;
};
export default function UserNewEditForm({ currentUser, roleOptions }: Props) {
  const router = useRouter();

  const { enqueueSnackbar } = useSnackbar();
  const [newPassword, setNewPassword] = useState("");
  const [resetPassword, setResetPassword] = useState("");
  const [showNewPasswordField, setShowNewPasswordField] = useState(false);
  const [showPassword, setShowPassword] = useState(false);

  const handleTogglePasswordVisibility = () => {
    setShowPassword((prevShowPassword) => !prevShowPassword);
  };

  const handleNewPasswordClick = () => {
    const generatedPassword = generateStrongPassword();
    setNewPassword(generatedPassword);
    setShowPassword(true);
    setShowNewPasswordField(true);
  };

  const handleNewPasswordSubmit = () => {
    setNewPassword("");
    setShowNewPasswordField(false);
  };

  const handleResetPasswordReset = () => {
    setResetPassword("");
    alert(
      `Un lien de réinitialisation du mot de passe a été envoyé par e-mail à ${currentUser?.name}.`
    );
  };

  const NewUserSchema = Yup.object().shape({
    name: Yup.string().required("Name is required"),
    email: Yup.string()
      .required("Email is required")
      .email("Email must be a valid email address"),
    phoneNumber: Yup.string().required("Phone number is required"),
    address: Yup.string().required("Address is required"),
    country: Yup.string().required("Country is required"),
    company: Yup.string().required("Company is required"),
    state: Yup.string().required("State is required"),
    city: Yup.string().required("City is required"),
    role: Yup.string().required("Role is required"),
    pseudo: Yup.string().required("Pseudo is required"),
    zipCode: Yup.string().required("Zip code is required"),
    avatarUrl: Yup.mixed<any>().nullable().required("Avatar is required"),
    // not required
    status: Yup.string(),
    isVerified: Yup.boolean(),
  });

  const defaultValues = useMemo(
    () => ({
      name: currentUser?.name || "",
      city: currentUser?.city || "",
      role: currentUser?.role || "",
      email: currentUser?.email || "",
      state: currentUser?.state || "",
      status: currentUser?.status || "",
      address: currentUser?.address || "",
      pseudo: currentUser?.pseudo || "",
      country: currentUser?.country || "",
      zipCode: currentUser?.zipCode || "",
      company: currentUser?.company || "",
      avatarUrl: currentUser?.avatarUrl || null,
      phoneNumber: currentUser?.phoneNumber || "",
      isVerified: currentUser?.isVerified || true,
    }),
    [currentUser]
  );

  const methods = useForm({
    resolver: yupResolver(NewUserSchema),
    defaultValues,
  });

  const {
    reset,
    watch,
    control,
    setValue,
    handleSubmit,
    formState: { isSubmitting },
  } = methods;

  const values = watch();

  const onSubmit = handleSubmit(async (data) => {
    try {
      await new Promise((resolve) => setTimeout(resolve, 500));
      reset();
      enqueueSnackbar(currentUser ? "Update success!" : "Create success!");
      router.push(paths.dashboard.user.all_users);
      console.info("DATA", data);
    } catch (error) {
      console.error(error);
    }
  });

  const handleDrop = useCallback(
    (acceptedFiles: File[]) => {
      const file = acceptedFiles[0];

      const newFile = Object.assign(file, {
        preview: URL.createObjectURL(file),
      });

      if (file) {
        setValue("avatarUrl", newFile, { shouldValidate: true });
      }
    },
    [setValue]
  );

  return (
    <FormProvider methods={methods} onSubmit={onSubmit}>
      <Grid container spacing={3}>
        <Grid xs={12} md={4}>
          <Card sx={{ pt: 10, pb: 5, px: 3 }}>
            {currentUser && (
              <Label
                color={
                  (values.status === "approved" && "success") ||
                  (values.status === "banned" && "error") ||
                  "warning"
                }
                sx={{ position: "absolute", top: 24, right: 24 }}
              >
                {values.status}
              </Label>
            )}

            <Box sx={{ mb: 5 }}>
              <RHFUploadAvatar
                name="avatarUrl"
                maxSize={3145728}
                onDrop={handleDrop}
                helperText={
                  <Typography
                    variant="caption"
                    sx={{
                      mt: 3,
                      mx: "auto",
                      display: "block",
                      textAlign: "center",
                      color: "text.disabled",
                    }}
                  >
                    Allowed *.jpeg, *.jpg, *.png, *.gif
                    <br /> max size of {fData(3145728)}
                  </Typography>
                }
              />
            </Box>

            {currentUser && (
              <FormControlLabel
                labelPlacement="start"
                control={
                  <Controller
                    name="status"
                    control={control}
                    render={({ field }) => (
                      <Switch
                        {...field}
                        checked={field.value === "banned"}
                        onChange={(event) =>
                          field.onChange(
                            event.target.checked ? "banned" : "approved"
                          )
                        }
                      />
                    )}
                  />
                }
                label={
                  <>
                    <Typography variant="subtitle2" sx={{ mb: 0.5 }}>
                      Banned
                    </Typography>
                    <Typography
                      variant="body2"
                      sx={{ color: "text.secondary" }}
                    >
                      Apply disable account
                    </Typography>
                  </>
                }
                sx={{ mx: 0, mb: 3, width: 1, justifyContent: "space-between" }}
              />
            )}
          </Card>
        </Grid>

        <Grid xs={12} md={8}>
          <Card sx={{ p: 3 }}>
            <Box
              rowGap={3}
              columnGap={2}
              display="grid"
              gridTemplateColumns={{
                xs: "repeat(1, 1fr)",
                sm: "repeat(2, 1fr)",
              }}
            >
              <RHFTextField name="name" label="Full Name" />
              <RHFTextField name="pseudo" label="Pseudo" />
              <RHFTextField name="email" label="Email Address" />
              <RHFSelect name="role" label="Role">
                {roleOptions &&
                  roleOptions.map((option) => (
                    <MenuItem key={option} value={option}>
                      {option}
                    </MenuItem>
                  ))}
              </RHFSelect>
              <Box sx={{ mb: 3 }}>
                <Typography variant="subtitle1">
                  Address de Facturation
                </Typography>
                <Box sx={{ mt: 2 }}>
                  <RHFAutocomplete
                    name="country"
                    type="country"
                    label="Country"
                    placeholder="Choose a country"
                    fullWidth
                    options={countries.map((option) => option.label)}
                    getOptionLabel={(option) => option}
                  />
                </Box>
                <Box sx={{ mt: 2 }}>
                  <RHFTextField name="city" label="City" />
                </Box>
                <Box sx={{ mt: 2 }}>
                  <RHFTextField name="state" label="State/Region" />
                </Box>
                <Box sx={{ mt: 2 }}>
                  <RHFTextField name="zipCode" label="Zip/Code" />
                </Box>
                <Box sx={{ mt: 2 }}>
                  <RHFTextField name="address" label="Address" />
                </Box>
                <Box sx={{ mt: 2 }}>
                  <RHFTextField name="phoneNumber" label="Phone Number" />
                </Box>
              </Box>
              <Box display="grid" sx={{ mb: 3 }}>
                <Typography variant="subtitle1">Delivery Address</Typography>
                <Box sx={{ mt: 2 }}>
                  <RHFAutocomplete
                    name="country"
                    type="country"
                    label="Country"
                    placeholder="Choose a country"
                    fullWidth
                    options={countries.map((option) => option.label)}
                    getOptionLabel={(option) => option}
                  />
                </Box>
                <Box sx={{ mt: 2 }}>
                  <RHFTextField name="city" label="City" />
                </Box>
                <Box sx={{ mt: 2 }}>
                  <RHFTextField name="state" label="State/Region" />
                </Box>
                <Box sx={{ mt: 2 }}>
                  <RHFTextField name="zipCode" label="Zip/Code" />
                </Box>
                <Box sx={{ mt: 2 }}>
                  <RHFTextField name="address" label="Address" />
                </Box>
                <Box sx={{ mt: 2 }}>
                  <RHFTextField name="phoneNumber" label="Phone Number" />
                </Box>
              </Box>
              <Box sx={{ mb: 3 }}>
                <Typography variant="subtitle1">Gestion du Compte</Typography>
                <Box sx={{ display: "flex", alignItems: "center", mt: 2 }}>
                  <Typography variant="subtitle2" sx={{ mr: 1 }}>
                    New Password
                  </Typography>
                  <Button variant="contained" onClick={handleNewPasswordClick}>
                    Set new password
                  </Button>
                </Box>
                {showNewPasswordField && (
                  <Box sx={{ mt: 2, display: "flex", alignItems: "center" }}>
                    <TextField
                      type={showPassword ? "text" : "password"}
                      value={newPassword}
                      onChange={(e) => setNewPassword(e.target.value)}
                      InputProps={{
                        endAdornment: (
                          <InputAdornment position="end">
                            <IconButton
                              onClick={handleTogglePasswordVisibility}
                            >
                              {showPassword ? (
                                <VisibilityOff />
                              ) : (
                                <Visibility />
                              )}
                            </IconButton>
                          </InputAdornment>
                        ),
                      }}
                      sx={{ mr: 1 }}
                    />
                    <Box sx={{ marginRight: 1 }} />
                    <Button
                      variant="contained"
                      onClick={handleNewPasswordSubmit}
                    >
                      Annuler
                    </Button>
                  </Box>
                )}
                <Box sx={{ display: "flex", alignItems: "center", mt: 2 }}>
                  <Typography variant="subtitle2" sx={{ mr: 1 }}>
                    Reset password
                  </Typography>
                  <Button
                    variant="contained"
                    onClick={handleResetPasswordReset}
                  >
                    Send reset link
                  </Button>
                </Box>
                <Typography variant="body2" sx={{ color: "text.secondary" }}>
                  Envoyez à {currentUser?.name} un lien pour réinitialiser son
                  mot de passe. Cela ne changera pas son mot de passe, ni ne
                  l’obligera à le modifier.
                </Typography>
              </Box>
              <Box sx={{ mb: 3 }}>
                <Typography variant="subtitle1">
                  Permissions supplémentaires
                </Typography>

                <Box sx={{ display: "flex", alignItems: "center", mt: 2 }}>
                  <Typography variant="subtitle2" sx={{ mr: 1 }}>
                    Permission
                  </Typography>
                  <Button
                    component={RouterLink}
                    href={paths.dashboard.user.role}
                    variant="contained"
                  >
                    Modify
                  </Button>
                </Box>
              </Box>
            </Box>

            <Stack alignItems="flex-end" sx={{ mt: 3 }}>
              <LoadingButton
                type="submit"
                variant="contained"
                loading={isSubmitting}
              >
                {!currentUser ? "Create User" : "Save Changes"}
              </LoadingButton>
            </Stack>
          </Card>
        </Grid>
      </Grid>
    </FormProvider>
  );
}
